Package: cjoint
Version: 2.1.3
Date: 2026-05-18
Title: AMCE Estimator for Conjoint Experiments
Authors@R: c(
    person("Soubhik", "Barari", role = "aut"),
    person("Elissa", "Berwick", role = "aut"),
    person("Jens", "Hainmueller", role = "aut"),
    person("Daniel", "Hopkins", role = "aut"),
    person("Sean", "Liu", role = "aut"),
    person("Anton", "Strezhnev", role = c("aut", "cre"),
           email = "strezhnev@wisc.edu"),
    person("Teppei", "Yamamoto", role = "aut"))
Depends: R (>= 2.10), sandwich, lmtest, ggplot2, survey, Matrix
Imports: stats, utils, methods, grDevices, DT, shiny, shinyjs
Suggests: testthat
Description: An R implementation of the Average Marginal Component-specific
    Effects (AMCE) estimator presented in Hainmueller, J., Hopkins, D., and Yamamoto
    T. (2014) <DOI:10.1093/pan/mpt024> Causal Inference in Conjoint Analysis: Understanding Multi-Dimensional
    Choices via Stated Preference Experiments. Political Analysis 22(1):1-30.
License: GPL (>= 2)
RoxygenNote: 6.1.0
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2026-05-18 19:10:22 UTC; anton
Author: Soubhik Barari [aut],
  Elissa Berwick [aut],
  Jens Hainmueller [aut],
  Daniel Hopkins [aut],
  Sean Liu [aut],
  Anton Strezhnev [aut, cre],
  Teppei Yamamoto [aut]
Maintainer: Anton Strezhnev <strezhnev@wisc.edu>
Repository: CRAN
Date/Publication: 2026-05-19 05:30:02 UTC
Built: R 4.5.2; ; 2026-05-19 06:35:45 UTC; unix

Package: cjoint
Version: 2.1.1
Date: 2023-08-21
Title: AMCE Estimator for Conjoint Experiments
Author: Soubhik Barari, Elissa Berwick, Jens Hainmueller, Daniel Hopkins, Sean Liu, Anton Strezhnev, Teppei Yamamoto
Maintainer: Anton Strezhnev <astrezhnev@uchicago.edu>
Depends: R (>= 2.10), sandwich, lmtest, ggplot2, survey, Matrix
Imports: stats, utils, methods, DT, shiny, shinyjs
Suggests: testthat
Description: An R implementation of the Average Marginal Component-specific
    Effects (AMCE) estimator presented in Hainmueller, J., Hopkins, D., and Yamamoto
    T. (2014) <DOI:10.1093/pan/mpt024> Causal Inference in Conjoint Analysis: Understanding Multi-Dimensional
    Choices via Stated Preference Experiments. Political Analysis 22(1):1-30.
License: GPL (>= 2)
RoxygenNote: 6.1.0
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2023-08-21 20:00:11 UTC; anton
Repository: CRAN
Date/Publication: 2023-08-22 07:50:07 UTC
Built: R 4.2.0; ; 2023-08-27 03:40:38 UTC; unix

Package: cjoint
Version: 2.1.3
Date: 2026-05-18
Title: AMCE Estimator for Conjoint Experiments
Authors@R: c(
    person("Soubhik", "Barari", role = "aut"),
    person("Elissa", "Berwick", role = "aut"),
    person("Jens", "Hainmueller", role = "aut"),
    person("Daniel", "Hopkins", role = "aut"),
    person("Sean", "Liu", role = "aut"),
    person("Anton", "Strezhnev", role = c("aut", "cre"),
           email = "strezhnev@wisc.edu"),
    person("Teppei", "Yamamoto", role = "aut"))
Depends: R (>= 2.10), sandwich, lmtest, ggplot2, survey, Matrix
Imports: stats, utils, methods, grDevices, DT, shiny, shinyjs
Suggests: testthat
Description: An R implementation of the Average Marginal Component-specific
    Effects (AMCE) estimator presented in Hainmueller, J., Hopkins, D., and Yamamoto
    T. (2014) <DOI:10.1093/pan/mpt024> Causal Inference in Conjoint Analysis: Understanding Multi-Dimensional
    Choices via Stated Preference Experiments. Political Analysis 22(1):1-30.
License: GPL (>= 2)
RoxygenNote: 6.1.0
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2026-05-18 19:10:22 UTC; anton
Author: Soubhik Barari [aut],
  Elissa Berwick [aut],
  Jens Hainmueller [aut],
  Daniel Hopkins [aut],
  Sean Liu [aut],
  Anton Strezhnev [aut, cre],
  Teppei Yamamoto [aut]
Maintainer: Anton Strezhnev <strezhnev@wisc.edu>
Repository: CRAN
Date/Publication: 2026-05-19 05:30:02 UTC
Built: R 4.5.2; ; 2026-05-19 10:34:10 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,amce)
S3method(print,summary.amce)
S3method(summary,amce)
export(amce)
export(makeDesign)
export(plot.amce)
export(print.summary.amce)
export(read.qualtrics)
export(read.with.qualtRics)
export(summary.amce)
export(view)
import(Matrix)
import(ggplot2)
import(lmtest)
import(sandwich)
import(shiny)
import(survey)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(methods,Quote)
importFrom(stats,as.formula)
importFrom(stats,coefficients)
importFrom(stats,formula)
importFrom(stats,lm)
importFrom(stats,model.matrix)
importFrom(stats,pnorm)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,relevel)
importFrom(stats,reshape)
importFrom(stats,terms)
importFrom(stats,var)
importFrom(stats,vcov)
importFrom(utils,View)
importFrom(utils,data)
importFrom(utils,packageDescription)
importFrom(utils,read.csv)

# cjoint: An R Package for Estimating Average Marginal Component-specific Effects for Conjoint Survey Experiments

## Description
This package allows researchers to estimate the causal effects of attributes in conjoint survey experiments. It implements the Average Marginal Component-specific Effects (AMCE) estimator presented in Hainmueller, J., Hopkins, D., and Yamamoto T. (2014) Causal Inference in Conjoint Analysis: Understanding Multi-Dimensional Choices via Stated Preference Experiments. Political Analysis 22(1):1-30

## Details
  Package: `cjoint`
  Type: `R Package`
  Version: 2.1.3
  Date: 2026-05-18
  License: GPL (>= 2)

## Authors: 
Soubhik Barari, Elissa Berwick, Jens Hainmueller, Daniel Hopkins, Sean Liu, Anton Strezhnev, Teppei Yamamoto

## Maintainer: 
Anton Strezhnev <strezhnev@wisc.edu>

## References:
Hainmueller, J., Hopkins, D., and Yamamoto T. (2014) Causal Inference in Conjoint Analysis: Understanding Multi-Dimensional Choices via Stated Preference Experiments. Political Analysis 22(1):1-30

## Installation Guide:

To install the `Github` version of the `cjoint` package, use the following command:

`remotes::install_github("astrezhnev/cjoint")`

################ Demo for cjoint package - Strezhnev, Berwick, Hainmueller, Hopkins, Yamamoto ####################

# Load data from immigration conjoint data (from the replication data to "Causal Inference in Conjoint Analysis")
data("immigrationconjoint")
data("immigrationdesign")

##  You can construct the conjoint design manually in in R
attribute_list <- list()
attribute_list[["Education"]] <-c("no formal","4th grade",
                                  "8th grade","high school",
                                  "two-year college","college degree",
                                  "graduate degree")
attribute_list[["Gender"]] <- c("female","male")
attribute_list[["Country of Origin"]] <-  c("Germany","France","Mexico",
                                            "Philippines","Poland","India",
                                            "China","Sudan","Somalia","Iraq")
attribute_list[["Reason for Application"]] <- c("reunite with family",
                                                "seek better job",
                                                "escape persecution")
attribute_list[["Job"]] <- c("janitor","waiter","child care provider",
                             "gardener","financial analyst",
                             "construction worker","teacher",
                             "computer programmer","nurse",
                             "research scientist","doctor")
attribute_list[["Job Experience"]] <- c("none","1-2 years",
                                        "3-5 years","5+ years")
attribute_list[["Job Plans"]] <- c("contract with employer",
                                   "interviews with employer", "will look for work",
                                   "no plans to look for work")
attribute_list[["Prior Entry"]] <- c("never","once as tourist",
                                     "many times as tourist","six months with family",
                                     "once w/o authorization")
attribute_list[["Language Skills"]] <- c("fluent English",
                                         "broken English",
                                         "tried English but unable",
                                         "used interpreter")

# Randomization constraints in the conjoint design
constraint_list <- list()
constraint_list[[1]] <- list()
## constraint_list[[1]][["Education"]] <- c("no formal","4th grade",
##                                   "8th grade","high school",
##                                          "two-year college","college degree")
constraint_list[[1]][["Education"]] <- c("no formal")
constraint_list[[1]][["Job"]] <- c("waiter","child care provider",
                             "gardener","financial analyst",
                             "construction worker","teacher",
                             "computer programmer","nurse",
                             "research scientist","doctor")
constraint_list[[2]] <- list()
constraint_list[[2]][["Reason for Application"]] <- c("escape persecution")
constraint_list[[2]][["Country of Origin"]] <- c("Germany","France","Mexico",
                                            "Philippines","Poland","India",
                                            "China","Sudan","Somalia")

immigrationdesign2 <- makeDesign(type='constraints', attribute.levels=attribute_list, constraints=constraint_list)

# Run AMCE estimator - this calls the main amce() function in the cjoint package
results <- amce(Chosen_Immigrant ~  Gender + Education + `Language Skills` + `Country of Origin` + Job + `Job Experience` + `Job Plans` + `Reason for Application` + `Prior Entry`, data=immigrationconjoint, cluster=TRUE, respondent.id="CaseID", design=immigrationdesign2)

# Print summary
summary(results)
# Plot results
plot(results, xlab="Change in Pr(Immigrant Preferred for Admission to U.S.)", xlim=c(-.3,.3), breaks=c(-.2, 0, .2), labels=c("-.2","0",".2"), text.size=13)

# You can specify interactions in the formula using : or *
interaction_results <- amce(Chosen_Immigrant ~ Gender + Education + Job + Education*Job, data=immigrationconjoint, cluster=TRUE, respondent.id="CaseID", design=immigrationdesign) 
# Print summary
summary(interaction_results)
# Plot results
plot(interaction_results, xlab="Change in Pr(Immigrant Preferred for Admission to U.S.)", xlim=c(-.3,.3), breaks=c(-.2, 0, .2), labels=c("-.2","0",".2"), text.size=13)

# Alternative specification using ":"
interaction_results <- amce(Chosen_Immigrant ~ Gender + Education + Job + Education:Job, data=immigrationconjoint, cluster=TRUE, respondent.id="CaseID", design=immigrationdesign)
summary(interaction_results)

### Specify respondent-varying interactions the same way, but make sure to add the variable to the respondent.varying argument # Add respondent-varying interaction
# Subset out NA values prior to running - otherwise it will throw an error
interaction_results <- amce(Chosen_Immigrant ~ Education + Job + Education*ethnocentrism, data=subset(immigrationconjoint, !is.na(immigrationconjoint$ethnocentrism)), cluster=TRUE, respondent.id="CaseID",design=immigrationdesign, respondent.varying=c("ethnocentrism"))

# Summarize results, at quantiles of respondent-varying attribute
summary(interaction_results)

# Plot results by continuous variable, showing non-interacted or not
plot(interaction_results, facet.name=c("ethnocentrism"))
plot(interaction_results, facet.name=c("ethnocentrism"), show.all=T)

# You can manually set the baselines used in the function (in addition to just releveling the factors in your dataset) 
baselines <- list()
baselines$Education <- "graduate degree"

# Run function
interaction_results <- amce(Chosen_Immigrant ~ Gender + Education + Job + Education:Job, data=immigrationconjoint, cluster=TRUE, respondent.id="CaseID",design=immigrationdesign,baselines=baselines)

# Summarize results
summary(interaction_results)

# Plot results by factor variable
plot.amce(interaction_results,facet.name =c("Education"))

# Only plot for some levels of factor variable
facet.levels1 <- list()
facet.levels1[["Education"]] <-(c("college degree","two-year college"))
names(facet.levels1[["Education"]]) <- c("college degree","two-year college")
plot.amce(interaction_results,facet.name =c("Education"), facet.levels = facet.levels1)

#' @import shiny
#' @export view

## view(amce): R Shiny exploratory visualization tool that ships with `amce`; useed 
## to quickly perform `amce` on a pre-loaded dataset/design.
## June, 2018
## Soubhik Barari

PKG.ENV <- new.env(parent = emptyenv())

# ==================================================================================
# ==================================== USER API ==================================== 
# ==================================================================================

view <- function(object) {
    ## Top-level function for the user to call in order to view 
    ## the `amce` Shiny dashboard
    
    data("japan2014conjoint", envir = environment())
    data("immigrationconjoint", envir = environment())
    data("immigrationdesign", envir = environment())

    if (typeof(object) == "closure" & deparse(substitute(object)) == "amce") {
        
        if (!requireNamespace("shiny", quietly=TRUE)) {
            stop("Package \"shiny\" needed for `view(amce)` to work. Please install it")
        }

        if (!requireNamespace("shinyjs", quietly=TRUE)) {
            stop("Package \"shinyjs\" needed for `view(amce)` to work. Please install it")
        }

        if (!requireNamespace("DT", quietly=TRUE)) {
            stop("Package \"DT\" needed for `view(amce)` to work. Please install it")
        }
        
        #--------------------------------------------
        # Set DESIGN/DATA globals
        #--------------------------------------------
        
        ## Global collection of design + data objects
        ## (update below)
        PKG.ENV$DATA_OPTIONS    <- NULL
        PKG.ENV$DESIGN_OPTIONS  <- NULL
        
        ## Default design + data to load on start up
        PKG.ENV$DEFAULT_DATA_NAME   <- "immigrationconjoint"
        PKG.ENV$DEFAULT_DESIGN_NAME <- "immigrationdesign"
        
        ## Names of current design + data
        PKG.ENV$SELECTED_DATA_NAME   <- PKG.ENV$DEFAULT_DATA_NAME
        PKG.ENV$SELECTED_DESIGN_NAME <- PKG.ENV$DEFAULT_DESIGN_NAME
        
        ## Objects for current design + data
        ## (update below)
        PKG.ENV$SELECTED_DATA   <- NULL
        PKG.ENV$SELECTED_DESIGN <- NULL
        
        ## Last set of results
        PKG.ENV$LAST_RESULTS    <- NULL
        
        #--------------------------------------------
        # Set UI globals
        #--------------------------------------------
        
        PKG.ENV$TITLE_TEXT        <- "cjoint"
        PKG.ENV$WINDOW_TITLE_TEXT <- "cjoint AMCE dashboard"
        
        ## Name of UI elements shown/hidden
        ## when AMCE is run
        PKG.ENV$RESULTS_ID               <- "analysisResultsPanel"
        PKG.ENV$RESULTS_SUMMARY_ID       <- "resultsSummary"
        PKG.ENV$RESULTS_PLOT_ID          <- "resultsPlot"
        PKG.ENV$RESULTS_UNCOND_PLOT_ID   <- "resultsUncondPlot"
        PKG.ENV$RESULTS_INTERACT_PLOT_ID <- "resultsInteractPlot"
        PKG.ENV$DEFAULT_RESULTS_HEIGHT   <- 1000
        
        ## Temporary "baselines" UI elements
        ## that appear/disappear based on user
        ## selection
        PKG.ENV$RESP_BASELINE_ID   <- c()
        PKG.ENV$ATTR_BASELINE_IDS  <- c()
        
        ## Plot UI elements
        PKG.ENV$PLOT_SETTINGS_ID        <- "plotSettings"
        
        PKG.ENV$PLOT_TEXT_SIZE_DEFAULT_MAX  <- 25
        PKG.ENV$PLOT_TEXT_SIZE_DEFAULT_MIN  <- 5
        PKG.ENV$PLOT_TEXT_SIZE_DEFAULT_VAL  <- 11
        PKG.ENV$PLOT_TEXT_SIZE_SELECTED            <- PKG.ENV$PLOT_TEXT_SIZE_DEFAULT_VAL
        PKG.ENV$PLOT_UNCOND_TEXT_SIZE_SELECTED     <- PKG.ENV$PLOT_TEXT_SIZE_DEFAULT_VAL
        PKG.ENV$PLOT_INTERACT_TEXT_SIZE_SELECTED   <- PKG.ENV$PLOT_TEXT_SIZE_DEFAULT_VAL
        
        PKG.ENV$PLOT_DEFAULT_CI  <- 0.95
        PKG.ENV$PLOT_CI          <- 0.95
        PKG.ENV$PLOT_UNCOND_CI   <- 0.95
        PKG.ENV$PLOT_INTERACT_CI <- 0.95
        
        ## Number of significant digits to display
        PKG.ENV$SIG_DIGITS <- 2
        
        PKG.ENV$PLOT_FACET_NAMES         <- c()
        PKG.ENV$PLOT_UNCOND_FACET_NAMES  <- c()
        PKG.ENV$PLOT_INTERACT_FACE_NAMES <- c()
        
        #--------------------------------------------
        # Set SERVER globals
        #--------------------------------------------
        
        ## Keep track of baseline selectors that need 
        ## to be rendered
        PKG.ENV$NUM_ATTR_BASELINE_IDS <- 0
        
        ## Keep track of whether we can
        ## actually plot interaction effects
        PKG.ENV$NO_RESP_VARYING <- TRUE
        PKG.ENV$NO_FACET_NAMES  <- TRUE
        PKG.ENV$NO_INTERACTIONS <- TRUE
        
        #--------------------------------------------
        # Initialize + run app
        #--------------------------------------------
        
        ## Update available data + design options
        PKG.ENV$DATA_OPTIONS          <- list(environment()$immigrationconjoint)
        names(PKG.ENV$DATA_OPTIONS)   <- c(PKG.ENV$DEFAULT_DATA_NAME)
        
        PKG.ENV$DESIGN_OPTIONS        <- list(environment()$immigrationdesign, "uniform")
        names(PKG.ENV$DESIGN_OPTIONS) <- c(PKG.ENV$DEFAULT_DESIGN_NAME, "'uniform'")
        
        .LOAD_POSSIBLE_USER_DATA()
        .LOAD_POSSIBLE_USER_DESIGNS()
        
        ## Update selected data + design (default)
        PKG.ENV$SELECTED_DATA   <- PKG.ENV$DATA_OPTIONS[[PKG.ENV$SELECTED_DATA_NAME]]
        PKG.ENV$SELECTED_DESIGN <- PKG.ENV$DESIGN_OPTIONS[[PKG.ENV$SELECTED_DESIGN_NAME]]
        
        ## Create app components
        UI     <- .buildAmceAppUI()
        SERVER <- .buildAmceAppSERVER()
        
        ## Start up
        cat("Launching `amce` browser dashboard")
        runApp(launch.browser=TRUE, list(ui = UI, server = SERVER))
    } else {
        View(object)
    }
}

# =================================================================================
# ==================================== HELPERS ==================================== 
# =================================================================================


.LOAD_POSSIBLE_USER_DATA <- function() {
    ## Scan workspace and load in possible user-generated
    ## results dataframes. Always do this after loading in 
    ## the default results options.
    
    ### get names of applicable data frames
    data_frame_names <- names(which(sapply(.GlobalEnv, is.data.frame)))
    
    data_frame_names <- data_frame_names[!(data_frame_names %in% names(PKG.ENV$DATA_OPTIONS))]
    data_frame_names <- data_frame_names[!(data_frame_names %in% names(PKG.ENV$DESIGN_OPTIONS))]
    data_frame_names <- data_frame_names[(data_frame_names != "SELECTED_DATA") & (data_frame_names != "SELECTED_DESIGN")]
    
    ### get matching data frames themselves
    data_frames <- lapply(data_frame_names, get)
    names(data_frames) <- data_frame_names
    
    ### add to globals
    PKG.ENV$DATA_OPTIONS <- c(PKG.ENV$DATA_OPTIONS, data_frames)
    
}

.LOAD_POSSIBLE_USER_DESIGNS <- function() {
    ## Scan workspace and load in possible user-generated 
    ## conjoint designs. Always do this after loading in 
    ## the default design options.
    
    ### get names of applicable designs
    design_names <- names(which(sapply(.GlobalEnv, function(x){ typeof(x) == "conjointDesign" })))
    design_names <- design_names[!(design_names %in% names(PKG.ENV$DESIGN_OPTIONS))]
    
    ### get matching designs themselves
    designs <- lapply(design_names, get)
    names(designs) <- design_names
    
    ### add to globals
    PKG.ENV$DESIGN_OPTIONS <- c(PKG.ENV$DESIGN_OPTIONS, design_names)
    
}

# =====================================================================================
# ==================================== UI BACK-END ==================================== 
# =====================================================================================

.buildAmceAppUI <- function() {
    # Function that builds the AMCE dashboard RShiny app's UI object
    # in a componentwise manner.
    
#######################################
    analysisInputPanel <- sidebarPanel(
#######################################
        # ------------------------------------------------------       
        h4("INPUT"),
        # ------------------------------------------------------       
        selectInput("designName", HTML("Choose a conjoint design (<code>design</code> object ):"), 
                    choices = c()),
        selectInput("dataName", HTML("Choose a results dataset (<code>data.frame</code>):"),
                    choices = c()),
        # ------------------------------------------------------       
        h4("RESPONSES"),
        # ------------------------------------------------------       
        selectInput("respondent.id", "Respondent ID:",
                    choices = c()),
        strong("Responses to exclude:"),
        verbatimTextOutput('selectedTableRows'),
        # ------------------------------------------------------       
        h4("ESTIMATION"),
        # ------------------------------------------------------       
        selectInput("outcome", "Outcome variable:",
                    choices=c()),
        selectInput("attributes", "Attribute variables:",
                    multiple=TRUE,
                    selectize=TRUE,
                    choices=c()),
        div(id="baselines"),
        selectInput("interactions", "Interaction terms:",
                    multiple=TRUE,
                    selectize=TRUE,
                    choices=c()),
        radioButtons("cluster", "Cluster:",
                     choices = c("Yes", "No")),
        radioButtons("na.ignore", "Ignore missing data:",
                     choices = c("Yes", "No")),
        selectInput("weights", "Weights column:",
                    choices = c()),
        selectInput("respondent.varying", "Respondent varying:",
                    choices = c()),
        
        div(id="baselinesResp"),
        # ------------------------------------------------------       
        actionButton("run", "Estimate AMCE", class="btn-primary")
        # ------------------------------------------------------       
    )

#######################################
    analysisTablePanel <- wellPanel(
#######################################
        height="500px",
        # ------------------------------------------------------
        h4("DATA"),
        # ------------------------------------------------------
        p("(Selected rows will be excluded from analysis)", style="font-size:75%"),
        div(DT::dataTableOutput('selectedDataTable'), style = "font-size: 75%;"),
        # actionButton('selectAllRows',label="Select all entries", class="btn-warning btn-sm"),
        actionButton('resetSelectedRows',label="Deselect all entries", class="btn-warning btn-sm")
    )
    
#######################################
    analysisResultsPanel <- wellPanel(
#######################################
        # ------------------------------------------------------  
        h4("RESULTS"),
        # ------------------------------------------------------
        tabsetPanel(
            tabPanel(
                # ------------------------------------------------------
                h5("Summary"),
                # ------------------------------------------------------
                br(),
                div(id=PKG.ENV$RESULTS_SUMMARY_ID,
                    verbatimTextOutput('results_summary'),
                    tags$style(type='text/css', '#results_summary {max-height: 1000px; overflow-y:scroll; }'),
                    sliderInput("sigdigits",
                                HTML("<small>Significant digits:</small>"),
                                min=1,
                                max=4,
                                value=2),
                    downloadButton("downloadSummary", "Download report")
                )
            ),
            tabPanel(
                h5("Plot"),
                br(),
                div(id=PKG.ENV$RESULTS_PLOT_ID, 
                    plotOutput("results_plot", height="800"),
                    br(),
                    selectInput("plotFacetNames",
                                HTML("<small>Facet names (optional):</small>"),
                                multiple=TRUE,
                                selectize=TRUE,
                                choices=c()),
                    textInput("plotFontFamily",
                              HTML("<small>Font family (specify if non-latin characters do not render, else leave blank):</small>")),
                    sliderInput("plotTextSize",
                                HTML("<small>Text size:</small>"),
                                min=PKG.ENV$PLOT_TEXT_SIZE_DEFAULT_MIN,
                                max=PKG.ENV$PLOT_TEXT_SIZE_DEFAULT_MAX,
                                value=PKG.ENV$PLOT_TEXT_SIZE_DEFAULT_VAL),
                    sliderInput("plotCI",
                                HTML("<small>Confidence intervals:</small>"),
                                min=0.0,
                                max=1.0,
                                value=PKG.ENV$PLOT_DEFAULT_CI,
                                step=0.01),
                    downloadButton("downloadPlot", "Download PDF")
                )
            ),
            tabPanel(
                h5("Plot (unconditional)"),
                br(),
                div(id=PKG.ENV$RESULTS_UNCOND_PLOT_ID, 
                    plotOutput("results_uncond_plot", height="800"),
                    shinyjs::hidden(
                        div(id="errorUncondMessage",
                            HTML("<br><p><i>Could not render plot. Please specify different settings below.</i></p><br>")
                        )
                    ),
                    br(),
                    textInput("plotUncondFontFamily",
                              HTML("<small>Font family (specify if non-latin characters do not render, else leave blank):</small>")),
                    # selectInput("plotUncondFacetNames",
                    #             HTML("<small>Facet names (optional):</small>"),
                    #             multiple=TRUE,
                    #             selectize=TRUE,
                    #             choices=c()),
                    sliderInput("plotUncondTextSize",
                                HTML("<small>Text size:</small>"),
                                min=PKG.ENV$PLOT_TEXT_SIZE_DEFAULT_MIN,
                                max=PKG.ENV$PLOT_TEXT_SIZE_DEFAULT_MAX,
                                value=PKG.ENV$PLOT_TEXT_SIZE_DEFAULT_VAL),
                    sliderInput("plotUncondCI",
                                HTML("<small>Confidence intervals:</small>"),
                                min=0.0,
                                max=1.0,
                                value=PKG.ENV$PLOT_DEFAULT_CI,
                                step=0.01),
                    downloadButton("downloadPlotUncond", "Download PDF")
                )
            ),
            tabPanel(
                h5("Plot (interaction)"),
                br(),
                div(id=PKG.ENV$RESULTS_INTERACT_PLOT_ID, 
                    shinyjs::hidden(
                        div(id="noInteractMessage",
                            HTML("<p><i>No interactions specified in model <i>or</i> no <code>facet.names</code>
                                 provided. Please re-run model with <code>respondent.varying</code>, and/or
                                 interaction terms specified, and (optionally) provide <code>facet.names</code> 
                                 below.</i></p><br>")
                            )
                    ),
                    plotOutput("results_interact_plot", height="800"),
                    shinyjs::hidden(
                        div(id="errorInteractMessage",
                            HTML("<br><p><i>Could not render plot. Please specify different settings below.</i></p><br>")
                        )
                    ),
                    br(),
                    selectInput("plotInteractFacetNames",
                                multiple=TRUE,
                                selectize=TRUE,
                                HTML("<small>Facet names (optional):</small>"),
                                choices=c()),
                    textInput("plotInteractFontFamily",
                              HTML("<small>Font family (specify if non-latin characters do not render, else leave blank):</small>")),
                    sliderInput("plotInteractTextSize",
                                HTML("<small>Text size:</small>"),
                                min=PKG.ENV$PLOT_TEXT_SIZE_DEFAULT_MIN,
                                max=PKG.ENV$PLOT_TEXT_SIZE_DEFAULT_MAX,
                                value=PKG.ENV$PLOT_TEXT_SIZE_DEFAULT_VAL),
                    sliderInput("plotInteractCI",
                                HTML("<small>Confidence intervals:</small>"),
                                min=0.0,
                                max=0.99,
                                value=PKG.ENV$PLOT_DEFAULT_CI,
                                step=0.01),
                    downloadButton("downloadPlotInteract", "Download PDF")
                            )
                    )
            )
        )
#######################################
    analysisTabPanel <- tagList("",
#######################################
                                analysisInputPanel,
                                mainPanel(
                                    analysisTablePanel,
                                    shinyjs::hidden(
                                        div(id=PKG.ENV$RESULTS_ID,
                                            analysisResultsPanel
                                        )
                                    )
                                )
    )
    
#######################################
    UI <- shinyUI(fluidPage(shinyjs::useShinyjs(),
#######################################
                            shinyjs::extendShinyjs(text = "shinyjs.closeWindow = function() { window.close(); }", functions = c("closeWindow")),
                            br(),
                            ## SITE TITLE
                            titlePanel(HTML("<code>cjoint</code> AMCE dashboard")),
                            ## SITE SUBTITLE
                            p(("For any loaded conjoint design and a corresponding dataset of conjoint experiment results, 
                               quickly estimate and analyze the AMCE (Average Marginal Component Effects)."),
                              ## EXIT BUTTON
                              actionButton("close", "Close app", icon = icon("times"), style='padding:4px; font-size:80%; position:absolute;right:3em; top:3em;')
                            ),
                            br(),
                            ## SITE CONTENT
                            analysisTabPanel, 
                            ## WINDOW TITLE
                            title=PKG.ENV$WINDOW_TITLE_TEXT))
    
    return(UI)
    
}


# =========================================================================================
# ==================================== SERVER BACK-END ==================================== 
# =========================================================================================

.buildAmceAppSERVER <- function() {
    # Function that builds the AMCE dashboard RShiny app's SERVER function
    # in a componentwise manner. 
    SERVER <- function(input, output, clientData, session) {
        
        observe({if (input$close > 0) {
            shinyjs::js$closeWindow()
            stopApp()
        }})
        
        .amceApp.initSession(input, output, clientData, session)
    
        .amceApp.waitForUIUpdates(input, output, clientData, session)
        .amceApp.waitForRunAction(input, output, clientData, session)
        .amceApp.waitForResultsSettingsUpdates(input, output, clientData, session)
        .amceApp.waitForPlotSettingsUpdates(input, output, clientData, session)
        .amceApp.waitForPlotUncondSettingsUpdates(input, output, clientData, session)
        .amceApp.waitForPlotInteractSettingsUpdates(input, output, clientData, session)
    }
    return(SERVER)
}

#--------------------------------------------
# INIT fxn
#--------------------------------------------

.amceApp.initSession <- function(input, output, clientData, session){
    ######################################################
    ## Initialize the current server session            ##
    ## (first thing to call upon running the Shiny app) ##
    ######################################################
    withProgress(message="Initializing dashboard session", value=0, {
        
        ### Hide elements that must conditionally appear
        # shinyjs::hide(RESULTS_SUMMARY_ID)
        # shinyjs::hide(RESULTS_PLOT_ID)
        # shinyjs::hide(PLOT_SETTINGS_ID)
        shinyjs::hide(PKG.ENV$RESULTS_ID)
        shinyjs::hide("baselines")
        shinyjs::hide("baselinesResp")
        
        # ### Initialize all UI elements with default values 
        # .amceApp.updateUI() 
        
    })
}


#--------------------------------------------
# ACTION fxns
#--------------------------------------------


.amceApp.updateUI <- function(session) {
    ## Update user interface elements based on user selected data/design
    ## which is stored in global variables
    updateSelectInput(session, 
                      "designName",
                      choices=names(PKG.ENV$DESIGN_OPTIONS),
                      selected=PKG.ENV$SELECTED_DESIGN_NAME)
    
    updateSelectInput(session, 
                      "dataName",
                      choices=names(PKG.ENV$DATA_OPTIONS),
                      selected=PKG.ENV$SELECTED_DATA_NAME)
    
    updateSelectInput(session,
                      "outcome",
                      choices=names(PKG.ENV$SELECTED_DATA),
                      selected=NULL)
    
    updateSelectInput(session,
                      "attributes",
                      choices=names(PKG.ENV$SELECTED_DATA),
                      selected=NULL)
    
    updateSelectInput(session,
                      "interactions",
                      choices=NULL,
                      selected=NULL)
    
    updateSelectInput(session, 
                      "respondent.id",
                      choices=names(PKG.ENV$SELECTED_DATA),
                      selected=names(PKG.ENV$SELECTED_DATA)[1])
    
    updateSelectInput(session, 
                      "respondent.varying",
                      choices=c("None", names(PKG.ENV$SELECTED_DATA)),
                      selected="None")
    
    updateSelectInput(session, 
                      "weights",
                      choices=c("None", names(PKG.ENV$SELECTED_DATA)),
                      selected="None") 
}

.amceApp.getInteractionTerms <- function(varNames) {
    ## From a list of variables, e.g. [a, b], output
    ## all possible interaction terms, e.g.
    ## [a:b].
    cartesProd <- expand.grid(varNames, varNames)
    interTerms <- sapply(1:nrow(cartesProd), 
                         function(s) { 
                             paste(cartesProd[s, "Var1"], cartesProd[s, "Var2"], sep=":") 
                         })
    return(interTerms)
}

.amceApp.getResults <- function(input, excl) {
    ########################################
    ## Perform an AMCE run given UI input ##
    ########################################
    encFxn <- function(x){return(paste0("`",x,"`"))}
    
    ### Create formula
    interactions <- sapply(input$interactions, function(x){paste(sapply(strsplit(x, ":"), encFxn), collapse=":")}, USE.NAMES=FALSE)
    all_terms    <- c(sapply(c(input$attributes), encFxn, USE.NAMES=FALSE), interactions)
    fmula        <- as.formula(paste(encFxn(input$outcome), paste(all_terms, collapse=" + "), sep=" ~ "))
    
    ### Subset on selected rows
    if (!is.null(input$selectedDataTable_rows_selected)) {
        rowSubset <- sapply(1:nrow(PKG.ENV$SELECTED_DATA), function(x){ return(ifelse(x %in% input$selectedDataTable_rows_selected, FALSE, TRUE)) })
    } else {
        rowSubset <- NULL
    }
    
    if (input$respondent.id == "None"){
        respondentID <-  NULL
    } else {
        respondentID <-  input$respondent.id
    }
    
    if (input$respondent.varying == "None"){
        respondentVarying <-  NULL
        PKG.ENV$NO_RESP_VARYING <- TRUE
    } else {
        respondentVarying <- input$respondent.varying
        PKG.ENV$NO_RESP_VARYING <- FALSE
    }
    
    if (length(input$interactions) == 0) {
        PKG.ENV$NO_INTERACTIONS <- TRUE
    } else {
        PKG.ENV$NO_INTERACTIONS <- FALSE
    }
    
    ### Collect user inputs for attr baselines if any 
    baselines <- c()
    for (name in colnames(PKG.ENV$SELECTED_DATA)){
        baseline <- input[[paste0("btn", name)]]
        if (!(is.null(baseline))){
            baselines[[name]] <- baseline
        }
    }
    ### Collect user inputs for resp. varying baselines if any 
    for (name in colnames(PKG.ENV$SELECTED_DATA)){
        baseline <- input[[paste0("btnR", name)]]
        baseline <- input[[paste0("btn", name)]]
        if (!(is.null(baseline))){
            baselines[[name]] <- baseline
        }
    }

    ### Run AMCE fxn with all other user inputs as options
    results <- amce(fmula,
                    data=PKG.ENV$SELECTED_DATA,
                    subset=rowSubset,
                    cluster=ifelse(input$cluster == "Yes", TRUE, FALSE),
                    respondent.id=respondentID,
                    respondent.varying=respondentVarying, 
                    na.ignore=ifelse(input$na.ignore == "Yes", TRUE, FALSE),
                    baselines=baselines,
                    design=PKG.ENV$SELECTED_DESIGN)
    return(results)
}


.amceApp.plotResults <- function(results, ylim, textsize, ci, facetnames, font.family=NULL){
    #####################################
    ## Plot the results of an AMCE run ##
    #####################################
    graphics::plot(results, 
         height=PKG.ENV$DEFAULT_RESULTS_HEIGHT,
         ylim=ylim,
         ci=ci,
         # breaks=c(-.2, 0, .2),
         # labels=c("-.2","0",".2"), 
         text.size=textsize,
         plot.theme=theme(text=element_text(family=font.family, size=textsize)),
         plot.display="all",
         facet.names=facetnames)
}


.amceApp.plotUncondResults <- function(results, ylim, textsize, ci, facetnames, font.family=NULL){
    ############################################
    ## Plot the UNCOND results of an AMCE run ##
    ############################################
    graphics::plot(results, 
         height=PKG.ENV$DEFAULT_RESULTS_HEIGHT,
         ylim=ylim,
         ci=ci,
         # breaks=c(-.2, 0, .2),
         # labels=c("-.2","0",".2"), 
         text.size=textsize,
         plot.theme=theme(text=element_text(family=font.family, size=textsize)),
         plot.display="unconditional",
         facet.names=facetnames)
}


.amceApp.plotInteractResults <- function(results, ylim, textsize, ci, facetnames, font.family=NULL){
    
    if (PKG.ENV$NO_RESP_VARYING & PKG.ENV$NO_FACET_NAMES) {
        shinyjs::show("noInteractMessage")
    } else {
        shinyjs::hide("noInteractMessage")
    }
    
    ##############################################
    ## Plot the INTERACT results of an AMCE run ##
    ##############################################
    graphics::plot(results, 
         height=PKG.ENV$DEFAULT_RESULTS_HEIGHT,
         ylim=ylim,
         ci=ci,
         # breaks=c(-.2, 0, .2),
         # labels=c("-.2","0",".2"), 
         text.size=textsize,
         plot.display="interaction",
         plot.theme=theme(text=element_text(family=font.family, size=textsize)),
         facet.names=facetnames)
}

#--------------------------------------------
# LISTENER fxns
#--------------------------------------------

.amceApp.waitForUIUpdates <- function(input, output, clientData, session) {
    ###############################################
    ## Update the dashboard state based on input ## 
    ## selected by the user                      ##
    ###############################################
    
    observeEvent(input$designName, {
        ## (Design is changed)
        if(input$designName != PKG.ENV$SELECTED_DESIGN_NAME) {
            PKG.ENV$SELECTED_DESIGN_NAME <- input$designName
            PKG.ENV$SELECTED_DESIGN <- PKG.ENV$DESIGN_OPTIONS[[PKG.ENV$SELECTED_DESIGN_NAME]]
        }
    })
    
    observeEvent(input$dataName, {
        ## (Dataset is changed)
        
        if(input$dataName != PKG.ENV$SELECTED_DATA_NAME) {
            
            ### Remove baseline selectors for existing dataset
            for(i in 1:length(colnames(PKG.ENV$SELECTED_DATA))) {
                attrBaselineID <- gsub(" ", "_", colnames(PKG.ENV$SELECTED_DATA)[i])
                removeUI(selector=paste0("div#", attrBaselineID), immediate=TRUE)
            }
            
            ### Extract chosen input or choose default input
            if(input$designName == "" | input$dataName == "") {
                PKG.ENV$SELECTED_DESIGN_NAME <- PKG.ENV$DEFAULT_DESIGN_NAME
                PKG.ENV$SELECTED_DATA_NAME   <- PKG.ENV$DEFAULT_DATA_NAME
            } else {
                PKG.ENV$SELECTED_DESIGN_NAME <- input$designName
                PKG.ENV$SELECTED_DATA_NAME   <- input$dataName
            }
            
            PKG.ENV$SELECTED_DESIGN <-  PKG.ENV$DESIGN_OPTIONS[[PKG.ENV$SELECTED_DESIGN_NAME]]
            PKG.ENV$SELECTED_DATA   <-  PKG.ENV$DATA_OPTIONS[[PKG.ENV$SELECTED_DATA_NAME]]
            
            ### Update data/design specific elements dynamically
            withProgress(message="Updating UI", value=0, {
                
                .amceApp.updateUI(session)
                
                # updateSelectInput(session, 
                #                   "respondent.id",
                #                   choices=names(SELECTED_DATA),
                #                   selected=names(SELECTED_DATA)[1])
                # 
                # updateSelectInput(session, 
                #                   "weights",
                #                   choices=c("None", names(SELECTED_DATA)),
                #                   selected="None")
                
            })
            
            ### Update data table
            selectedRows <- reactiveValues(sel=c())
            
            output$selectedDataTable <- DT::renderDataTable(
                DT::datatable(
                    input$resetSelectedRows,
                    data=PKG.ENV$SELECTED_DATA,
                    options=list(
                        pageLength=5,
                        scrollX=TRUE,
                        scrollY=TRUE,
                        lengthChange=FALSE,
                        processing=FALSE,
                        rowCallback=DT::JS("function(r,d) {$(r).attr('min-height', '20px')}")
                    ),
                    selection=list(mode="multiple", selected=selectedRows$sel),
                    rownames = TRUE,
                    width="100%"
                ))
            
            output$selectedTableRows <- renderPrint({
                s = input$selectedDataTable_rows_selected
                if (length(s)) {
                    cat(s, sep=",\n")
                } else {
                    cat("None")
                }
            })
            
            
            
        }
    }, ignoreInit=FALSE)
    
    observeEvent(input$attributes, {
        ## (Input attributes are changed)
        
        updateSelectInput(session,
                          "interactions",
                          choices=.amceApp.getInteractionTerms(input$attributes),
                          selected=NULL)
        
        if(!is.null(input$attributes)) {
            shinyjs::show("baselines")
        }
        
        if (length(input$attributes) > PKG.ENV$NUM_ATTR_BASELINE_IDS) {
            ### ADD baseline factors if needed
            PKG.ENV$NUM_ATTR_BASELINE_IDS <- length(input$attributes)
            
            for(i in 1:length(input$attributes)) {
                attrBaselineID <- gsub(" ", "_", input$attributes[i])
                if (is.factor(PKG.ENV$SELECTED_DATA[[input$attributes[i]]]) & !(attrBaselineID %in% PKG.ENV$ATTR_BASELINE_IDS) ) {
                    PKG.ENV$ATTR_BASELINE_IDS <- unique(c(PKG.ENV$ATTR_BASELINE_IDS, attrBaselineID))
                    insertUI(selector="#baselines",
                             ui=tags$div(id=attrBaselineID)
                    )
                    
                    insertUI(selector=paste0("#",attrBaselineID),
                             ui=selectInput(paste0("btn",attrBaselineID), 
                                            HTML(paste0("<small>&emsp;&#9830; ",input$attributes[i]," baseline</small>:")), 
                                            levels(PKG.ENV$SELECTED_DATA[[input$attributes[i]]]))
                    )
                }    
            }
            
        } else {
            ### REMOVE baseline factors if needed
            PKG.ENV$NUM_ATTR_BASELINE_IDS <- length(input$attributes)
            
            for(i in 1:length(colnames(PKG.ENV$SELECTED_DATA))) {
                attrBaselineID <- gsub(" ", "_", colnames(PKG.ENV$SELECTED_DATA)[i])
                if ((attrBaselineID %in% PKG.ENV$ATTR_BASELINE_IDS) & !(colnames(PKG.ENV$SELECTED_DATA)[i] %in% input$attributes)){
                    removeUI(selector=paste0("div#", attrBaselineID), immediate=TRUE)
                }
            }
        }
        
    })
    
    observeEvent(input$respondent.varying, {
        ### (Respondent varying variable is changed)
        
        if(input$respondent.varying != "None") {
            shinyjs::show("baselinesResp")
            if (is.factor(PKG.ENV$SELECTED_DATA[[input$respondent.varying]])) {
                ### REMOVE previous baseline factor (if exists)
                removeUI(selector=paste0("div#", PKG.ENV$RESP_BASELINE_ID), immediate=TRUE)
                
                ### ADD baseline factor
                PKG.ENV$RESP_BASELINE_ID <- gsub(" ", "_", input$respondent.varying)
                PKG.ENV$RESP_BASELINE_ID <- paste0("R", PKG.ENV$RESP_BASELINE_ID)
                
                insertUI(selector="#baselinesResp",
                         ui=tags$div(id=PKG.ENV$RESP_BASELINE_ID)
                )
                
                insertUI(selector=paste0("#", PKG.ENV$RESP_BASELINE_ID),
                         ui=selectInput(paste0("btn", PKG.ENV$RESP_BASELINE_ID), 
                                        HTML(paste0("<small>&emsp;&emsp;&emsp;&emsp;&#9830; ",input$respondent.varying," baseline</small>:")), 
                                        levels(PKG.ENV$SELECTED_DATA[[input$respondent.varying]]))
                ) 
            } else {
                removeUI(selector=paste0("div#", PKG.ENV$RESP_BASELINE_ID), immediate=TRUE)
                shinyjs::hide("baselinesResp")
            }
        } else {
            shinyjs::hide("baselinesResp")
        }
        
    })
    
}

.amceApp.waitForRunAction <- function(input, output, clientData, session) {
    #############################################################
    ## Update the dashboard when the user chooses to hit "run" ##
    #############################################################
    observeEvent(input$run, {
        withProgress(message="Computing results", value=0, {
            incProgress(1/4, detail = "Running AMCE estimator")
            
            tryCatch({
                ## Hide `facet.names` selectors in case there's an
                ## existing plot already shown (unsafe to change b/t plots!)
                shinyjs::hide("plotFacetNames")
                shinyjs::hide("plotUncondFacetNames")
                shinyjs::hide("plotInteractFacetNames")
                
                PKG.ENV$LAST_RESULTS <- .amceApp.getResults(input)
                PKG.ENV$SIG_DIGITS   <- input$sigdigits
                
                incProgress(2/4, detail="Getting summary")
                output$results_summary <- renderPrint( 
                    ## Show according to significant digits specified by user
                    print(summary(PKG.ENV$LAST_RESULTS), digits=PKG.ENV$SIG_DIGITS)
                )
                
                incProgress(3/4, detail="Creating plot")
                
                ## Update possible `facet.names` the user can select for each plot
                interacted_terms <- unique(Reduce(c,sapply(input$interactions, function(x) { strsplit(x, ":") }) ))
                if (input$respondent.varying == "None") {
                    possible_facet_names <- interacted_terms
                } else {
                    possible_facet_names <- c(input$respondent.varying, interacted_terms)
                }
                
                PKG.ENV$PLOT_FONT_FAMILY          <- NULL
                PKG.ENV$PLOT_UNCOND_FONT_FAMILY   <- NULL
                PKG.ENV$PLOT_INTERACT_FONT_FAMILY <- NULL

                PKG.ENV$PLOT_FACET_NAMES <- NULL
                updateSelectInput(session, "plotFacetNames",
                                  choices=possible_facet_names,
                                  selected=NULL)
                
                PKG.ENV$PLOT_UNCOND_FACET_NAMES <- NULL
                updateSelectInput(session, "plotUncondFacetNames",
                                  choices=possible_facet_names,
                                  selected=NULL)
                
                PKG.ENV$PLOT_INTERACT_FACET_NAMES <- NULL
                updateSelectInput(session, "plotInteractFacetNames",
                                  choices=possible_facet_names,
                                  selected=NULL)
                
                ## Show `facet.names` selectors when it's safe again
                shinyjs::show("plotFacetNames")
                shinyjs::show("plotUncondFacetNames")
                shinyjs::show("plotInteractFacetNames")
                
                output$results_plot <- renderPlot({
                    withProgress(message="Rendering plot (all estimates) in UI", {
                        .amceApp.plotResults(PKG.ENV$LAST_RESULTS, 
                                    ci=PKG.ENV$PLOT_DEFAULT_CI,
                                    ylim=c(PKG.ENV$PLOT_YLIM_DEFAULT_MIN, PKG.ENV$PLOT_YLIM_DEFAULT_MAX),
                                    textsize=PKG.ENV$PLOT_TEXT_SIZE_DEFAULT_VAL,
                                    facetnames=PKG.ENV$PLOT_FACET_NAMES)       
                    })
                })
                
                output$results_uncond_plot <- renderPlot({
                    withProgress(message="Rendering plot (unconditional estimates) in UI", {
                        .amceApp.plotUncondResults(PKG.ENV$LAST_RESULTS, 
                                          ci=PKG.ENV$PLOT_DEFAULT_CI,
                                          ylim=c(PKG.ENV$PLOT_YLIM_DEFAULT_MIN, PKG.ENV$PLOT_YLIM_DEFAULT_MAX),
                                          textsize=PKG.ENV$PLOT_TEXT_SIZE_DEFAULT_VAL,
                                          facetnames=PKG.ENV$PLOT_UNCOND_FACET_NAMES)      
                    })
                })
                
                output$results_interact_plot <- renderPlot({
                    withProgress(message="Rendering plot (interaction estimates) in UI", {
                        .amceApp.plotInteractResults(PKG.ENV$LAST_RESULTS, 
                                            ci=PKG.ENV$PLOT_DEFAULT_CI,
                                            ylim=c(PKG.ENV$PLOT_YLIM_DEFAULT_MIN, PKG.ENV$PLOT_YLIM_DEFAULT_MAX),
                                            textsize=PKG.ENV$PLOT_TEXT_SIZE_DEFAULT_VAL,
                                            facetnames=PKG.ENV$PLOT_INTERACT_FACET_NAMES)      
                    })
                })
                
                
                incProgress(4/4, detail="Generating downloads")
                output$downloadSummary <- downloadHandler(
                    filename =  paste(PKG.ENV$SELECTED_DATA_NAME, "summary", Sys.Date(), ".txt", sep="_"),
                    content = function(file){
                        sink(file=file)
                        print(summary(PKG.ENV$LAST_RESULTS), digits=PKG.ENV$SIG_DIGITS)
                        sink()
                    })
                output$downloadPlot <- downloadHandler(
                    filename =  paste(PKG.ENV$SELECTED_DATA_NAME, "plot", Sys.Date(), ".pdf"),
                    content = function(file){
                        pdf(file, width=8.5, height=15, paper="special")
                        .amceApp.plotResults(PKG.ENV$LAST_RESULTS, 
                                    ylim=PKG.ENV$PLOT_YLIM_SELECTED,
                                    ci=PKG.ENV$PLOT_CI,
                                    textsize=PKG.ENV$PLOT_TEXT_SIZE_SELECTED,
                                    facetnames=PKG.ENV$PLOT_FACET_NAMES)                          
                        dev.off()
                    })
                output$downloadPlotUncond <- downloadHandler(
                    filename =  paste(PKG.ENV$SELECTED_DATA_NAME, "plotUnconditional", Sys.Date(), ".pdf", sep="_"),
                    content = function(file){
                        pdf(file, width=8.5, height=15, paper="special")
                        .amceApp.plotUncondResults(PKG.ENV$LAST_RESULTS, 
                                          ylim=PKG.ENV$PLOT_UNCOND_YLIM_SELECTED,
                                          ci=PKG.ENV$PLOT_UNCOND_CI,
                                          textsize=PKG.ENV$PLOT_UNCOND_TEXT_SIZE_SELECTED,
                                          facetnames=PKG.ENV$PLOT_UNCOND_FACET_NAMES)                          
                        dev.off()
                    })
                output$downloadPlotInteract <- downloadHandler(
                    filename =  paste(PKG.ENV$SELECTED_DATA_NAME, "plotInteraction", Sys.Date(), ".pdf", sep="_"),
                    content = function(file){
                        pdf(file, width=8.5, height=15, paper="special")
                        .amceApp.plotInteractResults(PKG.ENV$LAST_RESULTS, 
                                            ylim=PKG.ENV$PLOT_INTERACT_YLIM_SELECTED,
                                            ci=PKG.ENV$PLOT_INTERACT_CI,
                                            textsize=PKG.ENV$PLOT_INTERACT_TEXT_SIZE_SELECTED,
                                            facetnames=PKG.ENV$PLOT_INTERACT_FACET_NAMES)                          
                        dev.off()
                    })
                
                
                shinyjs::show(PKG.ENV$RESULTS_ID)
            },
            error=function(e){
                
                # shinyjs::hide(RESULTS_ID)
                
                ### Check for obvious user input errors first
                if (is.null(input$attributes)) {
                    e <- "Please enter a valid set of attributes."
                }
                
                showModal(modalDialog(e, title="Error", size="m", easyClose=TRUE))
                # print(e)
            })
        })
    })
}


.amceApp.waitForResultsSettingsUpdates <- function(input, output, clientData, session){
    #################################################################
    ## Functions to run when user adjusts results summary settings ##
    #################################################################
    observeEvent(input$sigdigits, {
        
        PKG.ENV$SIG_DIGITS <- input$sigdigits
        
        output$results_summary <- renderPrint( 
            ## Show according to significant digits specified by user
            print(summary(PKG.ENV$LAST_RESULTS), digits=PKG.ENV$SIG_DIGITS)
        )
    })
    
}

.amceApp.waitForPlotSettingsUpdates <- function(input, output, clientData, session){
    ######################################################
    ## Functions to run when user adjusts plot settings ##
    ######################################################
    observeEvent(input$plotTextSize, {
        PKG.ENV$PLOT_TEXT_SIZE_SELECTED <- input$plotTextSize
        
        output$results_plot <- renderPlot({
            withProgress(message="Rendering plot (all estimates) in UI", {
                .amceApp.plotResults(PKG.ENV$LAST_RESULTS, 
                            ci=PKG.ENV$PLOT_CI,
                            ylim=PKG.ENV$PLOT_YLIM_SELECTED,
                            textsize=PKG.ENV$PLOT_TEXT_SIZE_SELECTED,
                            facetnames=PKG.ENV$PLOT_FACET_NAMES,
                            font.family=PKG.ENV$PLOT_FONT_FAMILY)       
            })
        })
    })
    
    observeEvent(input$plotFontFamily, {
        PKG.ENV$PLOT_FONT_FAMILY <- input$plotFontFamily
        
        output$results_plot <- renderPlot({
            withProgress(message="Rendering plot (all estimates) in UI", {
                .amceApp.plotResults(PKG.ENV$LAST_RESULTS, 
                                    ci=PKG.ENV$PLOT_CI,
                                    ylim=PKG.ENV$PLOT_YLIM_SELECTED,
                                    textsize=PKG.ENV$PLOT_TEXT_SIZE_SELECTED,
                                    facetnames=PKG.ENV$PLOT_FACET_NAMES,
                                    font.family=PKG.ENV$PLOT_FONT_FAMILY)       
            })
        })
    }) 
    
    observeEvent(input$plotCI, {
        PKG.ENV$PLOT_CI <- input$plotCI
        
        output$results_plot <- renderPlot({
            withProgress(message="Rendering plot (all estimates) in UI", {
                .amceApp.plotResults(PKG.ENV$LAST_RESULTS, 
                            ci=PKG.ENV$PLOT_CI,
                            ylim=PKG.ENV$PLOT_YLIM_SELECTED,
                            textsize=PKG.ENV$PLOT_TEXT_SIZE_SELECTED,
                            facetnames=PKG.ENV$PLOT_FACET_NAMES,
                            font.family=PKG.ENV$PLOT_FONT_FAMILY)       
            })
        })
    })
    
    observeEvent(input$plotFacetNames, {
        
        PKG.ENV$PLOT_FACET_NAMES <- input$plotFacetNames
        
        output$results_plot <- renderPlot({
            withProgress(message="Rendering plot (all estimates) in UI", {
                .amceApp.plotResults(PKG.ENV$LAST_RESULTS, 
                            ci=PKG.ENV$PLOT_CI,
                            ylim=PKG.ENV$PLOT_YLIM_SELECTED,
                            textsize=PKG.ENV$PLOT_TEXT_SIZE_SELECTED,
                            facetnames=PKG.ENV$PLOT_FACET_NAMES,
                            font.family=PKG.ENV$PLOT_FONT_FAMILY)       
            })
        })  
    }, ignoreNULL=FALSE)
}


.amceApp.waitForPlotUncondSettingsUpdates <- function(input, output, clientData, session){
    #############################################################
    ## Functions to run when user adjusts UNCOND plot settings ##
    #############################################################
    observeEvent(input$plotUncondTextSize, {
        
        PKG.ENV$PLOT_UNCOND_TEXT_SIZE_SELECTED <- input$plotUncondTextSize
        
        output$results_uncond_plot <- renderPlot({
            withProgress(message="Rendering plot (unconditional estimates) in UI", {
                .amceApp.plotUncondResults(PKG.ENV$LAST_RESULTS, 
                                  ci=PKG.ENV$PLOT_UNCOND_CI,
                                  ylim=PKG.ENV$PLOT_UNCOND_YLIM_SELECTED,
                                  textsize=PKG.ENV$PLOT_UNCOND_TEXT_SIZE_SELECTED,
                                  facetnames=PKG.ENV$PLOT_UNCOND_FACET_NAMES,
                                  font.family=PKG.ENV$PLOT_UNCOND_FONT_FAMILY)       
            })
        })
    })
    
    observeEvent(input$plotUncondFontFamily, {
        PKG.ENV$PLOT_UNCOND_FONT_FAMILY <- input$plotUncondFontFamily
        
        output$results_uncond_plot <- renderPlot({
            withProgress(message="Rendering plot (all estimates) in UI", {
                .amceApp.plotUncondResults(PKG.ENV$LAST_RESULTS, 
                                     ci=PKG.ENV$PLOT_UNCOND_CI,
                                     ylim=PKG.ENV$PLOT_UNCOND_YLIM_SELECTED,
                                     textsize=PKG.ENV$PLOT_UNCOND_TEXT_SIZE_SELECTED,
                                     facetnames=PKG.ENV$PLOT_UNCOND_FACET_NAMES,
                                     font.family=PKG.ENV$PLOT_UNCOND_FONT_FAMILY)       
            })
        })        
    }) 
    
    observeEvent(input$plotUncondCI, {
        PKG.ENV$PLOT_UNCOND_CI <- input$plotUncondCI
        
        output$results_uncond_plot <- renderPlot({
            withProgress(message="Rendering plot (unconditional estimates) in UI", {
                .amceApp.plotUncondResults(PKG.ENV$LAST_RESULTS, 
                                  ci=PKG.ENV$PLOT_UNCOND_CI,
                                  ylim=PKG.ENV$PLOT_UNCOND_YLIM_SELECTED,
                                  textsize=PKG.ENV$PLOT_UNCOND_TEXT_SIZE_SELECTED,
                                  facetnames=PKG.ENV$PLOT_UNCOND_FACET_NAMES,
                                  font.family=PKG.ENV$PLOT_UNCOND_FONT_FAMILY)       
            })
        })
    })
    
    observeEvent(input$plotUncondFacetNames, {
        
        PKG.ENV$PLOT_UNCOND_FACET_NAMES <- input$plotUncondFacetNames
        
        output$results_uncond_plot <- renderPlot({
            withProgress(message="Rendering plot (unconditional estimates) in UI", {
                .amceApp.plotUncondResults(PKG.ENV$LAST_RESULTS, 
                                  ci=PKG.ENV$PLOT_UNCOND_CI,
                                  ylim=PKG.ENV$PLOT_UNCOND_YLIM_SELECTED,
                                  textsize=PKG.ENV$PLOT_UNCOND_TEXT_SIZE_SELECTED,
                                  facetnames=PKG.ENV$PLOT_UNCOND_FACET_NAMES,
                                  font.family=PKG.ENV$PLOT_UNCOND_FONT_FAMILY)       
            })
        })  
    }, ignoreNULL=FALSE)
}


.amceApp.waitForPlotInteractSettingsUpdates <- function(input, output, clientData, session){
    #############################################################
    ## Functions to run when user adjusts UNCOND plot settings ##
    #############################################################
    observeEvent(input$plotInteractTextSize, {
        
        PKG.ENV$PLOT_INTERACT_TEXT_SIZE_SELECTED <- input$plotInteractTextSize
        
        output$results_interact_plot <- renderPlot({
            withProgress(message="Rendering plot (interaction estimates) in UI", {
                .amceApp.plotInteractResults(PKG.ENV$LAST_RESULTS, 
                                             ci=PKG.ENV$PLOT_INTERACT_CI,
                                             ylim=PKG.ENV$PLOT_INTERACT_YLIM_SELECTED,
                                             textsize=PKG.ENV$PLOT_INTERACT_TEXT_SIZE_SELECTED,
                                             facetnames=PKG.ENV$PLOT_INTERACT_FACET_NAMES,
                                             font.family=PKG.ENV$PLOT_INTERACT_FONT_FAMILY)       
            })
        })
        
    })
    
    
    observeEvent(input$plotInteractFontFamily, {
        PKG.ENV$PLOT_INTERACT_FONT_FAMILY <- input$plotInteractFontFamily
        
        output$results_interact_plot <- renderPlot({
            withProgress(message="Rendering plot (all estimates) in UI", {
                .amceApp.plotInteractResults(PKG.ENV$LAST_RESULTS, 
                                             ci=PKG.ENV$PLOT_INTERACT_CI,
                                             ylim=PKG.ENV$PLOT_INTERACT_YLIM_SELECTED,
                                             textsize=PKG.ENV$PLOT_INTERACT_TEXT_SIZE_SELECTED,
                                             facetnames=PKG.ENV$PLOT_INTERACT_FACET_NAMES,
                                             font.family=PKG.ENV$PLOT_INTERACT_FONT_FAMILY)       
            })
        })        
    }) 
    
    
    observeEvent(input$plotInteractCI, {
        
        PLOT_INTERACT_CI <- input$plotInteractCI
        
        output$results_interact_plot <- renderPlot({
            withProgress(message="Rendering plot (interaction estimates) in UI", {
                .amceApp.plotInteractResults(PKG.ENV$LAST_RESULTS, 
                                             ci=PKG.ENV$PLOT_INTERACT_CI,
                                             ylim=PKG.ENV$PLOT_INTERACT_YLIM_SELECTED,
                                             textsize=PKG.ENV$PLOT_INTERACT_TEXT_SIZE_SELECTED,
                                             facetnames=PKG.ENV$PLOT_INTERACT_FACET_NAMES,
                                             font.family=PKG.ENV$PLOT_INTERACT_FONT_FAMILY)       
            })
        })
    })
    
    observeEvent(input$plotInteractFacetNames, {
        
        PKG.ENV$PLOT_INTERACT_FACET_NAMES <- input$plotInteractFacetNames
        
        if (length(PKG.ENV$PLOT_INTERACT_FACET_NAMES) == 0) {
            PKG.ENV$NO_FACET_NAMES <- TRUE
        } else {
            PKG.ENV$NO_FACET_NAMES <- FALSE
        }
        
        output$results_interact_plot <- renderPlot({
            withProgress(message="Rendering plot (interaction estimates) in UI", {
                .amceApp.plotInteractResults(PKG.ENV$LAST_RESULTS, 
                                             ci=PKG.ENV$PLOT_INTERACT_CI,
                                             ylim=PKG.ENV$PLOT_INTERACT_YLIM_SELECTED,
                                             textsize=PKG.ENV$PLOT_INTERACT_TEXT_SIZE_SELECTED,
                                             facetnames=PKG.ENV$PLOT_INTERACT_FACET_NAMES,
                                             font.family=PKG.ENV$PLOT_INTERACT_FONT_FAMILY)       
            })
        })  
    }, ignoreNULL=FALSE)
    
}

#' @import ggplot2
#' @import lmtest
#' @import Matrix
#' @import sandwich
#' @import survey
#' @importFrom methods Quote
#' @importFrom stats coefficients formula lm model.matrix pnorm qnorm 
#' @importFrom stats quantile relevel terms var vcov
#' @importFrom stats as.formula reshape
#' @importFrom utils read.csv packageDescription View data
#' @importFrom grDevices pdf dev.off
#' @export amce
#' @export makeDesign
#' @export plot.amce
#' @export print.summary.amce
#' @export read.qualtrics
#' @export read.with.qualtRics
#' @export summary.amce
## cjoint: An R Package for estimating Average Marginal Component-specific Effects from conjoint survey experiments
## August, 2017
## Elissa Berwick, Jens Hainmueller, Daniel Hopkins, Anton Strezhnev, Teppei Yamamoto

#############################
# Clustered standard errors
#############################

cluster_se_glm <- function(model, cluster){
    
    #  Drop unused cluster indicators, if cluster var is a factor
    if (inherits(cluster, "factor")) {
        cluster <- droplevels(cluster)
    }
    
    if (nrow(model.matrix(model)) != length(cluster)) {
        stop("check your data: cluster variable has different N than model - you may have observations with missing data") 
    }
    
    M <- length(unique(cluster))
    N <- length(cluster)           
    K <- model$rank

    dfc <- (M/(M - 1)) * ((N - 1)/(N - K))
    uj  <- apply(estfun(model), 2, function(x) tapply(x, cluster, sum))
    rcse.cov <- dfc * sandwich(model, meat. = crossprod(uj)/N)
    return(rcse.cov)
}

#########################################################################
## function for removing ALL punctuation, symbols, and spaces from string
## from elements of vector "vec"
########################################################################

clean.names <- function(str) {
    #split components of interactions
    x <- strsplit(str,":")[[1]]
    #and apply cleaning separately, removing any punctuation (P), symbols (S), and separators (Z)
    x <- gsub("[\\p{P}\\p{S}\\p{Z}]","",x,perl=T)
    #re-attach
    paste(x,collapse=":")
}
clean.names <- Vectorize(clean.names,vectorize.args=("str"),USE.NAMES = F)

#########################
## amce function
#########################

amce <- function(formula, data, design="uniform", respondent.varying = NULL, subset=NULL, respondent.id=NULL, cluster=TRUE, na.ignore=FALSE, weights=NULL, baselines = NULL) {
    
###### Formula and Variables

    #we will split the formula into separate lists
    #unique_vars = all VARIABLES in formula
    #respondent_vars = VARIABLES varying by respondent
    #profile_vars = VARIABLES varying by profile
    #orig_effects = all EFFECTS in formula 
    #profile_effects = profile varying EFFECTS in formula
    #user inputted names for above end with "_user"
    
##### Parse formula, clean variable and input names

    formula_user <- formula
    #all variables in formula
    formula_char_user <- all.vars(formula)
    #lists with original names of variables and levels
    user_names <- list()
    user_levels <- list()
    for (char in formula_char_user) {
        user_names[[clean.names(char)]] <- char
        if (inherits(data[[char]], "factor")) {
            old_names <- names(user_levels)
            user_levels <- c(user_levels,levels(data[[char]]))
            new_names <- sapply(clean.names(levels(data[[char]])),function(x) paste(clean.names(char),x,sep=""))
            names(user_levels) <- c(old_names,new_names)
        }
    }

    #make sure no duplicates after spaces and special characters removed
    formula_char <- clean.names(formula_char_user)    
    #if this makes for non-unique names, stop
    if(length(unique(formula_char)) != length(formula_char)) {
        stop("Error: Variable names must be unique when whitespace and meta-characters are removed. Please rename.")
    }
        
    #separate dependent and independent variables and clean
    y_var <- clean.names(formula_char_user[1])
    #identify ALL original effects; will add in missing base terms automatically
    orig_effects <- clean.names(attr(terms(formula_user),"term.labels"))
    #formula sorting part I: sort non-interaction terms and put them first
    orig_effects <- c(sort(orig_effects[!grepl(":",orig_effects)]), orig_effects[grepl(":",orig_effects)])
    vars_plus <- paste(orig_effects,collapse = " + ")
    form <- formula(paste(c(y_var,vars_plus),collapse = " ~ "))
    orig_effects <- attr(terms(form),"term.labels")

    #find missing base terms
    full_terms <- attr(terms(formula(paste(y_var,paste(sapply(orig_effects,function(x) gsub(":","*",x)),collapse=" + "),sep=" ~ "))),"term.labels")
    # add in any missing base terms for interactions
    missing_terms <- full_terms[!is.element(full_terms,orig_effects)]
    if (length(missing_terms > 0)) {
        orig_effects <- c(orig_effects,missing_terms)
        warning("Missing base terms for interactions added to formula")
    }

    #formula sorting redux: sort non-interaction terms and put them first
    orig_effects <- c(sort(orig_effects[!grepl(":",orig_effects)]), orig_effects[grepl(":",orig_effects)])
    #combine with "+"
    vars_plus <- paste(orig_effects,collapse = " + ")
    #then remake formula 
    form <- formula(paste(c(y_var,vars_plus),collapse = "~"))
    orig_effects <- clean.names(attr(terms(form),"term.labels"))

    #unique variables only (no interactions)
    unique_vars <- clean.names(rownames(attr(terms(form),"factor"))[-1])
    #respondent variables
    respondent_vars <- clean.names(respondent.varying)
    #profile variables
    profile_vars <- unique_vars[!is.element(unique_vars,respondent_vars)]

    #identify the REQUESTED profile effects and respondent effects (if any)
    if (length(respondent_vars) > 0) {
        #identify profile only effects
        profile_effects <- unlist(sapply(orig_effects,USE.NAMES = F,function(x) {
            y <- strsplit(x,":")[[1]]
            if (!any(is.element(y,respondent_vars))) x
        }))
        #terms containing a respondent var
        resp_only <- unlist(sapply(orig_effects,USE.NAMES = F, function(x) {
            y <- strsplit(x,":")[[1]]
            if(any(is.element(y,respondent_vars))) x
        }))
        #things that respondent vary is interacted with
        resp_mod <- unlist(sapply(resp_only,USE.NAMES = F,function(x) {
            y <- strsplit(x,":")[[1]]
            vars <- y[!is.element(y,respondent_vars)]
            if (length(vars) > 0) paste(vars,collapse = ":")
        }))
        resp_effects <- c(resp_mod,resp_only)  
    } else {
        profile_effects <- orig_effects
        resp_effects <- NULL
    }

    ### Extra name cleaning
    #cleaning additional inputs
    if (!is.null(respondent.id)) respondent.id <- clean.names(respondent.id)
    if (!is.null(weights)) weights <- clean.names(weights)
    if (!is.null(baselines)) {
        names(baselines) <- clean.names(names(baselines))
        baselines <- lapply(baselines,function(x) clean.names(x))
    }

    #cleaning within data 
    colnames(data) <- clean.names(colnames(data))
    ## data <- data.frame(data) #in case of dplyr etc.
    var_to_check_levels <- unique(c(formula_char_user, respondent_vars, profile_vars, profile_effects, orig_effects))
    
    for (var in var_to_check_levels) {
        if (inherits(data[[var]], "factor")) {
            clean.labels <- clean.names(levels(data[[var]]))
            if (length(unique(clean.labels)) != length(clean.labels)) {
                stop (paste("Error: levels of variable", var, "are not unique when whitespace and meta-characters are removed. Please rename."))
            }
            data[[var]] <- factor(data[[var]],levels=levels(data[[var]]), labels=clean.names(levels(data[[var]])))
        }
    }
  
#######  Sanity Checks Re: Data

    # Are variables in data?
    for(var in formula_char) {
        if(!(var %in% colnames(data))) {
            stop(paste("Error:", var, "not in 'data'"))
        }
    }

    # Make sure non-respondent varying are factors
    for (var in profile_vars) {
        if (!inherits(data[[var]], "factor")) {
            data[[var]] <- as.factor(data[[var]])
            warning(paste(c("Warning: ",var," changed to factor"),collapse=""))
        }
    }
  
    # Is there missing data?
    if(na.ignore == FALSE){
      for(variab in formula_char){
        if (sum(is.na(data[[variab]])) != 0 ){
          stop(paste("Error:", variab, "has missing values in 'data'"))
        }
      }
    }

    # Is the respondent varying characteristic even in the formula obj?
    if (!is.null(respondent_vars)) {
        for (var in respondent_vars) {
            found <- 0
            for (formulavars in formula_char) {
                if (var == formulavars) {
                    found <- 1
                }
            }
            if (found == 0) {
                stop(paste("Error:", var, "is specified in respondent.varying, but is not in the formula"))
            }
        }
    }

    # Check whether outcome variable is a binary 0-1 or numeric
    if (!is.numeric(data[[y_var]]) & !is.integer(data[[y_var]])) {
        stop(paste("Error:", y_var, "is not numeric or integer"))
    }

    # Are the user-supplied desired baselines in the data?
    if (!is.null(baselines)) {
        for(var in names(baselines)) {
            if(!(baselines[[var]] %in% data[[var]])) {
                stop(paste("Error: user supplied baseline",baselines[[var]],"is not a level of",var))
            }
        }      
    }

#######  Sanity Checks Re: clustering

    # If respondent.id is NULL make sure cluster is FALSE
    if(is.null(respondent.id) & cluster == TRUE) {
        warning("respondent.id is NULL - setting cluster to FALSE. Please specify a respondent.id variable if you want to estimate clustered standard errors")
        cluster <- FALSE
    }

    # If (cluster = TRUE), make sure respondent.id is specified and in data
    if (cluster == TRUE) {
        if (is.null(respondent.id)) {
            stop('Error: Must specify a respondent.id if cluster = TRUE')
        } else if (!(respondent.id %in% colnames(data))) {
            stop('Error: respondent.id not in data')
        }
    }

######## Sanity checks re: weights

    if(!is.null(weights)) {
        #is the weight var in the data matrix?
        if (!weights %in% colnames(data)) {
            stop('Error: weights not in data')
        }
        #are weights uniform? if so turn off weights
        if(length(unique(data[[weights]])) == 1) {
            weights <- NULL 
        }
    }

##### Sanity Checks re: design matrix
    
    # If design is already conjointDesign object, proceed to relevant sanity checks
    if (inherits(design, "conjointDesign")) {
        # Remove whitespaces etc from dimension names of design array 
        names(dimnames(design$J)) <- clean.names(names(dimnames(design$J)))
        dimnames(design$J) <- lapply(dimnames(design$J),function(x) clean.names(x))
        
        #and design dependencies
        names(design$depend) <- clean.names(names(design$depend))
        design$depend <- lapply(design$depend,function(x) clean.names(x))  
        #Now check to make sure profile varying attributes are in conjointDesign
        for (eff in profile_vars) {   
            if (!(eff %in% names(dimnames(design$J)))) {
                stop(paste("Error:", eff, "not in 'design' object"))
            }
        }      
        #Check to make sure conjointDesign attributes are in data and level names match
        for (eff in names(dimnames(design$J))) {
            if (!(eff %in% colnames(data))){
                stop(paste("Error: attribute", eff, "in 'design' object is not in 'data'"))
            } else {
        # Check all level names for the attribute in dataset appear in design
                for (lev in clean.names(levels(as.factor(data[[eff]])))) {
                    if (!(lev %in% dimnames(design$J)[[eff]])) {
                        #print(paste0('checking level ', lev))
                        stop(paste("Error: factor level", lev, "of attribute", eff, "not in 'design' object"))
                    }
                }
            }
        }
        
        depend_to_check <- NULL
        for (var in names(design$depend)) {
          depend_to_check <- c(depend_to_check,design$depend[[var]])
        }
        ## only check any variables that weren't already checked
        depend_to_check <- depend_to_check[!depend_to_check %in% var_to_check_levels]
        ## Check that all dependencies have unique levels
        for (var in depend_to_check){
          if (inherits(data[[var]], "factor")) {
            clean.labels <- clean.names(levels(data[[var]]))
            if (length(unique(clean.labels)) != length(clean.labels)) {
              stop (paste("Error: levels of variable", var, "used as a dependency, are not unique when whitespace and meta-characters are removed. Please rename."))
            }
            data[[var]] <- factor(data[[var]],levels=levels(data[[var]]), labels=clean.names(levels(data[[var]])))
          }
        }
    } else if (design == "uniform") {    
        # else if design == "uniform", create J-dimensional array 
        design <- list()        
        # Determine dimensions
        # And create J matrix with uniform probabilities across all vars
        design.dim <- vector(length=length(profile_vars))
        dim_list <- list()
        for (i in 1:length(profile_vars)) {
            dim_list[[i]] <- levels(factor(data[[profile_vars[i]]]))
            design.dim[i] <- length(dim_list[[i]])
        }
        names(dim_list) <- profile_vars
        design$J <- array(1/prod(design.dim), dim=design.dim, dimnames=dim_list)
        design$depend <- compute_dependencies(design$J)       
    } else {
         #if neither uniform nor conjointDesign, error
        stop("Design object must be a valid character string 'uniform' or a conjointDesign object")   
    }

####### Subsetting data    

    if (is.null(subset)) {
        data <- data 
    } else {
        if (inherits(subset, "logical")) {
            if (length(subset) == nrow(data)) {
                data <- subset(data, subset) 
            } else {
                warning("Warning: invalid argument to 'subset' - must be the same length as the number of rows in data")
            }
        } else {
            warning("Warning: invalid argument to 'subset' - must be a logical")
        }
    }

###### Adjust baselines if given    

    if (!is.null(baselines)) {
        for (var in names(baselines)) {
            data[[var]] <- factor(data[[var]])
            data[[var]] <- relevel(data[[var]], baselines[[var]])
        } 
    }

####### Adding relevant interaction terms to model

    #If there are any dependencies-- only for profile-varying!
    if(any(profile_vars %in% names(design$depend))) {
        #initialize full interaction set
        depend_vars <- c()     
        #loop over effects with dependencies
        for(eff in profile_vars[profile_vars %in% names(design$depend)]) {
            #initialize interaction set for given variable
            inter <- c()
            #identify higher order occurences of variable
            #make sure it's just that variable, not followed or begun by "_"
            eff_all <- grep(paste(c(":",eff,"(?!_)","|",eff,":(?<!_)"),collapse=""),
                            orig_effects,value=T,perl=T)
            #if you find some, break up, sort and replace ":" with "*"
            if (length(eff_all) > 0) {
                eff_all <- sapply(strsplit(eff_all,":"),function(x) paste(sort(x),collapse="*"))
            }
            #combine with lower order (main effect)
            eff_all <- c(eff,eff_all)
            #for each occurrence, create interaction
            inter <- sapply(eff_all,USE.NAMES = F,function(x) {
                #get conditioning set
                T_r <- design$depend[[eff]]
                #make factors
                for (t in T_r){
                    data[[t]] <- as.factor(data[[t]])
                }
                #combine name and dependency
                T_r_d <- c(x,T_r)
                #make interaction term
                paste(T_r_d,collapse="*")
            })
            #add to list
            depend_vars <- c(depend_vars,inter)
        }      
        #drop repeats
        depend_vars <- unique(depend_vars)
        #add to formula
        form_full <- formula(paste(c(form,depend_vars),collapse = " + "))
    } else {
      form_full <- form
  }

    #all variables to be run
    all_run_vars <- attr(terms(form_full),"term.labels")
   #formula sorting redux: sort non-interaction terms and put them first
    all_run_vars <- c(sort(all_run_vars[!grepl(":",all_run_vars)]), all_run_vars[grepl(":",all_run_vars)])
    #combine with "+"
    vars_plus <- paste(all_run_vars,collapse = " + ")
    #then remake formula 
    form_full<- formula(paste(c(y_var,vars_plus),collapse = "~"))
    all_run_vars <- attr(terms(form_full),"term.labels")
    
####### If there are respondent varying terms, split into two formulas
######## One contains only profile effects
######## Second is full formula

    if (length(respondent_vars) > 0) {
        ### profile only formula
        #remove those involving respondent things
        prof_only <- unlist(sapply(all_run_vars,function(x) {
            y <- clean.names(strsplit(x,":")[[1]])
            if(!any(is.element(y,respondent_vars))) x
        }))
        prof_only_plus <- paste(prof_only,collapse = " + ")
        #formula with profile only
        form_prof <- paste(all.vars(form_full)[1],prof_only_plus,sep=" ~ ")
        form_prof <- formula(form_prof)        
    } else {
        #otherwise use full formula
        form_prof <- form_full
    }
    all_prof <- clean.names(attr(terms(form_prof),"term.labels"))
    all_run_vars <- clean.names(all_run_vars)
    if (any(!is.element(all_prof,all_run_vars))) {
        warning("Warning: mismatch of term names between full formula and profile formula")
    }
    
####### Running OLS

    #run model(s)-- if using weights, use tools from "survey" package
    #otherwise run usual lm function
    if (is.null(weights)) {
        lin.mod.prof <- lm(form_prof, data=data)
        if (length(respondent_vars) > 0) {
            lin.mod.full <- lm(form_full, data=data)
        } else {
            lin.mod.full <- NULL
        }
    } else {
        if (cluster) {
            out.design <- svydesign(ids = data[[respondent.id]], weights = data[[weights]], data=data)
        } else {
            out.design <- svydesign(ids = ~0, weights = data[[weights]], data=data)
        }
        lin.mod.prof <- svyglm(form_prof, data=data, design = out.design)
        if (length(respondent_vars) > 0) {
            lin.mod.full <- svyglm(form_full, data=data, design = out.design)
        } else {
            lin.mod.full <- NULL
        }
    }

    #If there's missing data - flag it
    #if (na.ignore == TRUE & !is.null(lin.mod.full$na.action)) {
     #   stop(paste("Error: Observations with missing data in 'data'"))
    #}
  
    #Get sample size
    sample_size_prof <- length(lin.mod.prof$residuals)
    if (length(respondent.varying) > 0) {
        sample_size_full <- length(lin.mod.full$residuals)
    } else {
        sample_size_full <- NULL
    }
    
    #Compute vcov of OLS
    if (is.null(weights) & cluster == TRUE) {
        #clusters but no weights
        vcov_mat_prof <- cluster_se_glm(lin.mod.prof, data[[respondent.id]])
        if (length(respondent.varying) > 0) {
            vcov_mat_full <- cluster_se_glm(lin.mod.full, data[[respondent.id]])
        } else {
            vcov_mat_full <- NULL
        }
    } else if (!is.null(weights)) {
        #weights with or without cluster
        vcov_mat_prof <- vcov(lin.mod.prof)
        if (length(respondent_vars) > 0) {
            vcov_mat_full <- vcov(lin.mod.full)
        } else {
            vcov_mat_full <- NULL
        }        
    } else {
    #Not clustered or weighted
        vcov_mat_prof <- vcovHC(lin.mod.prof,type="HC2")
        if (length(respondent.varying) > 0) {
            vcov_mat_full <- vcovHC(lin.mod.full,type="HC2")
        } else {
            vcov_mat_full <- NULL
        }
    }

    ## function for fixing variance-covariance matrices
    ## input in vcov output from lm and a matrix of varprobs
    ## draws on matrix package
    fix.vcov <- function(varprob,vcov) {
        if (!requireNamespace("Matrix", quietly = TRUE)){
          stop("Matrix package needed for this function to work. Please install it.",
               call. = FALSE)
        }
        #designate inputs as sparse
        varprob2 <- Matrix(varprob,sparse=TRUE)
        vcov2 <- Matrix(vcov,sparse=TRUE)
        #calculate and add in single sum corrections
        fix1 <- varprob2 %*% vcov2 + t(varprob2 %*% vcov2) + vcov2
        #vcov matrix as vector
        vcov_vector <- matrix(as.vector(vcov2),ncol=1,nrow=length(as.vector(vcov2)))
        #use to multiply combinations of varprobs by covariances and sum
        weighted_covs <- kronecker(varprob2,varprob2) %*% vcov_vector
        #make corrections into a matrix
        weighted_covs <- matrix(weighted_covs,nrow=nrow(vcov), ncol=ncol(vcov))
        #add to single sum corrections
        fix2 <- fix1 + weighted_covs
        #return as normal matrix
        out <- matrix(fix2,ncol=ncol(vcov),nrow=nrow(vcov))
        colnames(out) <- rownames(out) <- rownames(vcov)
        return(out)
    }

######### Extract Effects from the profile-vars only linear model

# proposed nomenclature here:
# effect = attribute in question, which has "effect levels"
# depends = attributes it depends on, each of which has "depend levels"

    #Make R CMD check happy
    J_baseline <- NULL
    J_effect <- NULL
    
    #before we start, make a blank call to the design array J
    J_call <- Quote(design$J[])
    J_call <- J_call[c(1, 2, rep(3, length(dim(design$J))))]

    #warnings counter
    warn_i <- 0
    
############## loop over unique profile vars only (AMCE and ACIE); interactions etc. below

    #blank list for output
    estimates <- list()
    #re-sort profile effects
    profile_effects <- c(sort(profile_effects[!grepl(":",profile_effects)]), profile_effects[grepl(":",profile_effects)])
    #initialize list for weighted cross-terms
    covariance_list <- list()
    #blank matrix of var probs
    varprob_mat <- matrix(0,nrow(vcov_mat_prof),ncol(vcov_mat_prof))
    colnames(varprob_mat) <- rownames(varprob_mat) <- colnames(vcov_mat_prof)
        
    for(i in 1:length(profile_effects)) {

        #split into sections if it's an interaction
        substrings <- strsplit(profile_effects[i], "[:*]", perl=TRUE)[[1]]

        #administrative loop to find levels
        all_levels <- list()
        all_levels_coefs <- list()
        for(effect in substrings) {
            #get all level names and coefficient names-- sans baseline!!!
            all_levels[[effect]] <- levels(data[[effect]])[-1]
            all_levels_coefs[[effect]] <- sapply(all_levels[[effect]], function(x) {
                paste(c(effect,x), collapse="")
            })
        }
  
        #find all combinations of level names-- add as FIRST column
        levels <- expand.grid(all_levels, stringsAsFactors = FALSE)
        #make level combos in first column
        levels <- cbind(apply(levels,1,function(x) paste(x,collapse=":")),levels)
        colnames(levels) <- c("name",substrings)

        #and all combinations of actual coefficient names
        coefs <- expand.grid(all_levels_coefs, stringsAsFactors = FALSE)
        coefs <- apply(coefs,1,function(x) paste(x,collapse=":"))
       
        # Initialize the results 
        results <- matrix(nrow=2, ncol = nrow(levels))
        if (length(substrings) > 1) {
            rownames(results) <- c("ACIE", "Std. Error")
        } else {
            rownames(results) <- c("AMCE", "Std. Error")
        }
        colnames(results) <- coefs
        results[2,] <- NA

         #### find extra times when this effect is mentioned
        all_depends <- unlist(sapply(all_prof,USE.NAMES = F,function(x) {
            y <- strsplit(x,":")[[1]]
            if (all(is.element(substrings,y))) x
        }))
        # remove the actual term
        all_depends <- all_depends[-is.element(all_depends,profile_effects[i])]

 #### loop over every combination of levels of component effects
        for(j in 1:nrow(levels)) {
                
            #figure out which level of inter we're doing
            ## effect_level <- as.character(levels[j,1])
            effect_level_coef <- coefs[j]
            #get its beta 
            initial_beta <- coefficients(lin.mod.prof)[effect_level_coef]
 
            #if interaction,make sure there is baseline support for this level combination
            if (!is.na(initial_beta) & length(substrings) > 1) {
                for (effect1 in substrings) {
                    #get effect base
                    effect_base1 <- levels(data[[effect1]])[1]
                    #subset data to that level
                    base.subset <- data[which(data[[effect1]] == effect_base1),]                       
                    #loop over other profile-varying vars in interaction to subset further
                    for(effect in substrings[!(substrings %in% effect1)]) {
                        base.subset <- base.subset[which(base.subset[[effect]] == as.character(levels[j,effect])),]
                    }                        
                    #if there's no support left, change beta and var to NA
                    if (nrow(base.subset) == 0) {
                        initial_beta <- NA
                        #and give a warning that you had to do it
                        warn_i <- warn_i + 1
                    }
                }
            }
               
            # If initial_beta and initial_variance are not NA (are valid level combination)
            # and there are dependent variables to incorporate
            if (!is.na(initial_beta) & length(all_depends) > 0) {
                
                #get the slice of design array J associated with baseline and inter level
                J_effect_call <- J_base_call <-  J_call
                for(effect in substrings) {
                    #identify its baseline and modify baseline call accordingly
                    base <- levels(data[[effect]])[1]
                    effect_index <- which(names(dimnames(design$J)) == effect)
                    J_base_call[effect_index + 2] <- base                            
                    #identify level of each effect and modify inter call accordingly
                    level <- levels[j,effect]
                    J_effect_call[effect_index + 2] <- level
                }
                eval(call("<-", Quote(J_baseline), J_base_call))
                eval(call("<-", Quote(J_effect), J_effect_call))

                #### loop over dependencies for all components of interaction
                for(k in 1:length(all_depends)) {

                    #attribute effect is dependent on
                    depend <- all_depends[[k]]
                    #figure out what levels of what variables are involved
                    substrings_d <- strsplit(depend,":")[[1]]
                    substrings_d <- substrings_d[!is.element(substrings_d,substrings)]
                    all_depend_coefs <- list()
                    for (sub in substrings_d) {
                        all_depend_coefs[[sub]] <-  sapply(levels(data[[sub]]), function(x) paste(c(sub,x),collapse=""))
                    }
                    all_depend_levels <- expand.grid(all_depend_coefs)
                    substrings_l <- strsplit(effect_level_coef,":")[[1]]
                    for (l in length(substrings_l):1) {
                        all_depend_levels <- cbind(rep(substrings_l[l], nrow(all_depend_levels)), all_depend_levels)
                    }
                    colnames(all_depend_levels)[1:length(substrings_l)] <- substrings
                    all_depend_levels <- all_depend_levels[sort(colnames(all_depend_levels))]
                    all_depend_level_coefs <- apply(all_depend_levels, 1, function(x) paste(x,collapse=":"))
                
                    #baseline support for depend attribute level 
                    if (!(is.null(dim(J_baseline)))){
                        baseline_support <- apply(J_baseline,substrings_d,sum)
                    } else {
                        baseline_support <- J_baseline
                    }
                    baseline_support[baseline_support != 0] <- 1

                    #probs for depend attribute levels WITH baseline support
                    if (!is.null(dim(J_effect))) {
                        joint_prob <- apply(J_effect, substrings_d, sum)*baseline_support
                    } else {
                        joint_prob <- J_effect*baseline_support
                    }
                    #make it a vector
                    joint_prob <- as.vector(joint_prob)
                    names(joint_prob) <- all_depend_level_coefs

                    all_depend_level_coefs <- all_depend_level_coefs[!is.na(lin.mod.prof$coefficients[all_depend_level_coefs])]
                    varprob_mat[effect_level_coef,all_depend_level_coefs] <- as.numeric(joint_prob[all_depend_level_coefs])/as.numeric(sum(joint_prob))
                    
                    ##### if all_depend_level_coefs is 1 or longer 
                    if (length(all_depend_level_coefs)) {
                        #calculate probabilities for this effect and depend level 
                        var_prob <- joint_prob[all_depend_level_coefs]
                        var_prob <- as.numeric(var_prob)/as.numeric(sum(joint_prob))
                        # add weighted beta to initial_beta
                        depend_betas <- lin.mod.prof$coefficients[all_depend_level_coefs]
                        initial_beta <- sum(initial_beta,var_prob*depend_betas,na.rm=T)
                    }
                    
                } #end for loop over different dependent attributes
            } #end if has valid beta, var, dependencies

            # Store effect and standard error estimates
            results[1,j] <- initial_beta
                
        } #end for loop over all level combinations

         # combine estimates + SEs into single matrix - store in list
        estimates[[profile_effects[i]]] <- results
            
    } #end for loop over profile effects      

    ### fix var-cov matrix
    vcov_prof <- suppressMessages(fix.vcov(varprob_mat,vcov_mat_prof))
    #write in adjusted variances to output matrix
    for (i in 1:length(estimates)) {
        coef_names <- colnames(estimates[[i]])
        variances <- sqrt(diag(vcov_prof)[coef_names])
        estimates[[i]][2,] <- ifelse(is.na(estimates[[i]][1,]),NA,variances)
    }

    #determine term names for profile effects (no depends) to keep
    profile_effects_plus <- paste(profile_effects,collapse=" + ")
    profile_effects_form <- formula(paste(c(y_var,profile_effects_plus),collapse = " ~ "))   
    profile_effects_terms <- colnames(model.matrix(profile_effects_form,data))   
    profile_effects_terms <- profile_effects_terms[profile_effects_terms %in% colnames(vcov_mat_prof)]
    vcov_prof <- vcov_prof[profile_effects_terms,profile_effects_terms]

######### Extract Effects from the full model (if have respondent interactions)

# proposed nomenclature here:
# effect = attribute in question, which has "effect levels"
# depends = attributes it depends on, each of which has "depend levels"
# inters = attributes in interaction terms each of which has "inter levels"

 #if there are any respondent effects
    if (length(respondent_vars) > 0) {

        #blank list for output
        conditional.estimates <- list()
        #re-sort respondent effects
        resp_effects <- c(sort(resp_effects[!grepl(":",resp_effects)]), resp_effects[grepl(":",resp_effects)])
        #initialize list for weighted cross-terms
        covariance_list <- list()
        #blank matrix for var probs
        varprob_mat <- matrix(0,nrow(vcov_mat_full),ncol(vcov_mat_full))
        colnames(varprob_mat) <- rownames(varprob_mat) <- colnames(vcov_mat_full)

        #loop over respondent-related effects
        for (i in 1:length(resp_effects)) {
            
            #split into component effects, if interaction
            substrings <- strsplit(resp_effects[i], "[:*]", perl=TRUE)[[1]]

            ## start by finding levels
            #administrative loop over components of interaction
            all_levels <- list()
            all_levels_coefs <- list()
            for(effect in substrings) {
                #if it's not a factor, only has the 1 "level" and coefficient name stays
                if (!inherits(data[[effect]] ,"factor")) {
                    all_levels[[effect]] <- effect
                    all_levels_coefs[[effect]] <- effect
                } else {
                    #if it is a factor, get all level names and coefficient names-- sans baseline!!!
                    all_levels[[effect]] <- levels(data[[effect]])[-1]
                    all_levels_coefs[[effect]] <- sapply(all_levels[[effect]],
                                                         function(x) {
                                                             paste(c(effect,x), collapse="")
                                                         })
                }
            }
                
            #find all combinations of level names-- add as FIRST column
            levels <- expand.grid(all_levels, stringsAsFactors = FALSE)
            levels <- cbind(apply(levels,1,function(x) paste(x,collapse=":")),levels)
            colnames(levels) <- c("name",substrings)

            #and all combinations of coefficient names
            coefs <- expand.grid(all_levels_coefs, stringsAsFactors = FALSE)
            coefs <- apply(coefs,1,function(x) paste(x,collapse=":"))
       
            # Initialize the results 
            results <- matrix(nrow=2, ncol = nrow(levels))
            rownames(results) <- c("Conditional Estimate", "Std. Error")
            colnames(results) <- coefs
            #write NA to SE row to start
            results[2,] <- NA

            #### find extra times when this effect is mentioned in full formula
            # only if anything related to profile var is involved
            if (any(substrings %in% profile_vars)) {
                all_depends <- unlist(sapply(all_run_vars,USE.NAMES = F,function(x) {
                    y <- strsplit(x,":")[[1]]
                    if (all(is.element(substrings,y))) x
                }))
                # remove the actual term
                all_depends <- all_depends[!is.element(all_depends,resp_effects[i])]
                # remove any that involve any other respondent varying terms
                resp.other <- respondent_vars[!is.element(respondent_vars,substrings)]
                all_depends <- unlist(sapply(all_depends,function(x) {   
                    sub_depends <- strsplit(x,":")[[1]]
                    if (all(!is.element(sub_depends,resp.other))) x
                }))
            } else {
                #no profile vars, no depends
                all_depends <- c()
            }
                              
            #### loop over every combination of levels of component effects
            for(j in 1:nrow(levels)) {
                
                #figure out which level of inter we're doing
                effect_level_coef <- coefs[j]
                #get its beta 
                initial_beta <- coefficients(lin.mod.full)[effect_level_coef]

                #make sure there is baseline support for this level combination
                if (!is.na(initial_beta)) {                   
                    for (effect1 in substrings[substrings %in% profile_vars]) {
                        #get effect base
                        effect_base1 <- levels(data[[effect1]])[1]
                        #subset data to that level
                        base.subset <- data[which(data[[effect1]] == effect_base1),]                       
                        #loop over other profile-varying vars in interaction to subset further
                        for(effect in substrings[substrings %in% profile_vars][!(substrings[substrings %in% profile_vars] %in% effect1)]) {
                            base.subset <- base.subset[which(base.subset[[effect]] == as.character(levels[j,effect])),]
                        }                        
                        #if there's no support left, change beta and var to NA
                        if (nrow(base.subset) == 0) {
                            initial_beta <- NA
                            #and give a warning that you had to do it
                            warn_i <- warn_i + 1
                        }
                    }
                }
               
                # If initial_beta and initial_variance are not NA and there are depends
                # proceed to add to beta and var
                if (!is.na(initial_beta) & length(all_depends) > 0) {

                    #get the slice of design array J associated with baseline and inter level
                    #profile variables only!
                    J_effect_call <- J_base_call <-  J_call
                    for(effect in substrings[substrings %in% profile_vars]) {
                        #identify its baseline and modify baseline call accordingly
                        base <- levels(data[[effect]])[1]
                        effect_index <- which(names(dimnames(design$J)) == effect)
                        J_base_call[effect_index + 2] <- base                           
                        #identify level of each effect and modify inter call accordingly
                        level <- levels[j,effect]
                        J_effect_call[effect_index + 2] <- level
                    }
                    eval(call("<-", Quote(J_baseline), J_base_call))
                    eval(call("<-", Quote(J_effect), J_effect_call))
  
                    #### loop over dependencies for all components of effect
                    for(k in 1:length(all_depends)) {

                        #attribute effect is dependent on
                        depend <- all_depends[[k]]
                       #figure out what levels of what variables are involved
                        substrings_d <- strsplit(depend,":")[[1]]
                        substrings_d <- substrings_d[!is.element(substrings_d,substrings)]
                        all_depend_coefs <- list()
                        for (sub in substrings_d) {
                            all_depend_coefs[[sub]] <-  sapply(levels(data[[sub]]), function(x) paste(c(sub,x),collapse=""))
                        }
                        all_depend_levels <- expand.grid(all_depend_coefs)
                        substrings_l <- strsplit(effect_level_coef,":")[[1]]
                        for (l in length(substrings_l):1) {
                            all_depend_levels <- cbind(rep(substrings_l[l], nrow(all_depend_levels)), all_depend_levels)
                        }
                        colnames(all_depend_levels)[1:length(substrings_l)] <- substrings
                        ####put terms together in proper order
                        all_depend_levels <- all_depend_levels[sort(colnames(all_depend_levels))]
                        all_depend_level_coefs <- apply(all_depend_levels, 1, function(x) paste(x,collapse=":"))
                            
                       #baseline support for depend attribute level in inter
                        if (!(is.null(dim(J_baseline)))) {
                            baseline_support <- apply(J_baseline,substrings_d,sum)
                        } else {
                            baseline_support <- J_baseline
                        }
                        baseline_support[baseline_support != 0] <- 1

                        #support for depend attribute levels WITH baseline support
                        if (!is.null(dim(J_effect))) {
                            joint_prob <- apply(J_effect, substrings_d, sum)*baseline_support
                        } else {
                            joint_prob <- J_effect*baseline_support
                        }
                        #make it a vector
                        joint_prob <- as.vector(joint_prob)
                        names(joint_prob) <- all_depend_level_coefs

                        all_depend_level_coefs <- all_depend_level_coefs[!is.na(lin.mod.full$coefficients[all_depend_level_coefs])]
                        varprob_mat[effect_level_coef,all_depend_level_coefs] <- as.numeric(joint_prob[all_depend_level_coefs])/as.numeric(sum(joint_prob))
                        
                        ## If there are non-null # of depend-level-coefs
                        if (length(all_depend_level_coefs)) {
                           #calculate probabilities for this effect and depend level 
                            var_prob <- joint_prob[all_depend_level_coefs]
                            var_prob <- as.numeric(var_prob)/as.numeric(sum(joint_prob))
                            # add weighted beta to initial_beta
                            depend_betas <- lin.mod.full$coefficients[all_depend_level_coefs]
                            initial_beta <- sum(initial_beta,var_prob*depend_betas, na.rm=T)
                        }        
                        
                    } #end for loop over different dependent attributes

                } #end if initial beta and var are NA, has depends
                
                # Store effect estimates
                results[1,j] <- initial_beta

            } #end for loop over all level combinations

            # combine estimates + SEs into single matrix - store in list
            conditional.estimates[[resp_effects[i]]] <- results
            
        } #end for loop over respondent related effects      

        ##fix variance-covariance matrix
        vcov_resp <- suppressMessages(fix.vcov(varprob_mat,vcov_mat_full))
        #write in adjusted variances
        for (i in 1:length(conditional.estimates)) {
            coef_names <- colnames(conditional.estimates[[i]])
            variances <- sqrt(diag(vcov_resp)[coef_names])
            conditional.estimates[[i]][2,] <- ifelse(is.na(conditional.estimates[[i]][1,]), NA, variances)
        }

        ###terms to keep (no depends)
        resp_effects_plus <- paste(resp_effects,collapse=" + ")
        resp_effects_form <- formula(paste(c(y_var,resp_effects_plus),collapse = " ~ "))
        resp_effects_terms <- colnames(model.matrix(resp_effects_form,data))
        resp_effects_terms <- resp_effects_terms[resp_effects_terms %in% colnames(vcov_mat_full)]
        vcov_resp <- vcov_resp[resp_effects_terms,resp_effects_terms]
        
    } #end if there are any respondent related effects
      
############  create conjoint object for output
    
    output <- list()
    class(output) <- c("amce")
    
    #saving things for unconditional estimates
    output$estimates <- estimates
    #saving profile attributes
    output$attributes <- dimnames(design$J)
    #save original profile-only vcov matrix
    output$vcov.prof <- vcov_prof
    #save sample size used for unconditional estimates
    output$samplesize_prof <- sample_size_prof
    #save style edited formula (no depends)
    output$formula <- form

    #final warning tally
    if (warn_i > 0) {
        warning(paste("Warning: ",warn_i," interaction levels lacked support at baseline, effects undefined unless alternative baseline is provided."))
    }
    
    #saving things for conditional estimates
    if (length(respondent.varying) > 0) {     
        output$cond.estimates <- conditional.estimates
        output$vcov.resp <- vcov_resp
        output$samplesize_full <- sample_size_full
        #save style edited formula (no depends), only resp-related
        output$cond.formula <- resp_effects_form
    }
    
    # Save baselines of unique (main) effects (if factor) to "baselines"
    # If continuous save summary information to "continuous"
    output$baselines <- list()
    output$continuous <- list()
    for (k in unique_vars) {
        if (inherits(data[[k]], "factor")) {
            output$baselines[[k]] <- levels(data[[k]])[1]
        } else if (inherits(data[[k]], "numeric")) {
             output$continuous[[k]] <- quantile(model.matrix(form,data)[,k], probs=c(0.25,0.5,0.75), na.rm=T)
        }
    }

    #save number of respondents if ID given
    if (!is.null(respondent.id)) {
        output$numrespondents <- length(unique(data[[respondent.id]]))
    } else {
        output$numrespondents <- NULL
    }

    #save respondent variables if given
    if (!is.null(respondent.varying)) {
        output$respondent.varying <- respondent_vars
    } else {
        output$respondent.varying <- NULL
    }

    #save weights as output (if any)
    if (!is.null(weights)) {
        output$weights <- subset(data, select = weights)
    } else {
        output$weights <- NULL
    }

    #save original names
    output$user.names <- user_names
    output$user.levels <- user_levels
    #save the original data
    output$data <- data
    return(output)
}

###############################################################################

## Getting conditional estimates given (1) AMCE object
## (2) LIST containing levels at which conditional effects will be calculated
## (3) NAME of current conditional variable
## (4) VALUE of current level of current conditioning variable
## (5) NAME of profile attribute being modified by conditioning variable

#############################################################################

get.conditional.effects <- function(object, conditional.levels, current.effect, current.level, mod.var) {
    
    #amce object
    amce_obj <- object
    #make dummy data and set in base levels
    cond.data <- amce_obj$data
    #set factor vars to baselines
    for (var in names(amce_obj$baselines)) {
        cond.data[[var]] <- amce_obj$baselines[[var]]
    }
    #set in conditional vars to their first given level
    for (var in names(conditional.levels)) {
        cond.data[[var]] <- conditional.levels[[var]][1]
    }        
    #set current effect to current level
    cond.data[[current.effect]] <- current.level

     #original levels of conditional vars
    orig.levels <- sapply(all.vars(amce_obj$cond.formula)[-1] [all.vars(amce_obj$cond.formula)[-1] %in% names(amce_obj$baselines)], function(x) levels(amce_obj$data[[x]]), simplify=F)
     #coefficients associated with conditional estimates base term
    cond.base <- unlist(sapply(amce_obj$respondent.varying,USE.NAMES = F, function(x) colnames(amce_obj$cond.estimates[[x]])))
    #estimated coefficients, adding 0 for intercept
    cond.beta <- c(0,do.call(cbind,amce_obj$cond.estimates)[1,])

    #blank output
    estimates.vector <- c()
    error.vector <- c()
    names.vector <- c()
    
    #quick covariance getting function
    cov.ij <- function(var1,var2) {
        out <- pred_mat[var1]*pred_mat[var2]*amce_obj$vcov.resp[var1,var2]
        return(out)
    }
    cov.ij <- Vectorize(cov.ij,vectorize.args = c("var1","var2"))

    #function for NA multiplication to be used in special cases
    na.multiply <- function(x,y) {
        vec <- c(x,y)
        #If either is NA and other is 0, return 0
        if (any(is.na(vec)) && vec[!is.na(vec)] == 0) {
            out <- 0
        } else {
            #otherwise normal (so 1*NA = NA)
            out <- x*y
        }
    }
    na.multiply <- Vectorize(na.multiply,vectorize.args = c("x","y"))
    
    # split up modified variable
    mod_vars <- strsplit(mod.var,":")[[1]]
    # loop over levels
    for (mod_coef in colnames(amce_obj$cond.estimates[[mod.var]])) {
        ## (1) Edit conditional data
        # split level coefficient into components
        mod_coefs <- strsplit(mod_coef,":")[[1]]
        # edit cond data to fit this level
        for (x in 1:length(mod_coefs)) {
            mod_lev <- sub(mod_vars[x],"",mod_coefs[x])            
            cond.data[[mod_vars[x]]] <- mod_lev
        }
        ## (2) Make model matrix
        pred_mat <- model.matrix(amce_obj$cond.formula,cond.data,xlev = orig.levels)
        ## (3) Turn off base term for this conditional var
        turn_off <- rep(1,ncol(pred_mat))
        names(turn_off) <- colnames(pred_mat)         
        turn_off[cond.base] <- 0
        
        ### Fix 3/20/2025 - Need to turn off *all baselines* for interaction objects 
        ### (ones that don't contain the **specific interaction** of interest)
        mod_coef_contain <- grep(mod_coef, colnames(pred_mat), invert=T)
        turn_off[mod_coef_contain] <- 0
        
        # Use to turn off terms in pred_mat that only contain respondent varying items
        pred_mat <- pred_mat[1,]*turn_off
        ## (4) Calculate coefficient and SE
        # (a) Coefficient
        if (!any(is.na(cond.beta))) {
            pred_val <- sum(pred_mat*cond.beta)
        } else {
            #otherwise use special function
            pred_val <- sum(na.multiply(pred_mat,cond.beta))
        }
        # (b) SE
        if (!is.na(pred_val)) {     
            #variable names sans intercept
            vars <- colnames(amce_obj$vcov.resp)[2:ncol(amce_obj$vcov.resp)]
            #all other covariance combinations
            all_cov <- outer(vars,vars,FUN= function(x,y) cov.ij(x,y))
            pred_se <- sqrt(sum(all_cov))
        } else {
            pred_se <- NA
        }
        ## And print out
        estimates.vector <- c(estimates.vector,pred_val)
        error.vector <- c(error.vector,pred_se)
        names.vector <- c(names.vector,mod_coef)
    }

    #return list of modified coefficient estimates and se's
    names(estimates.vector) <- names(error.vector) <- names.vector
    out <- rbind(estimates.vector,error.vector)
    return(out)

}

############################################################
## summary function for results of main AMCE function
############################################################

# Function for summarizing output from main amce function
# LIST given to "covariate.values" contains VECTORS at which ...
# ... conditional effects will be calculated; default is quantiles...
# ... in the case of continuous, levels otherwise
# ... can be given manual names by naming entries
# Note that must be NAMED LIST, entry name is the respondent.varying effect
#' @method summary amce
#' @export
summary.amce <- function(object, covariate.values=NULL, ...) {
    amce_obj <- object

######################### administrative section
    
    # Initialize list to store summary object
    summary_results <- list()
    # Create header of data.frame
    header <- c("Attribute", "Level", "Estimate", "Std. Err", "z value", "Pr(>|z|)", " ")

    #checks on user-supplied covariate values (values at which conditional effects are calculated)
    if (!is.null(covariate.values)) {
        #clean variable names
        names(covariate.values) <- clean.names(names(covariate.values))
        for (i in 1:length(covariate.values)) {
            #make sure they appear AMCE object
            if (!names(covariate.values)[i] %in% amce_obj$respondent.varying) {
                stop(paste(c("Error: variable",names(covariate.values)[i],"is not a respondent-varying characteristic in AMCE object."),collapse=" "))
            }
            #if they do appear and are factors, clean them and make sure valid
            if (names(covariate.values)[i] %in% names(amce_obj$baselines)) {
                covariate.values[[i]] <- clean.names(covariate.values[[i]])
                if (any(!covariate.values[[i]] %in% levels(amce_obj$data[[names(covariate.values)[i]]]))) {
                    stop(paste("Error: some level(s) of variable",names(covariate.values)[i],"do not appear in data."))
                }
                if (is.null(names(covariate.values[[i]]))) {
                    cov.names <- sapply(covariate.values[[i]],function(x) paste(names(covariate.values)[i],x,sep=""))
                    names(covariate.values[[i]]) <- sapply(cov.names,USE.NAMES = F, function(x) amce_obj$user.levels[x])
                }                
            }
        }
    }

##################### reporting unconditional results

    #all attribute estimates
    all_prof <- names(amce_obj$estimates) 
    #get AMCE only
    all_amce <- grep(":",all_prof,value=T,invert=T)
    #get ACIE only
    all_acie <- grep(":",all_prof,value=T,invert=F)

     #How many AMCE?
    namce <-  sum(sapply(all_amce,function(x) length(unlist(amce_obj$estimates[[x]]))/2))
    # Create results matrix for AMCE only
    summary_results[["amce"]] <- matrix(nrow= namce, ncol=length(header))
    colnames(summary_results[["amce"]]) <- header
    summary_results[["amce"]] <- as.data.frame(summary_results[["amce"]])
    #amce index
    amce_i <- 1
    
    #summary results matrix for ACIE only
    if (length(all_acie) > 0) {
        #number affected
        nacie <- sum(sapply(all_acie,function(x) length(unlist(amce_obj$estimates[[x]]))/2))
        #make results matrix
        summary_results[["acie"]] <- matrix(nrow= nacie, ncol=length(header))
        colnames(summary_results[["acie"]]) <- header
        summary_results[["acie"]] <- as.data.frame(summary_results[["acie"]])
        #acie index
        acie_i <- 1
    }

    #objects for storing baselines
    baselines_amce <- c()
    baselines_acie <- c()
    #objects for storing effect names
    names_amce <- c()
    names_acie <- c()

    # Loop over non-respondent varying attributes, which are all factors by assumption
    for (effect in all_prof) {
        #split terms
        variates <- strsplit(effect, ":")[[1]]
        # Figure out the baseline level(s)
        lev_list <- c()
        print_names <- c()
        for (var in variates) {
			lev_list <- c(lev_list, amce_obj$user.levels[[clean.names(paste0(var,amce_obj$baselines[[var]]))]])
            print_names <- c(print_names,amce_obj$user.names[[var]])
        }
        print_level <- paste(lev_list,sep="",collapse=":")
        print_effect <- paste(print_names,sep="",collapse=":")       
        # If ACIE
        if (grepl(":",effect)) {
            #set entry name
            entry_name <- "acie"
            #report baselines and names
            baselines_acie <- c(baselines_acie,print_level)           
            names_acie <- c(names_acie,print_effect)
            #which index?
            index <- acie_i
        } else {
            entry_name <- "amce"
            # report baselines and names
            baselines_amce <- c(baselines_amce,print_level)
            names_amce <- c(names_amce, print_effect)
            #which index?
            index <- amce_i
        }   
         # Append results to the estimates dataframe
        for (p in 1:ncol(amce_obj$estimates[[effect]])) {
            variate_levels <- strsplit(colnames(amce_obj$estimates[[effect]])[p],":")[[1]]
            lev_list <- c()
            for (lev in variate_levels) {
                lev_list <- c(lev_list, amce_obj$user.levels[[lev]])
            }
            print_level <- paste(lev_list,sep="",collapse=":")
            summary_results[[entry_name]][index,1] <- print_effect
            summary_results[[entry_name]][index,2] <- print_level
            summary_results[[entry_name]][index,3] <- amce_obj$estimates[[effect]][1,p]
            summary_results[[entry_name]][index,4] <- amce_obj$estimates[[effect]][2,p]
            zscr <- amce_obj$estimates[[effect]][1,p]/amce_obj$estimates[[effect]][2,p]
            summary_results[[entry_name]][index,5] <- zscr
            pval <- 2*pnorm(-abs(zscr))
            summary_results[[entry_name]][index,6] <- pval            
                # Stars!
            if (!is.na(pval)) {
                if (pval < .001) {
                    summary_results[[entry_name]][index,7] <- "***"
                } else if (pval < .01) {
                    summary_results[[entry_name]][index,7] <- "**"
                } else if (pval < .05) {
                    summary_results[[entry_name]][index,7] <- "*"
                } else {
                    summary_results[[entry_name]][index,7] <- ""
                }
            } else {
                summary_results[[entry_name]][index,7] <- ""
            }
            index <- index + 1
        }
        #advance appropriate index
        if (grepl(":",effect)) {
            acie_i <- index
        } else {
            amce_i <- index
        }
    }

    #save as data frame and save baselines
    summary_results[["amce"]] <- as.data.frame(summary_results[["amce"]])
    summary_results[["baselines_amce"]] <- data.frame("Attribute" = names_amce, "Level" = baselines_amce)

    #if any acie save those too
    if (grepl(":",effect)) {
        summary_results[["acie"]] <- as.data.frame(summary_results[["acie"]])
        summary_results[["baselines_acie"]] <- data.frame("Attribute" = names_acie, "Level" = baselines_acie)
    }

################ reporting conditional results

## all appearances by the modified variable (say "var1")
## have already had the beta and var, cov for any other appearances added in
## (such as "dependency:var1:respondent.var" for "var1:respondent.var")
## So the only things that need to be grabbed are the profile term
## and the interaction with the respondent varying term (var1 and var1:respondent.var)
    
    # If there are respondent varying attributes, add estimates at levels/quantiles
    if (length(amce_obj$cond.estimates) > 0) {
            
        ### Setting covariate values at which conditional effects will be calculated
        if (is.null(covariate.values)) covariate.values <- list()
       #get levels at which conditional effects will be calculated if no values given or install given names
        #straight levels, NOT coefficient names
        if (any(!amce_obj$respondent.varying %in% names(covariate.values))) {
            for (var in amce_obj$respondent.varying[!amce_obj$respondent.varying %in% names(covariate.values)]) {
                if (var %in% names(amce_obj$baselines)) {
                    # if it's a factor get levels from column names
                    cov.vals <- colnames(amce_obj$cond.estimates[[var]])
                    #remove effect part
                    covariate.values[[var]] <- sub(var,"",cov.vals)
                    #add baseline 
                    covariate.values[[var]]  <- c(amce_obj$baselines[[var]],covariate.values[[var]])
                    #names from user input
                    cov.vals <- c(paste(var,amce_obj$baselines[[var]],sep=""),cov.vals)
                    names(covariate.values[[var]]) <-  sapply(cov.vals, USE.NAMES = F, function(x) amce_obj$user.levels[[x]])
                } else {
                    # otherwise get summary information from "continuous"
                    covariate.values[[var]] <- amce_obj$continuous[[var]]
                }
            }
        }

        #empty vectors for table key        
        tab_name_amce <- c()
        tab_var_amce  <- c()
        tab_val_amce  <- c()
        tab_name_acie <- c()
        tab_var_acie <- c()
        tab_val_acie <- c()
        
        # Loop over respondent-varying characteristics
        for (effect in amce_obj$respondent.varying) {

            #how to print the effect name
            print_effect <- amce_obj$user.names[[effect]]
            # identify all REQUESTED terms involving effect
            all_req_vars <- attr(terms(amce_obj$formula),"term.labels")
            all_resp <- unlist(sapply(all_req_vars,function(x) {
                y <- strsplit(x,":")[[1]]
                if (any(y == effect)) x                
            }))
            #figure out profile attributes these refer to
            all_mod <- unlist(sapply(all_resp,function(x) {
                subs <- strsplit(x,":")[[1]]
                subs <- subs[is.element(subs,all_prof)]
                if (length(subs) > 0) paste(subs,collapse=":")
            }))
            #just unique ones
            all_mod <- unique(all_mod)
            #make sure there are some
            if (length(all_mod) == 0) {
                stop(paste(c("respondent characteristic",effect,"not interacted with profile attributes, no interpretation"),collapse=" "))
            }
            
            #### split modified terms into AMCE or ACIE
            mod_amce <- grep(":",all_mod,value = T,invert = T)
            mod_acie <- grep(":",all_mod,value = T,invert = F)
            #how many AMCE are affected by this respondent characteristic?
            namce <- sum(sapply(mod_amce,function(x) length(unlist(amce_obj$estimates[[x]]))/2))
            #how many ACIE are affected by this respondent characteristic? 
            if (length(mod_acie) > 0) {                    
                nacie <- sum(sapply(mod_acie,function(x) length(unlist(amce_obj$estimates[[x]]))/2))
            }

            ######## Now loop over levels/quantiles of "effect"
            for (i in 1:length(covariate.values[[effect]])) {

                # get name of coefficient and level; also how to print level
                cond_lev <- covariate.values[[effect]][i]
                #how to print level name
                if (!is.null(names(covariate.values[[effect]]))) {
                    print_cond_lev <- names(covariate.values[[effect]])[i]
                } else {
                    print_cond_lev <- covariate.values[[effect]][i]
                }
            
                #### prep AMCE results
                # make results matrices
                entry_name_amce <- paste(c(effect,i,"amce"),collapse="")
                # make empty results matrix
                summary_results[[entry_name_amce]] <- matrix(NA,nrow = namce, ncol=length(header))
                colnames(summary_results[[entry_name_amce]]) <- header
                summary_results[[entry_name_amce]] <- as.data.frame(summary_results[[entry_name_amce]])
                amce_i <- 1
                #edit table key
                tab_name_amce <- c(tab_name_amce,entry_name_amce)
                tab_var_amce <- c(tab_var_amce, effect)
                tab_val_amce <- c(tab_val_amce, print_cond_lev)

                #### prep ACIE results, if any
                if (length(mod_acie) > 0) {
                    #entry name
                    entry_name_acie <- paste(c(effect,i,"acie"),collapse="")
                    #results matrix
                    summary_results[[entry_name_acie]] <- matrix(NA,nrow= nacie, ncol=length(header))
                    colnames(summary_results[[entry_name_acie]]) <- header
                    summary_results[[entry_name_acie]] <- as.data.frame(summary_results[[entry_name_acie]])
                    acie_i <- 1
                    #edit table key
                    tab_name_acie <- c(tab_name_acie,entry_name_acie)
                    tab_var_acie <- c(tab_var_acie, effect)
                    tab_val_acie <- c(tab_val_acie, print_cond_lev)
                }

                # loop over all attribute effects modified by "effect"
                for (mod_var in all_mod) {
                    #get name to print
                    mod_vars <- strsplit(mod_var, ":")[[1]]
                    print_vars <- c()
                    for (var in mod_vars) {
                        print_vars <- c(print_vars,amce_obj$user.names[[var]])  
                    }
                    print_mod_var <- paste(c(print_vars),collapse=":")
                    # Set entry to add to 
                    if (grepl(":",mod_var)) {
                        #set entry name
                        entry_name <- entry_name_acie
                        #which index?
                        index <- acie_i
                    } else {
                        entry_name <- entry_name_amce
                        #which index?
                        index <- amce_i
                    }
                    #calculate conditional effects
                    cond.effects <- get.conditional.effects(amce_obj,covariate.values,effect,cond_lev,mod_var)
                    #loop over the associated betas 
                    for (p in 1:ncol(amce_obj$cond.estimates[[mod_var]])) {
                        #level to be modified
                        mod_coef <- colnames(amce_obj$cond.estimates[[mod_var]])[p]
                        #how to print it
                        mod_coefs <- strsplit(mod_coef, ":")[[1]]        
                        print_levs <- c()
                        for (v in 1:length(mod_coefs)) {
                            print_levs <- c(print_levs, amce_obj$user.levels[[mod_coefs[v]]])
                        }
                        print_mod_level <- paste(print_levs,sep="",collapse=":")
                        #get beta and SE
                        cond_beta <- cond.effects[1,mod_coef]
                        cond_se <- cond.effects[2,mod_coef]
                        #modified zscr and pval
                        if (!is.na(cond_beta)) {
                            zscr <- cond_beta/cond_se
                            pval <- 2*pnorm(-abs(zscr))
                        } else {
                            zscr <- pval <- NA
                        }
                        #write results
                        summary_results[[entry_name]][index,1] <- print_mod_var
                        summary_results[[entry_name]][index,2] <- print_mod_level
                        summary_results[[entry_name]][index,3] <- cond_beta
                        summary_results[[entry_name]][index,4] <- cond_se
                        summary_results[[entry_name]][index,5] <- zscr
                        summary_results[[entry_name]][index,6] <- pval
                        # Stars!
                        if (!is.na(cond_beta)) {
                            if (pval < .001) {
                                summary_results[[entry_name]][index,7] <- "***"
                            } else if (pval < .01) {
                                summary_results[[entry_name]][index,7] <- "**"
                            } else if (pval < .05) {
                                summary_results[[entry_name]][index,7] <- "*"
                            } else {
                                summary_results[[entry_name]][index,7] <- ""
                            }
                        } else {
                            summary_results[[entry_name]][index,7] <- ""
                        }
                            index <- index + 1
                    } #end loop over levels of profile var
                    #advance appropriate index
                    if (grepl(":",effect)) {
                        acie_i <- index
                    } else {
                        amce_i <- index
                    }
                } #end loop over modified profile vars

                #save AMCE results as data frame
                summary_results[[entry_name_amce]] <- as.data.frame(summary_results[[entry_name_amce]])
                #and ACIE, if any
                if (length(mod_acie) > 0) {
                    summary_results[[entry_name_acie]] <- as.data.frame(summary_results[[entry_name_acie]])
                }
                    
            } #end loop over respondent var levels (lev_list)
                    
        } #end loop over respondent varying characteristics

        #save results as data frame and save baselines
         summary_results[["table_values_amce"]] <- data.frame("Table Name" = tab_name_amce, "Level Name" = tab_var_amce, "Level Value" = tab_val_amce)
        summary_results[["table_values_amce"]] <- apply(summary_results[["table_values_amce"]],c(1,2),function(x) as.character(x))
      
         summary_results[["table_values_acie"]] <- data.frame("Table Name" = tab_name_acie, "Level Name" = tab_var_acie, "Level Value" = tab_val_acie)
        summary_results[["table_values_acie"]] <- apply(summary_results[["table_values_acie"]],c(1,2),function(x) as.character(x))
 
        } else {
        summary_results[["table_values_amce"]] <- NULL
        summary_results[["table_values_acie"]] <- NULL
    }
  
    # Save sample size(s)
    summary_results[["samplesize_estimates"]] <- amce_obj$samplesize_prof
    if (!is.null(amce_obj$samplesize_full)) {
        summary_results[["samplesize_resp"]] <- amce_obj$samplesize_full
    } else {
        summary_results[["samplesize_resp"]] <- NULL
    }

    # If there's a respondent number, add that as well
    if (!is.null(amce_obj$numrespondents)) {
        summary_results[["respondents"]] <- amce_obj$numrespondents
    } else {
        summary_results[["respondents"]] <- NULL
    }

    # Set class
    class(summary_results) <- c("summary.amce")
  
    # Return
    return(summary_results)
}

############################################################
## print summary function for results of main AMCE function
############################################################
#' @method print summary.amce
#' @export
print.summary.amce <- function(x, digits=5, ...) {
    summary_result <- x

    #basic print for AMCE
    cat("------------------------------------------\n")
    cat("Average Marginal Component Effects (AMCE):\n")
    cat("------------------------------------------\n")
    print(summary_result$amce, digits=digits, row.names=F)
    cat("---\n")
    cat(paste("Number of Obs. = ", summary_result$samplesize_estimates, sep=""))
    cat("\n")
    cat("---\n")   
    if (!is.null(summary_result$respondents)) {
        cat(paste("Number of Respondents = ", summary_result$respondents, sep=""))
        cat("\n")
        cat("---\n")
    }
    
    #add extra tables for AMCE interactions with respondent varying
    if (!is.null(summary_result$table_values_amce)) {
        if (nrow(summary_result$table_values_amce) > 0) {
            all.vars <- unique(summary_result$table_values_amce[,2])
            for (i in 1:nrow(summary_result$table_values_amce)) {
                #How to print changing level
                print.lev <- paste(c("Conditional AMCE's ","(", summary_result$table_values_amce[i,2], " = ", summary_result$table_values_amce[i,3]), collapse="")
                #How to print stable level 
                if (length(all.vars) > 1) {
                    this.var <- summary_result$table_values_amce[i,2]
                    other.vars <- all.vars[!is.element(all.vars,this.var)]
                    for (x in other.vars) {
                        lev <- summary_result$table_values_amce[summary_result$table_values_amce[ ,"Level.Name"] == x, "Level.Value"][1]
                        print.lev <- paste(c(print.lev,"; ",x," = ",lev),collapse="")
                    }
                }
                print.lev <- paste(c(print.lev,"):\n"),collapse="")

                cat("------------------------------------------------------------\n")
                cat(print.lev)
                cat("------------------------------------------------------------\n")
                print(summary_result[[summary_result$table_values_amce[i,1]]], digits=digits, row.names=F)
                cat("---\n")
                cat(paste("Number of Obs. = ", summary_result$samplesize_resp, sep=""))
                cat("\n")            
                if (!is.null(summary_result$respondents)) {
                    cat(paste("Number of Respondents = ", summary_result$respondents, sep=""))
                    cat("\n")
                }
                cat("---\n")  
                cat("Signif. codes: 0 '***' 0.001 '**' 0.01 '*' 0.05")
                cat("\n")
                cat("\n") 
            }
        }
    }

    #print AMCE baselines
    cat("Signif. codes: 0 '***' 0.001 '**' 0.01 '*' 0.05")
    cat("\n")
    cat("\n")
    cat("--------------------\n")
    cat("AMCE Baseline Levels:\n")
    cat("--------------------\n")
    print(summary_result$baselines_amce, row.names=F)
    cat("\n")
    cat("\n")


    #Tables for UNCONDITIONAL interaction
    if (!is.null(summary_result$acie)) {
        cat("---------------------------------------------\n")
        cat("Average Component Interaction Effects (ACIE):\n")
        cat("---------------------------------------------\n")
        print(summary_result$acie, digits=digits, row.names=F)
        cat("---\n")
        cat(paste("Number of Obs. = ", summary_result$samplesize_estimates, sep=""))
        cat("\n")
        if (!is.null(summary_result$respondents)) {
            cat(paste("Number of Respondents = ", summary_result$respondents, sep=""))
            cat("\n")
            
        }
    }
    
    #add extra tables for ACIE interactions with respondent varying
    if (!is.null(summary_result$table_values_acie)) {
        if (nrow(summary_result$table_values_acie) > 0) {
            for (i in 1:nrow(summary_result$table_values_acie)) {
                cat("------------------------------------------------------------\n")
                cat(paste(c("Conditional ACIE's","(",summary_result$table_values_acie[i,2],"=", summary_result$table_values_acie[i,3],"):\n"),collapse=" "))
                cat("------------------------------------------------------------\n")
                print(summary_result[[summary_result$table_values_acie[i,1]]], digits=digits, row.names=F)
                cat("---\n")
                cat(paste("Number of Obs. = ", summary_result$samplesize_resp, sep=""))
                cat("\n")              
                if (!is.null(summary_result$respondents)) {
                    cat(paste("Number of Respondents = ", summary_result$respondents, sep=""))
                    cat("\n") 
                }
                cat("---\n")
                cat("Signif. codes: 0 '***' 0.001 '**' 0.01 '*' 0.05")
                cat("\n")
                cat("\n")
            }             
        }
    }

    #baselines for ACIE
    if (!is.null(summary_result$acie)) {
        cat("---\n")
        cat("Signif. codes: 0 '***' 0.001 '**' 0.01 '*' 0.05")
        cat("\n")
        cat("\n")
        cat("--------------------\n")
        cat("ACIE Baseline Levels:\n")
        cat("--------------------\n")
        print(summary_result$baselines_acie, row.names=F)
        cat("\n")
        cat("\n")
    }

}
########################################################
# plot amce function
#######################################################

# default will return single plot with point estimates and CI's
# to facet plots, give "facet.name" the name of the variable to facet by...
# ... by default variable facets will be ALL level combinations...
# ... or if a continuous variable is given, will use quantiles...
# ... to customize, directly give "facet.levels" a named LIST ...
# ... with desired values of each variable entered as DATA FRAME with desired names
# "display" takes one of: "all", "unconditional", "conditional"
# with no given facet and no respondent vars, this choice is irrelevant (unconditional only)
# with no given facet and respondent vars, default is all but can choose just unconditional or interaction
# with a facet given (respondent or otherwise), similarly can choose
#' @method plot amce
#' @export 
plot.amce <- function(x, main="", xlab="Change in E[Y]", ci=.95, colors=NULL, xlim=NULL, breaks=NULL, labels=NULL, attribute.names = NULL, level.names = NULL, label.baseline = TRUE, text.size=11, text.color = "black", point.size = .5, dodge.size=0.9, plot.theme = NULL, plot.display = "all", facet.names = NULL, facet.levels = NULL, group.order = NULL,font.family = NULL,...) {
  
  # You need ggplot2
  amce_obj <- x
  ylim <- xlim
  
  # Make R CMD check happy
  pe <- NULL
  se <- NULL
  group <- NULL
  lower <- NULL
  upper <- NULL
  var <- NULL
  printvar <- NULL
  facet <- NULL
  
  ############################## basic set-up: get attributes and levels
  
  # Extract raw attribute names from the amce_obj$estimates object
  raw_attributes <- names(amce_obj$estimates)
  # Extract raw levels (coefficient names)
  raw_levels <- lapply(amce_obj$estimates,colnames)
  
  # Determine baseline level for each effect estimate in raw_levels and append to beginning of each vector in raw_levels
  for (effect in names(raw_levels)) {
    effect_elements <- strsplit(effect, ":")[[1]]
    baseline_interactions <- c()
    for (elem in effect_elements) {
      # get baseline, as if coefficient name
      base_coef <- paste(c(elem,amce_obj$baselines[[elem]]),collapse="")
      baseline_interactions <- c(baseline_interactions,base_coef)
    }
    interaction_str <- paste(baseline_interactions,sep="",collapse=":")
    raw_levels[[effect]] <- c(interaction_str, raw_levels[[effect]])
  }
  
  ################################### Incorporate and adjust user-input: general
  
  # Convert ci to z-score
  if (ci < 1 & ci > 0) {
    zscr <- qnorm(1- ((1-ci)/2))
  } else {
    cat("Invalid confidence interval -- Defaulting to 95%")
    zscr <- qnorm(1- ((1-.95)/2))
  }
  
  ################################### Incorporate and adjust user-input: naming
  
  # Sanity check user-provided attribute.names against AMCE objects
  if (!is.null(attribute.names)) {
    attribute.names <- unique(attribute.names)
    if (length(attribute.names) != length(raw_attributes)) {
      cat(paste("Error: The number of unique elements in attribute.names ", length(attribute.names), " does not match the attributes in amce object for which estimates were obtained: ", paste(raw_attributes,collapse=", "), "\n", sep=""))
      cat("Defaulting attribute.names to attribute names in AMCE object\n")
      attribute.names <- NULL
    }
  }
  
  # Sanity check user-provided level.names against AMCE object
  if (!is.null(level.names)) {
    names(level.names) <- clean.names(names(level.names))
    for (name in names(level.names)) {
      if (name %in% names(raw_levels)) {
        if (length(level.names[[name]]) != length(raw_levels[[name]])) {
          cat(paste("Error: level.names lengths do not match levels for attribute ", name, "\n",sep=""))
          cat(paste("Defaulting level.names for attribute ", name, " to level names in AMCE object", "\n",sep=""))
          level.names[[name]] <- NULL
        }
      } else {
        cat(paste("Error: level.names entry ",name," not in AMCE object. Removing level.names for attribute.","\n",sep=""))
        level.names[[name]] <- NULL
      }
    }
  }
  
  # If no attribute name or changed to NULL, use initial user supplied names as attribute names
  if (is.null(attribute.names)) {
    attribute.names <- c()
    for (attr in names(amce_obj$estimates)) {
      attr_split <- strsplit(attr,":")[[1]]
      attr_lookup <- paste(unlist(sapply(attr_split,function(x) amce_obj$user.names[x])), collapse=":")
      attribute.names <- c(attribute.names,attr_lookup)
    }
  }
  
  # If no level names make blank list
  if (is.null(level.names)) level.names <- list()
  # fill in blank list or missing levels, if any
  if (any(!names(raw_levels) %in% names(level.names))) {
    for (attr in names(raw_levels)[!names(raw_levels) %in% names(level.names)]) {
      attr_split <- strsplit(raw_levels[[attr]],":")
      level.names[[attr]] <- unlist(lapply(attr_split,function(x) paste(sapply(x,function(y) amce_obj$user.levels[y]),collapse=":")))
    }
  }
  
  ################################### Incorporate and adjust user-input: facetting
  
  # valid plot.display option?
  plot.display.opts <- c("all","unconditional","interaction")
  if (!is.element(plot.display,plot.display.opts)) {
    stop(paste(c("Error-- plot.display must be once of: ",paste(plot.display.opts,collapse=", ")),collapse=" "))
  }
  
  #clean facet names; if levels but no names? level names are facets
  if (!is.null(facet.names)) {
    facet.names <- clean.names(facet.names)
  } else if (!is.null(facet.levels)) {
    facet.names <- clean.names(names(facet.levels))
  }
  
  #check that they are in AMCE object
  if (!is.null(facet.names)) {
    facet.names.check <- c()
    for (facet.name in facet.names) {
      if (grepl(":",facet.name)) stop("Error-- cannot facet by interaction in current version.")
      if (!facet.name %in% names(amce_obj$estimates) & !facet.name %in% names(amce_obj$cond.estimates)) {
        stop(paste(c("Error-- cannot find facet name",facet.name,"in AMCE object output."),collapse=" "))
      } else {
        facet.names.check <- c(facet.names.check,facet.name)
      }
    }
    facet.names <- facet.names.check
  }
  
  #if no facets but there are respondent varying characteristics, use those
  if ((is.null(facet.names)) & (length(amce_obj$respondent.varying) > 0) & (plot.display != "unconditional")) {
    facet.names <- amce_obj$respondent.varying
  }
  
  #no facet name or resp var, must be unconditional
  if (is.null(facet.names) & plot.display == "interaction") {
    warning("Warning: no facet name or respondent varying characteristic provided to calculate conditional estimates. Will display unconditional only")
    plot.display <- "unconditional"
  }
  
  #unconditional but facet names given? remove facet names
  if(plot.display == "unconditional" & !is.null(facet.names)) {
    warning("Warning-- plot display is set to unconditional, facet names will be ignored")
    facet.names <- NULL
    facet.levels <- NULL
  }
  
  #check and clean facet levels if provided
  if (!is.null(facet.levels)) {
    #clean names of facet levels
    names(facet.levels) <- clean.names(names(facet.levels))
    #clean actual levels
    for (facet.name in names(facet.levels)) {
      #if it's a factor, clean up level names
      if (facet.name %in% names(amce_obj$baselines)) {
        facet.levels[[facet.name]] <- clean.names(facet.levels[[facet.name]])
        #make sure that if it's profile-varying, there's more than base
        if (facet.name %in% names(amce_obj$estimates) && is.element(amce_obj$baselines[[facet.name]],facet.levels[[facet.name]])) {
          stop (paste(c("Error: Facet level \"",as.character(amce_obj$baselines[[facet.name]]), "\" is the baseline level of a profile varying attribute. Please provide alternative facet level or use defaults."), collapse=""))
        }
        #names from user input if none provided
        if (is.null(names(facet.levels[[facet.name]]))) {
          fac.levs <- sapply(facet.levels[[facet.name]],function(x) paste(facet.name,x,sep=""))
          names(facet.levels[[facet.name]]) <- sapply(fac.levs, USE.NAMES = F, function(x) amce_obj$user.levels[[x]])
        }
      } else if (is.null(names(facet.levels[[facet.name]]))) {
        #not a factor and no names, just take level values
        names(facet.levels[[facet.name]]) <- as.character(facet.levels[[facet.name]])
      }
    }
  }
  
  #if user didn't give any levels, make blank list
  if (is.null(facet.levels)) facet.levels <- list()
  #input missing levels if any
  if (any(!facet.names %in% names(facet.levels))) {
    for (facet.name in facet.names[!facet.names %in% names(facet.levels)]) {
      #if it's a factor, default facet levels are all levels
      if (facet.name %in%  names(amce_obj$baselines)) {
        if (facet.name %in% names(amce_obj$estimates)) {
          #if NOT respondent varying get levels and names from ESTIMATES
          fac.levs <- colnames(amce_obj$estimates[[facet.name]])
        } else {
          #get levels and names from COND.ESTIMATES
          fac.levs <- colnames(amce_obj$cond.estimates[[facet.name]])
        }
        # get pure levels
        facet.levels[[facet.name]] <- sub(facet.name,"",fac.levs)
        #add in baseline
        facet.levels[[facet.name]] <- c(amce_obj$baselines[[facet.name]], facet.levels[[facet.name]])
        #names from user input
        fac.levs <- c(paste(facet.name,amce_obj$baselines[[facet.name]],sep=""),fac.levs)
        names(facet.levels[[facet.name]])  <- sapply(fac.levs, USE.NAMES = F,function(x) amce_obj$user.levels[[x]])
      } else if (facet.name %in% names(amce_obj$continuous)) {
        #if it's continuous, default is quantiles
        facet.levels[[facet.name]] <-  amce_obj$continuous[[facet.name]]
      }
    }
  }
  
  #the equivalent of summary's "covariate values" are respondent-varying entries
  #so get just those
  covariate.values <- list()
  for (var in names(facet.levels)) {
    if (var %in% amce_obj$respondent.varying) {
      covariate.values[[var]] <- facet.levels[[var]]
    }
  }
  
  ################################### Compile estimates into plottable objects
  
  #blank data frame for plot data
  d <- data.frame(pe=c(), se=c(), upper=c(), lower=c(), var=c(), printvar = c(), group=c(), facet=c())
  
  ############# Unconditional estimates
  
  #only display if plot.display == all or unconditional
  if (plot.display != "interaction") {
    #if plot.display == all, add unconditional facet name (not needed for unconditional only)
    if (plot.display == "all") {
      uncond.facet.name <- "Unconditional"
    } else {
      uncond.facet.name <- NA
    }
    #if plot = all and there are non-respondent varying facet names
    #remove them from raw attributes
    if (plot.display == "all" && !is.null(facet.names)) {
      attr_remove <- c()
      for (facet.name in facet.names[!is.element(facet.names, amce_obj$respondent.varying)]) {
        attr_remove1 <- raw_attributes[grepl(":",raw_attributes)]
        attr_remove1 <- attr_remove1[grepl(facet.name,attr_remove1)]
        attr_remove <- c(attr_remove,attr_remove1)
      }
      raw_attributes <- raw_attributes[!is.element(raw_attributes,attr_remove)]
    }
    #loop over raw attribute names
    for (i in 1:length(raw_attributes)) {
      #get raw attribute name
      attr_name <- raw_attributes[i]
      #get attribute name to print
      print_attr_name <- attribute.names[which(names(amce_obj$estimates) == raw_attributes[i])]
      #set up basic group header and add to plot
      d_head <- data.frame(pe=NA, se=NA, upper=NA, lower=NA, var= attr_name, printvar=paste(print_attr_name, ":", sep=""), group="<NA>",facet=uncond.facet.name)
      d <- rbind(d,d_head)
      #iterate over levels
      for (j in 1:length(raw_levels[[attr_name]])) {
        #raw level name
        level_name <- raw_levels[[attr_name]][j]
        #get level name to print
        print_level_name <- level.names[[attr_name]][j]
        #if on the first level
        if (j == 1) {
          if (label.baseline) {
            print_level_name <- paste("(Baseline = ",print_level_name,")",sep="")
          }
          #get the baseline and print a blank line
          d_lev <- data.frame(pe=NA, se=NA, upper=NA, lower=NA, var=level_name, printvar=paste("   ", print_level_name,sep=""), group=print_attr_name, facet=uncond.facet.name)
        } else {
          #retrieve estimate and SE
          val_pe <- amce_obj$estimates[[attr_name]][1,level_name]
          val_se <- amce_obj$estimates[[attr_name]][2,level_name]
          #calculate bounds
          upper_bnd <- val_pe + zscr*val_se
          lower_bnd <- val_pe - zscr*val_se
          #make line to add to plot data
          d_lev <- data.frame(pe=val_pe, se=val_se, upper=upper_bnd, lower=lower_bnd, var=level_name, printvar=paste("   ", print_level_name,sep=""), group=print_attr_name, facet=uncond.facet.name)
        } #end if a baseline
        #add to plot
        d <- rbind(d,d_lev)
      } #end loop over levels
    } #end loop over non-facet related attribute names
  } #end if plot.display == all or plot.display == conditional
  
  ############# Conditional estimates
  
  #Only if plot.display is all or conditional and we got a facet name from somehere
  if (plot.display != "unconditional" & !is.null(facet.names)) {
    
    #loop over facets
    for (facet.name in facet.names) {
      
      #how to print it
      print_facet_name <- amce_obj$user.names[[facet.name]]
      #### identify all REQUESTED terms involving facet name
      all_req_vars <- attr(terms(amce_obj$formula),"term.labels")
      all_mod <- unlist(sapply(all_req_vars,function(x) {
        y <- strsplit(x,":")[[1]]
        if (any(y == facet.name)) x
      }))
      #figure out profile attributes these refer to
      all_mod <- unlist(sapply(all_mod,function(x) {
        subs <- strsplit(x,":")[[1]]
        subs <- subs[is.element(subs,names(amce_obj$estimates))]
        subs <- subs[subs != facet.name]
        if (length(subs) > 0) paste(subs,collapse=":")
      }))
      #make sure there are some
      if (length(all_mod) == 0) {
        stop(paste(c("Error: Facet variable",facet.name,"not interacted with profile attributes"),collapse=" "))
      }
      #just unique ones
      all_mod <- unique(all_mod)
      
      #Temp Bug Fixing
      if (is.element(facet.name,names(amce_obj$estimates))) {
        #if ACIE, then remove baseline of facet in the d dataset filling process
        facet.start <- 2
      } else {
        #if conditional on respondent varying
        facet.start <- 1
      }
      
      #for each actual facet level make new set of plot data
      for (k in facet.start:length(facet.levels[[facet.name]])) {
        # set level
        facet_lev <- facet.levels[[facet.name]][k]
        #how to print facet level
        if (is.element(facet.name,names(amce_obj$estimates))) {
          #if ACIE
          print_facet_level <- paste(c("ACIE", paste(c(print_facet_name, names(facet.levels[[facet.name]])[k]), collapse = " = ")), collapse = "\n")
        } else {
          #if conditional on respondent varying
          print_facet_level <- paste(c("Conditional on",paste(c(print_facet_name, names(facet.levels[[facet.name]])[k]), collapse = " = ")), collapse = "\n")
        }
        
        #loop over variables to be modified
        for (mod_var in all_mod) {
          #how to print modified attribute
          print_attr_name <- attribute.names[which(names(amce_obj$estimates) == mod_var)]
          #set up header to reflect base (non-facet) category
          d_head <- data.frame(pe=NA, se=NA, upper=NA, lower=NA,var=mod_var, printvar=paste(print_attr_name, ":", sep=""), group="<NA>", facet=print_facet_level)
          #add new header
          d <- rbind(d, d_head)
          #Get estimates
          if (facet.name %in% names(amce_obj$estimates)) {
            #figure out interaction name
            inter_coef <- paste(sort(c(mod_var,facet.name)),collapse = ":")
            #get from unconditional estimates
            estimate.source <- amce_obj$estimates[[inter_coef]]
            estimate.source <- estimate.source[,grep(paste0(facet.name, facet_lev), colnames(estimate.source))]
          } else {
            #calculate from function if conditional
            estimate.source <- get.conditional.effects(amce_obj, covariate.values, facet.name, facet_lev, mod_var)
          }
          
          #split into components
          mod_vars <- strsplit(mod_var,":")[[1]]
          #iterate over levels of modified variable
          for (p in 1:length(raw_levels[[mod_var]])) {
            #raw level name is original coefficient name
            mod_coef <- raw_levels[[mod_var]][p]
            #split it up
            mod_coefs <- strsplit(mod_coef,":")[[1]]
            #modify data dummy
            for (lev in 1:length(mod_coefs)) {
              #get level name from coefficient name
              mod_lev <- sub(mod_vars[lev],"",mod_coefs[lev])
            }

            #get level name to print
            print_level_name <- level.names[[mod_var]][p]
            #get the baseline of modified var and make a blank line
            if (p == 1) {
              if (label.baseline) {
                print_level_name <- paste("(Baseline = ",print_level_name,")",sep="")
              }
              d_lev <- data.frame(pe=NA, se=NA, upper=NA, lower=NA, var = mod_coef, printvar=paste("   ",  print_level_name,sep=""), group=print_attr_name, facet= print_facet_level)
            } else {
              #retrieve estimate and SE
              val_pe <- estimate.source[1,p-1]
              if (!is.na(val_pe)) {
                val_se <- estimate.source[2,p-1]
                #calculate bounds
                upper_bnd <- val_pe + zscr*val_se
                lower_bnd <- val_pe - zscr*val_se
              } else {
                val_se <- upper_bnd <- lower_bnd <- NA
              }
              #make line to add to plot data
              d_lev <- data.frame(pe=val_pe, se=val_se, upper=upper_bnd, lower=lower_bnd, var = mod_coef, printvar=paste("   ", print_level_name,sep=""), group=print_attr_name, facet=print_facet_level)
            }
            #add level data to plot data
            d  <- rbind(d, d_lev)
          } #end loop over levels of modified var
          
        } #end loop over all modified vars
      } #end loop over level of facetted variable
    } #end loop over facets
  } else {
    #if there are no facets or plot.display is unconditional, remove that column
    d <- d[,-which(colnames(d) == "facet")]
  }
  
  #################    format "d" dataframe
  
  # Set Y bounds
  if (is.null(ylim)) {
    max_upper <- max(d$upper, na.rm=T) + .05
    min_lower <- min(d$lower, na.rm=T) - .05
    ylim <- c(min_lower, max_upper)
    d[is.na(d)] <- max_upper + 100
  } else {
    d[is.na(d)] <- max(ylim) + 100
  }
  
  # Make group factors <NA> actually NA
  d$group[d$group == "<NA>"] <- NA
  #same with facet
  if(!is.null(facet.names)) d$facet[d$facet == "<NA>"] <- NA
  
  # Reverse factor ordering
  d$var <- factor(d$var,levels=unique(d$var)[length(d$var):1])
  #make facet into factor, if it exists
  if (!is.null(facet.names)) {
    d$facet <- factor(d$facet,levels=unique(d$facet))
  }
  
  ## Reorder if there is user-specified ordering
  if (!is.null(group.order)){
    
    n.row <- length(unique(as.character(d$var)))
    order.var <- vector("character", length = n.row)
    
    i <- 1
    while (i<n.row){
      for (j in group.order){
        order.var[i] <- unique(as.character(d$var[d$var==gsub(" ","",j)]))
        i <- i+1
        temp.d <- d
        temp.d$group <- gsub(" ","",temp.d$group)
        temp.d <- subset(temp.d, group==gsub(" ","",j))
        
        temp.var <- unique((as.character(temp.d$var)))
        order.var[i:(i+length(temp.var)-1)] <- temp.var
        i <- i+length(temp.var)
      }
    }
    order.var <- rev(order.var)
    
    order.df <- data.frame(order.var, 1:length(order.var))
    colnames(order.df) <- c("var", "order")
    
    d$var <- factor(d$var, levels=order.var)
    
    d <- merge(d, order.df, by.x="var", by.y="var", suffixes=c("",""))
    
  }
  ########## plot output
  
  p = ggplot(d, aes(y=pe, x=var, colour=group))
  p = p + coord_flip(ylim = ylim)
  p = p + geom_hline(yintercept = 0,size=.5,colour="black",linetype="dotted")
  p = p + geom_pointrange(aes(ymin=lower,ymax=upper),position=position_dodge(width=dodge.size),size=point.size)
  
  #add facetting
  if (!is.null(facet.names)) {
    p = p + facet_wrap(~ facet)
  }
  
  # If breaks and labels Null, use default
  if (is.null(breaks) & is.null(labels)) {
    p = p + scale_y_continuous(name=xlab)
  } else if (is.null(breaks) & !is.null(labels)) {
    p = p + scale_y_continuous(name=xlab, labels=labels)
  } else if (!is.null(breaks) & is.null(labels)) {
    p = p + scale_y_continuous(name=xlab, breaks=breaks)
  } else if (!is.null(breaks) & !is.null(labels)) {
    p = p + scale_y_continuous(name=xlab, breaks=breaks, labels=labels)
  }
  
  if (!is.null(group.order)){
    fix.xlabs.df <- d[!duplicated(d$var),]
    fix.xlabs <- fix.xlabs.df[order(-fix.xlabs.df$order),]$printvar
  } else {
    fix.xlabs <- as.character(d$printvar)[!duplicated(d$var)]
  }
  
  p = p + scale_x_discrete(name="",labels=fix.xlabs[length(fix.xlabs):1])
  
  # If there's a title,add it
  if (!is.null(main)) {
    if (main != "") {
      p = p + ggtitle(main)
    }
  }
  # If no colors specified, use default
  if (is.null(colors)) {
    p = p + scale_colour_discrete(" ")
  } else if (is.vector(colors)) {
    # Make manual palette
    cPal <- rep(colors, ceiling(length(unique(d$group))/length(colors)))
    # Use manual palette
    p = p + scale_colour_manual(values=cPal)
  } else {
    cat("Error: 'colors' must be a vector. Using default colors\n")
    p = p + scale_colour_discrete(" ")
  }
  
  # colour scheme
  # if no theme specified, use default
  if (is.null(plot.theme)){
    
      theme_bw1 <- function(base_size = text.size, base_family = "") {
          theme_grey(base_size = base_size, base_family = base_family) %+replace%
          theme(axis.text.x = element_text(size = base_size*.9, colour = text.color,  hjust = .5 , vjust=1),axis.text.y = element_text(size = base_size, colour = text.color, hjust = 0 , vjust=.5,family=font.family), axis.ticks = element_line(colour = "grey50"),axis.title.y = element_text(size = base_size,angle=90,vjust=.01,hjust=.1,family=font.family),plot.title = element_text(face = "bold",family=font.family),legend.position = "none")
      }
      
      p = p + theme_bw1()
      print(p)
    
  } else if (is.null(class(plot.theme)))  {
    
    cat("Error: 'plot.theme' is not a valid ggplot theme object. Using default theme\n")
      theme_bw1 <- function(base_size = text.size, base_family = "") {
          theme_grey(base_size = base_size, base_family = base_family) %+replace%
          theme(axis.text.x = element_text(size = base_size*.9, colour = text.color,  hjust = .5 , vjust=1),axis.text.y = element_text(size = base_size, colour = text.color, hjust = 0 , vjust=.5,family=font.family), axis.ticks = element_line(colour = "grey50"),axis.title.y = element_text(size = base_size,angle=90,vjust=.01,hjust=.1,family=font.family),plot.title = element_text(face = "bold",family=font.family),legend.position = "none")
      }
      
    p = p + theme_bw1()
    print(p)
    
  } else if (!inherits(plot.theme, "theme")) {
    
    cat("Error: 'plot.theme' is not a valid ggplot theme object. Using default theme\n")
      theme_bw1 <- function(base_size = text.size, base_family = "") {
          theme_grey(base_size = base_size, base_family = base_family) %+replace%
          theme(axis.text.x = element_text(size = base_size*.9, colour = text.color,  hjust = .5 , vjust=1),axis.text.y = element_text(size = base_size, colour = text.color, hjust = 0 , vjust=.5,family=font.family), axis.ticks = element_line(colour = "grey50"),axis.title.y = element_text(size = base_size,angle=90,vjust=.01,hjust=.1,family=font.family),plot.title = element_text(face = "bold",family=font.family),legend.position = "none")
      }
      
    p = p + theme_bw1()
    print(p)
    
    # otherwise use the user-passed theme
  } else {
    p = p + plot.theme
    print(p)
  }
  
  #console message with level to hold resp vars as
  if (length(covariate.values) > 1) {
    resp.message <- c("Note:")
    for (this.var in names(covariate.values)) {
      resp.message <- paste(c(resp.message," For AMCE and ACIE conditional on ",this.var,", "),collapse="")
      other.vars <- names(covariate.values)[names(covariate.values) != this.var]
      
      other.levels <- c()
      for (var in other.vars) {
        other.levels <- c(other.levels,paste(c(var," will be held at level \"",names(covariate.values[[var]])[1],"\""),collapse = ""))
      }
      other.levels <- paste(other.levels,collapse = ", and ")
      resp.message <- c(resp.message,other.levels,".")
      resp.message <- paste(resp.message,collapse = "")
    }
    cat(resp.message,"\n")
  }
  
}
#####################
# dependencies
######################

compute_dependencies <- function(J, tol=1e-14){
  # Get attribute names
  attribute_names <- names(dimnames(J))
  # If only one attribute, no dependence
  if (length(attribute_names) == 1){
    dependency_list <- list()
    dependency_list[[attribute_names[1]]] <- c()
    return(dependency_list)
  }else{
  # Create list for each attribute_name
  dependency_list <- list()
  for (k in attribute_names){
    dependency_list[[k]] <- c()
  }
  # Loop over each pair of attributes - figure out if they're independent
  for (i in 1:(length(attribute_names)-1)){
    for(j in (i+1):length(attribute_names)){
      attr1 <- attribute_names[i]
      attr2 <- attribute_names[j]
      cross_tab <- apply(J, c(attr1,attr2), sum)
      # Standardize
      sums <- apply(cross_tab, 1, sum)
      cross_tab_std <- cross_tab/sums
      # Compute similarities
      is_equal = TRUE
      r_1 <- cross_tab_std[1,]
      if (nrow(cross_tab_std) > 1){
        for (m in 2:nrow(cross_tab_std)){
          if (any(as.vector(r_1) - as.vector(cross_tab_std[m,]) > tol)){
            is_equal <- FALSE 
          }
        }
      }
      
      # If not the same, append to dependency dictionary
      if (!is_equal){
        dependency_list[[attr1]] <- c(dependency_list[[attr1]], attr2)
        dependency_list[[attr2]] <- c(dependency_list[[attr2]], attr1)
      } 
    }
  }
  return(dependency_list)
  }
}

##############################
# Make design matrix 
#############################

makeDesign <- function(type="file", J=NULL, filename=NULL, attribute.levels = NULL, constraints=NULL, level.probs=NULL, tol=1e-14){
  ## Initialize conjointDesign object
  design.obj <- NULL
  ## If type="file", then try to load from file generated by Conjoint SDT
  if (type == 'file'){
    if (is.null(filename)){
      cat("Error: Must provide a valid filename argument for type = 'file'\n")
      stop()
    }
    # Make a connection and read the lines
    connection <- file(filename, open="r")
    file_lines <- readLines(connection)
    close(connection)
    attr_index <- which(file_lines == "Attributes")
    weight_index <- which(file_lines == "Weights")
    restriction_index <- which(file_lines == "Restrictions")
    
    attributes <- file_lines[(attr_index+1):(weight_index-1)] 
    weight <- file_lines[(weight_index+1):(restriction_index-1)]
    if (restriction_index+1 != length(file_lines)){
      constr <- file_lines[(restriction_index+1):length(file_lines)]
    }else{
      constr <- NULL
    }
    attribute.levels <- list()
    for (attrstr in attributes){
      attributename <- strsplit(attrstr, ":")[[1]][1]
      levels <- strsplit(strsplit(attrstr, ":")[[1]][2], ",")[[1]]
      attribute.levels[[attributename]] <- levels
    }
    level.probs <- list()
    for (probstr in weight){
      attributename <- strsplit(probstr, ":")[[1]][1]
      weights <- strsplit(strsplit(probstr, ":")[[1]][2], ",")[[1]]
      level.probs[[attributename]] <- as.numeric(weights)
    }
    if (is.null(constr) != TRUE){
      constraints <- list()
      for (i in 1:length(constr)){
        allconstraints <- strsplit(constr[i], ";")[[1]]
        constraints[[i]] <- list()
        for (m in allconstraints){
          attributename <- strsplit(m, ":")[[1]][1]
          constrained_levels <- strsplit(strsplit(m, ":")[[1]][2], ",")[[1]]
          constraints[[i]][[attributename]] <- constrained_levels
        }
      }
    }else{
      constraints <- NULL
    }
  }
  ## if type = "array", check whether J is a valid array, then create conjointDesign object  
  if (type == 'array'){
    if (sum(J) != 1){
      cat("Error: Profile assignment probability array invalid: Does not sum to 1\n")
    }else{
      design.obj$J <- J
      design.obj$dependence <- compute_dependencies(J)
    }
  }else if (type == 'constraints' | type == 'file'){
    ## if type = "constraints"
    if (is.null(attribute.levels) | is.list(attribute.levels) != TRUE){
     cat("Error: Must provide a valid list() object in attribute.levels argument for type = 'constraints'\n")
    }
    # Calculate number of dimensions
    dimensions <- c()
    for (attr in names(attribute.levels)){
      dimensions <- c(dimensions, length(attribute.levels[[attr]]))
    }
    # Initialize 
    J_mat <- array(NA, dim=dimensions, dimnames = attribute.levels)

    # Fill in constrained cells with 0 probability
    for (cstr in constraints){
      # Save the names of the constrained attributes
      constraint_names <- names(cstr)
      # Construct a call to select relevant rows
      select_call <- Quote(J_mat[])
      select_call <- select_call[c(1, 2, rep(3, length(dim(J_mat))))]
      for (f in 1:length(constraint_names)){
        name <- constraint_names[f]
        index <- which(names(dimnames(J_mat)) == name)
        select_call[index+2] <- cstr[f]
      }
      # Make a Call
      eval(call("<-", select_call, 0))
    }
    
    # If no randomization weights, then uniform marginal randomization
    if (is.null(level.probs)){
      cell_prob <- 1/sum(is.na(J_mat))
      J_mat[is.na(J_mat)] <- cell_prob
    }else{
      # Normalize level.probs to sum to 1
      for (attr in names(level.probs)){
        level.probs[[attr]] <- level.probs[[attr]]/sum(level.probs[[attr]])
      }
      
      # If no names in level.probs, assume they're in order
      for (attr in names(level.probs)){
        if (is.null(names(level.probs[[attr]]))){
          names(level.probs[[attr]]) <- attribute.levels[[attr]]
        }
      }
      # If randomization weights specified: more calculations
      unconstrained_probs <- J_mat
      unconstrained_probs[TRUE] <- 1
      # For each Attribute
      for (attr in names(dimnames(J_mat))){
        # For each level in each Attribute
        for (level in attribute.levels[[attr]]){
          # Get a marginal probability
          marg_prob <- level.probs[[attr]][[level]]
        
          # Extract the rows pertaining to that marginal probability
          select_call <- Quote(unconstrained_probs[])
          select_call <- select_call[c(1, 2, rep(3, length(dim(J_mat))))]
          
          index <- which(names(dimnames(J_mat)) == attr)
          select_call[index+2] <- level
          # Make a Call
          
          eval(call("<-", select_call, eval(call("*",select_call, marg_prob))))
        }
      }
      missing_prob <- sum(unconstrained_probs[is.na(J_mat)==FALSE])
      increase_prob <- unconstrained_probs*1/(1-missing_prob)

      J_mat[is.na(J_mat)] <- increase_prob[is.na(J_mat)]
    }

    
    
    design.obj$J <- J_mat
    design.obj$dependence <- compute_dependencies(J_mat, tol)
    
  }else{
    cat("Invalid type argument: Must be either 'file', 'array', or 'constraints")
  }

  # Return design.obj if valid
  ## Make it a conjointDesign object 
  class(design.obj) <- "conjointDesign"
  return(design.obj)
}

###########################
# read in qualtrics data
###########################

re.escape <- function(strings){
  vals <- c("\\\\", "\\[", "\\]", "\\(", "\\)", 
            "\\{", "\\}", "\\^", "\\$","\\*", 
            "\\+", "\\?", "\\.", "\\|")
  replace.vals <- paste0("\\\\", vals)
  for(i in seq_along(vals)){
    strings <- gsub(vals[i], replace.vals[i], strings)
  }
  strings
}

read.qualtrics <- function(filename,responses=NULL,covariates = NULL,respondentID = NULL,letter="F",new.format = FALSE,ranks = NULL){
  
  ###### Load data and detect dimensions of things
  
  # Load CSV Results
  qualtrics_results <- read.csv(filename, stringsAsFactors=F)
  
  # Test whether it is of new format and modify the data
  if (new.format) {
    new.format.test <- grepl("ImportId",qualtrics_results[2,1])
    if (new.format==new.format.test) {
      print("New qualtrics format detected.")
      qualtrics_results <- qualtrics_results[-2,]
    } else {
      stop("You indicate the data is of new qualtrics format, but it seems to be in old format.")
      return (NULL)
    }
  }
  
  if (!new.format) {
    new.format.test <- grepl("ImportId",qualtrics_results[2,1])
    if (new.format==new.format.test) {
      print("Old qualtrics format detected.")
    } else {
      stop("You indicate the data is of old qualtrics format, but it seems to be in new format.")
      return (NULL)
    }
  }
  
  # Extract variable names/question names
  var_names <- as.character(qualtrics_results[1,])
  q_names <- colnames(qualtrics_results)
  # The rest is the raw data
  qualtrics_data <- qualtrics_results[2:nrow(qualtrics_results),]
  colnames(qualtrics_data) <- var_names
  # Make respondent index
  respondent_index <- 1:nrow(qualtrics_data)
  
  # Find the attribute names and number of tasks
  attr_regexp <-  paste(c("^",letter,"-[0-9]+-[0-9]+(?!-)"),collapse="")
  attr_name_cols <- grep(attr_regexp, var_names, perl=TRUE)
  # remove trailing whitespace
  qualtrics_data[attr_name_cols] <- lapply(qualtrics_data[attr_name_cols], function (x) sub("\\s+$", "", x))
  # Parse to matrix
  attr_name_matrix <- matrix(unlist(strsplit(var_names[attr_name_cols],"-")),nrow=3,ncol=length(attr_name_cols))
  colnames(attr_name_matrix) <- var_names[attr_name_cols]
  attr_name_matrix <- attr_name_matrix[2:nrow(attr_name_matrix),]
  attr_name_matrix <- as.data.frame(t(attr_name_matrix))
  
  num_tasks <-unique(as.integer(attr_name_matrix[,1]))
  
  # Find the level names and number of profiles
  level_regexp <- paste(c("^",letter,"-[0-9]+-[0-9]+-[0-9]+"),collapse="")
  level_name_cols <- grep(level_regexp, var_names, perl=TRUE)
  num_profiles <- length(unique(do.call(rbind,strsplit(var_names[level_name_cols],"-"))[,3]))
  
  # Convert to matrix
  level_name_matrix <- matrix(unlist(strsplit(var_names[level_name_cols],"-")),nrow=4,ncol=length(level_name_cols))
  colnames(level_name_matrix) <- var_names[level_name_cols]
  level_name_matrix <- level_name_matrix[2:nrow(level_name_matrix),]
  level_name_matrix <- as.data.frame(t(level_name_matrix))
  
  # Unique attributes
  all_attr <- c()
  for (attr_vec in attr_name_cols) {
    all_attr <- c(all_attr,qualtrics_data[,attr_vec])
  }
  ## Remove any trailing white spaces in strings
  all_attr <- gsub(pattern = "\\s+$", replacement = "", all_attr)
  #no missing values
  unique_attr <- unique(all_attr)[nchar(unique(all_attr)) != 0]
  #and no na's
  unique_attr <- unique_attr[!is.na(unique_attr)]
  
  ####### Checks on input
  
  # Are there any responses or ranks
  if (is.null(responses) & is.null(ranks)) {
    stop("Either responses or ranks must be non-NULL")
    return(NULL)
  }
  
  # If there are responses, are there the right number?
  if (!is.null(responses) && length(num_tasks) != length(responses)) {
    # If number of responses doesn't match num_tasks
    stop("Error: Number of response columns doesn't equal number of tasks in data")
    return(NULL)
  }
  
  # If there are ranks, are there the right number?
  if (!is.null(ranks) && length(num_tasks) != length(ranks)/num_profiles) {
    # If number of ranks doesn't match num_tasks
    stop("Error: Number of rank columns doesn't equal number of tasks times number of profiles in data")
    return(NULL)
  }
  
  # If no attributes fit the description
  if (length(attr_name_cols) == 0) {
    stop("Error: Cannot find any columns designating attributes and levels. Please make sure the input file originated from a Qualtrics survey designed using the Conjoint SDT")
    return(NULL)
  }
  
  # Check whether attribute columns are empty or not
  for (attr_column in attr_name_cols) { 
    if (is.null(unique(qualtrics_data[,attr_column]))) {
      stop(paste("Error, attribute column ", var_names[attr_column], " has no attribute names - recommend deleting this column"))
    } else if (unique(qualtrics_data[,attr_column])[1] == "" & length(unique(qualtrics_data[,attr_column])) == 1) {
      stop(paste("Error, attribute column ", var_names[attr_column], " has no attribute names - recommend deleting this column"))
    }
  }
  
  # Check whether level columns are empty or not
  for (lev_column in level_name_cols) {
    if (is.null(unique(qualtrics_data[,lev_column]))) {
      stop(paste("Error, level column ", var_names[lev_column], " has no attribute names - recommend deleting this column"))
    } else if (unique(qualtrics_data[,lev_column])[1] == "" & length(unique(qualtrics_data[,lev_column])) == 1) {
      stop(paste("Error, level column ", var_names[lev_column], " has no attribute names - recommend deleting this column"))
    }
  }  
  
  
  # If respondentID is not null
  if (!is.null(respondentID)){
    respondent_index <- qualtrics_data[,which(q_names %in% respondentID)]
    
  }else{
    respondent_index <- 1:nrow(qualtrics_data)
  }
  
  # Get the response rows
  if (is.character(responses[1])){
    response_vars <- which(q_names %in% responses)
  }else{
    response_vars <- responses
  }
  
  # Make Sure no reserved characters are used in attribute names
  if (sum(grepl("[\\'\"]",unique(all_attr)))>0){
    stop(paste("Error, attribute name has special characters. Some special characters are reserved for this function (if cjoint>v2.0.6) for the purpose of efficiency. If you still want to display special character in your plot, use the argument attribute.names in the plot function. See manual for more details."))
  } else {
    #grepl(paste0("^",unique(all_attr[unique(all_attr)!=""]),"_[0-9]+-[0-9]+$"), )
    if (sum(grepl("^attribute_[0-9]+$",unique(all_attr)))>0) {
      stop (paste("Error, attribute_[0-9]+ is reserved for the function."))
    }
    
    if (sum(grepl("^selected_[0-9]+-[0-9]+$",unique(all_attr)))>0) {
      stop (paste("Error, selected_[0-9]+-[0-9]+ is reserved for the function."))
    }
  }
  
  # Initialize output dataframe
  colnames(qualtrics_data)[which(q_names %in% covariates)] <- covariates
  out_data_set_cols <- c(which(q_names %in% respondentID),
                         which(q_names %in% covariates),
                         (attr_name_cols),
                         (level_name_cols)
  )
  out_data_dataset <- qualtrics_data[,out_data_set_cols]
  
  # Take care of null respondentID case
  if (!is.null(respondentID)){
    id_var_name <- colnames(out_data_dataset)[1] #By construction 1st element is the ID from out_data_dataset - thanks to bixiou
  } else {
    out_data_dataset <- cbind(out_data_dataset, respondent_index)
    id_var_name <- "respondent_index"
  }
  
  # Parameters 
  num_tasks <- unique(as.integer(attr_name_matrix[,1]))
  num_profiles <- as.integer(unique(level_name_matrix[,2]))
  num_attr <- unique(as.integer(attr_name_matrix[,2]))
  
  # Replace all - with _ in "F-X-Y"
  attr_regexp <-  paste(c("^",letter,"-[0-9]+-[0-9]+$"),collapse="")
  temp.col.index <- grep(attr_regexp,colnames(out_data_dataset), perl=TRUE)
  colnames(out_data_dataset)[temp.col.index] <- gsub("-","_",colnames(out_data_dataset)[temp.col.index])
  
  # Replace all - with _ in "F-X-Y-Z"
  level_regexp <- paste(c("^",letter,"-[0-9]+-[0-9]+-[0-9]+$"),collapse="")
  temp.col.index <- grep(level_regexp,colnames(out_data_dataset), perl=TRUE)
  colnames(out_data_dataset)[temp.col.index] <- gsub("-","_",colnames(out_data_dataset)[temp.col.index])
  
  # Clean attribute names
  for (i in num_attr){
    temp.cmd <- paste0("out_data_dataset['attribute_",i,"']<-","out_data_dataset['",letter,"_1_",i,"']")
    eval(parse(text=temp.cmd))
  }
  
  temp_regexp <-  paste(c("^",letter,"_[0-9]+_[0-9]+$"),collapse="")
  temp.col.index <- grep(temp_regexp,colnames(out_data_dataset), perl=TRUE)
  out_data_dataset <- out_data_dataset[,-temp.col.index]
  
  # Test Selected
  test.selected <- sum(!unique(unlist(qualtrics_data[,response_vars])) %in% c("",num_profiles))==0
  
  if (!test.selected){
    stop(paste0("Responses can only take values among (",paste(num_profiles, collapse = ","),")"))
    return (NULL)
  }
  
  # Generate Selected
  if (is.null(ranks)){
    for (i in num_tasks){
      temp.cmd <- paste0("temp.selected","<-","qualtrics_data[,",response_vars[i],"]")
      eval(parse(text=temp.cmd))
      for (j in num_profiles){
        temp.cmd <- paste0("out_data_dataset$'selected_",j,"-",i,"'<-","ifelse(temp.selected==j,1,0)")
        eval(parse(text=temp.cmd))
        temp.cmd <- paste0("out_data_dataset$'selected_",j,"-",i,"'[temp.selected","=='']","<-","''")
        eval(parse(text=temp.cmd))
      }
    }
  } else {
    ranks_col <- which(q_names %in% ranks)
    for (i in num_tasks){
      for (j in num_profiles){
        temp.cmd <- paste0("out_data_dataset$'selected_",j,"-",i,"'<-","qualtrics_data[,",ranks_col[(i-1)*length(num_profiles)+j],"]")
        eval(parse(text=temp.cmd))
      }
    }
  }
  
  # Remove row if attribute name is empty and trim all attribute entries
  trim <- function (x) gsub("^\\s+|\\s+$", "", x)
  
  for (i in num_attr){
    temp.cmd <- paste0("out_data_dataset","<-subset(out_data_dataset, attribute_",i,"!='')")
    eval(parse(text=temp.cmd))
    temp.cmd <- paste0("out_data_dataset['attribute_",i,"'] <- ","trim(out_data_dataset$'attribute_",i,"')")
    eval(parse(text=temp.cmd))
  }
  
  # Generate Attributes
  attribute_var_names <- unique(unlist(out_data_dataset[,grep("attribute_[0-9]+$", colnames(out_data_dataset))]))
  attribute_var_names_label <- gsub(" ",".", attribute_var_names)
  
  for (i in num_tasks){
    for (j in num_profiles){
      for (r in 1:length(attribute_var_names)){
        temp.cmd <- paste0("out_data_dataset['",attribute_var_names[r],"_",j,"-",i,"']<-''")
        eval(parse(text=temp.cmd))
        temp.cmd <- paste0("out_data_dataset['",attribute_var_names[r],".rowpos_",j,"-",i,"']<-''")
        eval(parse(text=temp.cmd))
      }
    }
  }
  
  for (i in num_tasks){
    for (j in num_profiles){
      for (k in num_attr){
        for (r in attribute_var_names){
          temp.cmd <- paste0("out_data_dataset['",r,"_",j,"-",i,"']",
                             "[out_data_dataset['attribute_",k,"']=='",r,
                             "']<-out_data_dataset['",letter,"_",i,"_",j,"_",k,
                             "'][out_data_dataset['attribute_",k,"']=='",r,"']")
          eval(parse(text=temp.cmd))
          
          temp.cmd <- paste0("out_data_dataset['",r,".rowpos","_",j,"-",i,"'",
                             "][out_data_dataset['attribute_",k,"']=='",r,
                             "']<-k")
          eval(parse(text=temp.cmd))
        }
      }
    }
  }
  
  temp_regexp <- paste(c("^",letter,"_[0-9]+_[0-9]+_[0-9]+$"),collapse="")
  temp.col.index <- grep(temp_regexp,colnames(out_data_dataset), perl=TRUE)
  out_data_dataset <- out_data_dataset[,-temp.col.index]
  
  # Delete attribute names
  regex.temp <- paste0("^attribute","_[0-9]+","$")
  out_data_dataset <- out_data_dataset[,-grep(regex.temp, colnames(out_data_dataset))]
  
  # Reshape the dataset Batch 1 - Round/Task
  regex.temp <- paste(paste0("^",re.escape(attribute_var_names),"_[0-9]+-[0-9]+","$"),collapse="|")
  regex.temp.2 <- paste(paste0("^",re.escape(attribute_var_names),".rowpos_[0-9]+-[0-9]+","$"),collapse="|")
  
  varying.temp <- colnames(out_data_dataset)[grep(regex.temp, colnames(out_data_dataset))]
  varying.temp.2 <- colnames(out_data_dataset)[grep(regex.temp.2, colnames(out_data_dataset))]
  varying.temp.3 <- colnames(out_data_dataset)[grep("^selected_[0-9]+-[0-9]+$", colnames(out_data_dataset))]
  
  varying.temp <- c(varying.temp, varying.temp.2, varying.temp.3)
  
  v.names.temp <- unique(gsub("-[0-9]+$","",varying.temp))
  v.names.temp <- v.names.temp[order(v.names.temp)]
  
  varying.temp <- paste0(rep(v.names.temp, length(num_tasks)),"-",
                         rep(num_tasks, each=length(v.names.temp)))
  
  
  out_data_dataset <- reshape(out_data_dataset,
                              idvar = id_var_name,
                              varying = varying.temp,
                              sep = "-",
                              timevar = "task",
                              times = num_tasks,
                              v.names = v.names.temp,
                              new.row.names	= 1:(length(num_tasks)*nrow(out_data_dataset)),
                              direction = "long")
  
  # Reshape the dataset Batch 2 - Profile
  regex.temp <- paste(paste0("^",re.escape(attribute_var_names),"_[0-9]+","$"), collapse="|")
  regex.temp.2 <- paste(paste0("^",re.escape(attribute_var_names),".rowpos_[0-9]+","$"), collapse="|")
  
  varying.temp <- colnames(out_data_dataset)[grep(regex.temp, colnames(out_data_dataset))]
  varying.temp.2 <- colnames(out_data_dataset)[grep(regex.temp.2, colnames(out_data_dataset))]
  varying.temp.3 <- colnames(out_data_dataset)[grep("^selected_[0-9]+$", colnames(out_data_dataset))]
  varying.temp <- c(varying.temp, varying.temp.2, varying.temp.3)
  
  v.names.temp <- unique(gsub("_[0-9]+$","",varying.temp))
  v.names.temp <- v.names.temp[order(v.names.temp)]
  
  varying.temp <- paste0(rep(v.names.temp, length(num_profiles)),"_",
                         rep(num_profiles, each=length(v.names.temp)))
  
  out_data_dataset <- reshape(out_data_dataset,
                              idvar = id_var_name,
                              varying = varying.temp,
                              sep = "_",
                              timevar = "profile",
                              times = num_profiles,
                              v.names = v.names.temp,
                              new.row.names	= 1:(length(num_profiles)*nrow(out_data_dataset)),
                              direction = "long")
  
  ## Post-processiong
  colnames(out_data_dataset)<- gsub(" ",".",colnames(out_data_dataset))
  
  for (m in attribute_var_names_label){
    out_data_dataset[[m]] <- as.factor(out_data_dataset[[m]])
  }
  
  colnames(out_data_dataset)[which(colnames(out_data_dataset)==id_var_name)] <- "respondent"
  out_data_dataset$respondentIndex <- as.factor(out_data_dataset$respondent)
  out_data_dataset$respondentIndex <- as.integer(out_data_dataset$respondentIndex)
  out_data_dataset$selected <- as.integer(out_data_dataset$selected)
  out_data_dataset$task <- as.integer(out_data_dataset$task)
  out_data_dataset$profile <- as.integer(out_data_dataset$profile)
  
  # Return dataset
  return(out_data_dataset)
}

###########################
# read in qualtRics data (the output from qualtRics package)
###########################


read.with.qualtRics <- function(filename,responses=NULL,covariates = NULL,respondentID = NULL,letter="F",new.format = FALSE,ranks = NULL){
  # Load CSV Results
  qualtrics_results <- filename
  
  # Replace all . with -
  replace.regexp <- paste0("^",letter,".[0-9]+.[0-9]+$")
  replace.col <- grep(replace.regexp, colnames(filename))
  colnames(qualtrics_results)[replace.col] <-
    gsub("[.]","-",colnames(qualtrics_results)[replace.col])
  
  replace.regexp <- paste0("^",letter,".[0-9]+.[0-9]+.[0-9]+$")
  replace.col <- grep(replace.regexp, colnames(filename))
  colnames(qualtrics_results)[replace.col] <-
    gsub("[.]","-",colnames(qualtrics_results)[replace.col])
  
# Extract variable names/question names
  var_names <- as.character(qualtrics_results[1,])
  q_names <- colnames(qualtrics_results)
  # The rest is the raw data
  qualtrics_data <- qualtrics_results[2:nrow(qualtrics_results),]
  colnames(qualtrics_data) <- var_names
  # Make respondent index
  respondent_index <- 1:nrow(qualtrics_data)
  
  # Find the attribute names and number of tasks
  attr_regexp <-  paste(c("^",letter,"-[0-9]+-[0-9]+(?!-)"),collapse="")
  attr_name_cols <- grep(attr_regexp, var_names, perl=TRUE)
  # remove trailing whitespace
  qualtrics_data[attr_name_cols] <- lapply(qualtrics_data[attr_name_cols], function (x) sub("\\s+$", "", x))
  # Parse to matrix
  attr_name_matrix <- matrix(unlist(strsplit(var_names[attr_name_cols],"-")),nrow=3,ncol=length(attr_name_cols))
  colnames(attr_name_matrix) <- var_names[attr_name_cols]
  attr_name_matrix <- attr_name_matrix[2:nrow(attr_name_matrix),]
  attr_name_matrix <- as.data.frame(t(attr_name_matrix))
  
  num_tasks <-unique(as.integer(attr_name_matrix[,1]))
  
  # Find the level names and number of profiles
  level_regexp <- paste(c("^",letter,"-[0-9]+-[0-9]+-[0-9]+"),collapse="")
  level_name_cols <- grep(level_regexp, var_names, perl=TRUE)
  num_profiles <- length(unique(do.call(rbind,strsplit(var_names[level_name_cols],"-"))[,3]))
  
  # Convert to matrix
  level_name_matrix <- matrix(unlist(strsplit(var_names[level_name_cols],"-")),nrow=4,ncol=length(level_name_cols))
  colnames(level_name_matrix) <- var_names[level_name_cols]
  level_name_matrix <- level_name_matrix[2:nrow(level_name_matrix),]
  level_name_matrix <- as.data.frame(t(level_name_matrix))
  
  # Unique attributes
  all_attr <- c()
  for (attr_vec in attr_name_cols) {
    all_attr <- c(all_attr,qualtrics_data[,attr_vec])
  }
  ## Remove any trailing white spaces in strings
  all_attr <- gsub(pattern = "\\s+$", replacement = "", all_attr)
  #no missing values
  unique_attr <- unique(all_attr)[nchar(unique(all_attr)) != 0]
  #and no na's
  unique_attr <- unique_attr[!is.na(unique_attr)]
  
  ####### Checks on input
  
  # Are there any responses or ranks
  if (is.null(responses) & is.null(ranks)) {
    stop("Either responses or ranks must be non-NULL")
    return(NULL)
  }
  
  # If there are responses, are there the right number?
  if (!is.null(responses) && length(num_tasks) != length(responses)) {
    # If number of responses doesn't match num_tasks
    stop("Error: Number of response columns doesn't equal number of tasks in data")
    return(NULL)
  }
  
  # If there are ranks, are there the right number?
  if (!is.null(ranks) && length(num_tasks) != length(ranks)/num_profiles) {
    # If number of ranks doesn't match num_tasks
    stop("Error: Number of rank columns doesn't equal number of tasks times number of profiles in data")
    return(NULL)
  }
  
  # If no attributes fit the description
  if (length(attr_name_cols) == 0) {
    stop("Error: Cannot find any columns designating attributes and levels. Please make sure the input file originated from a Qualtrics survey designed using the Conjoint SDT")
    return(NULL)
  }
  
  # Check whether attribute columns are empty or not
  for (attr_column in attr_name_cols) { 
    if (is.null(unique(qualtrics_data[,attr_column]))) {
      stop(paste("Error, attribute column ", var_names[attr_column], " has no attribute names - recommend deleting this column"))
    } else if (unique(qualtrics_data[,attr_column])[1] == "" & length(unique(qualtrics_data[,attr_column])) == 1) {
      stop(paste("Error, attribute column ", var_names[attr_column], " has no attribute names - recommend deleting this column"))
    }
  }
  
  # Check whether level columns are empty or not
  for (lev_column in level_name_cols) {
    if (is.null(unique(qualtrics_data[,lev_column]))) {
      stop(paste("Error, level column ", var_names[lev_column], " has no attribute names - recommend deleting this column"))
    } else if (unique(qualtrics_data[,lev_column])[1] == "" & length(unique(qualtrics_data[,lev_column])) == 1) {
      stop(paste("Error, level column ", var_names[lev_column], " has no attribute names - recommend deleting this column"))
    }
  }  
  
  
  # If respondentID is not null
  if (!is.null(respondentID)){
    respondent_index <- qualtrics_data[,which(q_names %in% respondentID)]
    
  }else{
    respondent_index <- 1:nrow(qualtrics_data)
  }
  
  # Get the response rows
  if (is.character(responses[1])){
    response_vars <- which(q_names %in% responses)
  }else{
    response_vars <- responses
  }
  
  # Make Sure no reserved characters are used in attribute names
  if (sum(grepl("[\\'\"]",unique(all_attr)))>0){
    stop(paste("Error, attribute name has special characters. Some special characters are reserved for this function (if cjoint>v2.0.6) for the purpose of efficiency. If you still want to display special character in your plot, use the argument attribute.names in the plot function. See manual for more details."))
  } else {
    #grepl(paste0("^",unique(all_attr[unique(all_attr)!=""]),"_[0-9]+-[0-9]+$"), )
    if (sum(grepl("^attribute_[0-9]+$",unique(all_attr)))>0) {
      stop (paste("Error, attribute_[0-9]+ is reserved for the function."))
    }
    
    if (sum(grepl("^selected_[0-9]+-[0-9]+$",unique(all_attr)))>0) {
      stop (paste("Error, selected_[0-9]+-[0-9]+ is reserved for the function."))
    }
  }
  
  # Initialize output dataframe
  colnames(qualtrics_data)[which(q_names %in% covariates)] <- covariates
  out_data_set_cols <- c(which(q_names %in% respondentID),
                         which(q_names %in% covariates),
                         (attr_name_cols),
                         (level_name_cols)
  )
  out_data_dataset <- qualtrics_data[,out_data_set_cols]
  
  # Take care of null respondentID case
  if (!is.null(respondentID)){
    id_var_name <- colnames(out_data_dataset)[1] #By construction 1st element is the ID from out_data_dataset - thanks to bixiou for noticing this
  } else {
    out_data_dataset <- cbind(out_data_dataset, respondent_index)
    id_var_name <- "respondent_index"
  }
  
  # Parameters 
  num_tasks <- unique(as.integer(attr_name_matrix[,1]))
  num_profiles <- as.integer(unique(level_name_matrix[,2]))
  num_attr <- unique(as.integer(attr_name_matrix[,2]))
  
  # Replace all - with _ in "F-X-Y"
  attr_regexp <-  paste(c("^",letter,"-[0-9]+-[0-9]+$"),collapse="")
  temp.col.index <- grep(attr_regexp,colnames(out_data_dataset), perl=TRUE)
  colnames(out_data_dataset)[temp.col.index] <- gsub("-","_",colnames(out_data_dataset)[temp.col.index])
  
  # Replace all - with _ in "F-X-Y-Z"
  level_regexp <- paste(c("^",letter,"-[0-9]+-[0-9]+-[0-9]+$"),collapse="")
  temp.col.index <- grep(level_regexp,colnames(out_data_dataset), perl=TRUE)
  colnames(out_data_dataset)[temp.col.index] <- gsub("-","_",colnames(out_data_dataset)[temp.col.index])
  
  # Clean attribute names
  for (i in num_attr){
    temp.cmd <- paste0("out_data_dataset['attribute_",i,"']<-","out_data_dataset['",letter,"_1_",i,"']")
    eval(parse(text=temp.cmd))
  }
  
  temp_regexp <-  paste(c("^",letter,"_[0-9]+_[0-9]+$"),collapse="")
  temp.col.index <- grep(temp_regexp,colnames(out_data_dataset), perl=TRUE)
  out_data_dataset <- out_data_dataset[,-temp.col.index]
  
  # Test Selected
  test.selected <- sum(!unique(unlist(qualtrics_data[,response_vars])) %in% c("",num_profiles))==0
  
  if (!test.selected){
    stop(paste0("Responses can only take values among (",paste(num_profiles, collapse = ","),")"))
    return (NULL)
  }
  
  # Generate Selected
  if (is.null(ranks)){
    for (i in num_tasks){
      temp.cmd <- paste0("temp.selected","<-","qualtrics_data[,",response_vars[i],"]")
      eval(parse(text=temp.cmd))
      for (j in num_profiles){
        temp.cmd <- paste0("out_data_dataset$'selected_",j,"-",i,"'<-","ifelse(temp.selected==j,1,0)")
        eval(parse(text=temp.cmd))
        temp.cmd <- paste0("out_data_dataset$'selected_",j,"-",i,"'[temp.selected","=='']","<-","''")
        eval(parse(text=temp.cmd))
      }
    }
  } else {
    ranks_col <- which(q_names %in% ranks)
    for (i in num_tasks){
      for (j in num_profiles){
        temp.cmd <- paste0("out_data_dataset$'selected_",j,"-",i,"'<-","qualtrics_data[,",ranks_col[(i-1)*length(num_profiles)+j],"]")
        eval(parse(text=temp.cmd))
      }
    }
  }
  
  # Remove row if attribute name is empty and trim all attribute entries
  trim <- function (x) gsub("^\\s+|\\s+$", "", x)
  
  for (i in num_attr){
    temp.cmd <- paste0("out_data_dataset","<-subset(out_data_dataset, attribute_",i,"!='')")
    eval(parse(text=temp.cmd))
    temp.cmd <- paste0("out_data_dataset['attribute_",i,"'] <- ","trim(out_data_dataset$'attribute_",i,"')")
    eval(parse(text=temp.cmd))
  }
  
  # Generate Attributes
  attribute_var_names <- unique(unlist(out_data_dataset[,grep("attribute_[0-9]+$", colnames(out_data_dataset))]))
  attribute_var_names_label <- gsub(" ",".", attribute_var_names)
  
  for (i in num_tasks){
    for (j in num_profiles){
      for (r in 1:length(attribute_var_names)){
        temp.cmd <- paste0("out_data_dataset['",attribute_var_names[r],"_",j,"-",i,"']<-''")
        eval(parse(text=temp.cmd))
        temp.cmd <- paste0("out_data_dataset['",attribute_var_names[r],".rowpos_",j,"-",i,"']<-''")
        eval(parse(text=temp.cmd))
      }
    }
  }
  
  for (i in num_tasks){
    for (j in num_profiles){
      for (k in num_attr){
        for (r in attribute_var_names){
          temp.cmd <- paste0("out_data_dataset['",r,"_",j,"-",i,"']",
                             "[out_data_dataset['attribute_",k,"']=='",r,
                             "']<-out_data_dataset['",letter,"_",i,"_",j,"_",k,
                             "'][out_data_dataset['attribute_",k,"']=='",r,"']")
          eval(parse(text=temp.cmd))
          
          temp.cmd <- paste0("out_data_dataset['",r,".rowpos","_",j,"-",i,"'",
                             "][out_data_dataset['attribute_",k,"']=='",r,
                             "']<-k")
          eval(parse(text=temp.cmd))
        }
      }
    }
  }
  
  temp_regexp <- paste(c("^",letter,"_[0-9]+_[0-9]+_[0-9]+$"),collapse="")
  temp.col.index <- grep(temp_regexp,colnames(out_data_dataset), perl=TRUE)
  out_data_dataset <- out_data_dataset[,-temp.col.index]
  
  # Delete attribute names
  regex.temp <- paste0("^attribute","_[0-9]+","$")
  out_data_dataset <- out_data_dataset[,-grep(regex.temp, colnames(out_data_dataset))]
  
  # Reshape the dataset Batch 1 - Round/Task
  regex.temp <- paste(paste0("^",re.escape(attribute_var_names),"_[0-9]+-[0-9]+","$"),collapse="|")
  regex.temp.2 <- paste(paste0("^",re.escape(attribute_var_names),".rowpos_[0-9]+-[0-9]+","$"),collapse="|")
  
  varying.temp <- colnames(out_data_dataset)[grep(regex.temp, colnames(out_data_dataset))]
  varying.temp.2 <- colnames(out_data_dataset)[grep(regex.temp.2, colnames(out_data_dataset))]
  varying.temp.3 <- colnames(out_data_dataset)[grep("^selected_[0-9]+-[0-9]+$", colnames(out_data_dataset))]
  
  varying.temp <- c(varying.temp, varying.temp.2, varying.temp.3)
  
  v.names.temp <- unique(gsub("-[0-9]+$","",varying.temp))
  v.names.temp <- v.names.temp[order(v.names.temp)]
  
  varying.temp <- paste0(rep(v.names.temp, length(num_tasks)),"-",
                         rep(num_tasks, each=length(v.names.temp)))
  
  
  out_data_dataset <- reshape(out_data_dataset,
                              idvar = id_var_name,
                              varying = varying.temp,
                              sep = "-",
                              timevar = "task",
                              times = num_tasks,
                              v.names = v.names.temp,
                              new.row.names	= 1:(length(num_tasks)*nrow(out_data_dataset)),
                              direction = "long")
  
  # Reshape the dataset Batch 2 - Profile
  regex.temp <- paste(paste0("^",re.escape(attribute_var_names),"_[0-9]+","$"), collapse="|")
  regex.temp.2 <- paste(paste0("^",re.escape(attribute_var_names),".rowpos_[0-9]+","$"), collapse="|")
  
  varying.temp <- colnames(out_data_dataset)[grep(regex.temp, colnames(out_data_dataset))]
  varying.temp.2 <- colnames(out_data_dataset)[grep(regex.temp.2, colnames(out_data_dataset))]
  varying.temp.3 <- colnames(out_data_dataset)[grep("^selected_[0-9]+$", colnames(out_data_dataset))]
  varying.temp <- c(varying.temp, varying.temp.2, varying.temp.3)
  
  v.names.temp <- unique(gsub("_[0-9]+$","",varying.temp))
  v.names.temp <- v.names.temp[order(v.names.temp)]
  
  varying.temp <- paste0(rep(v.names.temp, length(num_profiles)),"_",
                         rep(num_profiles, each=length(v.names.temp)))
  
  out_data_dataset <- reshape(out_data_dataset,
                              idvar = id_var_name,
                              varying = varying.temp,
                              sep = "_",
                              timevar = "profile",
                              times = num_profiles,
                              v.names = v.names.temp,
                              new.row.names	= 1:(length(num_profiles)*nrow(out_data_dataset)),
                              direction = "long")
  
  ## Post-processiong
  colnames(out_data_dataset)<- gsub(" ",".",colnames(out_data_dataset))
  
  for (m in attribute_var_names_label){
    out_data_dataset[[m]] <- as.factor(out_data_dataset[[m]])
  }
  
  colnames(out_data_dataset)[which(colnames(out_data_dataset)==id_var_name)] <- "respondent"
  out_data_dataset$respondentIndex <- as.factor(out_data_dataset$respondent)
  out_data_dataset$respondentIndex <- as.integer(out_data_dataset$respondentIndex)
  out_data_dataset$selected <- as.integer(out_data_dataset$selected)
  out_data_dataset$task <- as.integer(out_data_dataset$task)
  out_data_dataset$profile <- as.integer(out_data_dataset$profile)
  
  # Return dataset
  return(out_data_dataset)
}

## Tests on plot.amce
## 10/14/16 Elisha Cohen
results <- amce(Chosen_Immigrant ~  Gender + Education + `Language Skills` +
                  `Country of Origin` + Job + `Job Experience` + `Job Plans` +
                  `Reason for Application` + `Prior Entry`, data=immigrationconjoint,
                cluster=TRUE, respondent.id="CaseID", design=immigrationdesign)

cat("\nTEST PLOT OBJECT")
test_that("Plot object gives error with incorrect theme",{
  expect_error(plot(results, plot.theme = fake), "object 'fake' not found")
})


test_that("Incorrect CI returns error",{
  expect_output(plot(results, ci=1.5),"Invalid confidence interval -- Defaulting to 95%")
})

test_that("Valid plot display option",{
  expect_error(plot(results, plot.display = "other"),"Error-- plot.display must be once of:  all, unconditional, interaction")
})

data(immigrationconjoint)
data(immigrationdesign)

##-----------------------------------------------------
## Define Functions
##-----------------------------------------------------
## Additive, no arguments explicit
amce_mod <- function(varlist, data){
  myformula <- as.formula(paste("Chosen_Immigrant ~ ", paste(varlist, collapse = "+")))
  amce(myformula, data = data)
}

## only additive, no cluster
amce_additive_mod <- function(varlist, data){
  myformula <- as.formula(paste("Chosen_Immigrant ~ ", paste(varlist, collapse = "+")))
  amce(myformula, data = data,
       cluster = FALSE,
       respondent.id = NULL,
       design = "uniform",
       respondent.varying = NULL,
       subset = NULL,
       baselines = NULL,
       weights = NULL,
       na.ignore = FALSE)
}

## Additive and clustering
amce_additive_mod_cluster <- function(varlist, data, clustervar){
  myformula <- as.formula(paste("Chosen_Immigrant ~ ", paste(varlist, collapse = "+")))
  amce(myformula, data = data,
       cluster = TRUE,
       respondent.id = clustervar,
       design = "uniform",
       respondent.varying = NULL,
       subset = NULL,
       baselines = NULL,
       weights = NULL,
       na.ignore = FALSE)
}

## Additive, with cluster and weights
amce_additive_mod_cluster_weights <- function(varlist, data, clustervar, weightvar){
  myformula <- as.formula(paste("Chosen_Immigrant ~ ", paste(varlist, collapse = "+")))
  amce(myformula, data = data,
       cluster = TRUE,
       respondent.id = clustervar,
       design = "uniform",
       respondent.varying = NULL,
       subset = NULL,
       baselines = NULL,
       weights = weightvar,
       na.ignore = FALSE)
}

##Additive with interactions all variables, no cluster
amce_add_interact_mod <- function(varlist, data){
  myformula <- as.formula(paste("Chosen_Immigrant ~ ", paste(varlist, collapse = "*")))
  amce(myformula, data = data,
       cluster = FALSE,
       respondent.id = NULL,
       design = "uniform",
       respondent.varying = NULL,
       subset = NULL,
       baselines = NULL,
       weights = NULL,
       na.ignore = FALSE)
}

## Respondent varying
amce_add_mod_respvar <- function(varlist, data, respvar){
  myformula <- as.formula(paste("Chosen_Immigrant ~ ", paste(varlist, collapse = "+")))
  amce(myformula, data = data,
       cluster = FALSE,
       respondent.id = NULL,
       design = "uniform",
       respondent.varying = respvar,
       subset = NULL,
       baselines = NULL,
       weights = NULL,
       na.ignore = FALSE)
}

##-----------------------------------------------------
## Define Inputs
##-----------------------------------------------------
varlist1 <- c('Gender', 'Education', '`Language Skills`', '`Country of Origin`', 'Job',
              '`Job Experience`', '`Job Plans`', '`Reason for Application`', '`Prior Entry`')
varlist2 <- c('Gender', 'Education')

# create weights in data
weights <- runif(nrow(immigrationconjoint))
immigrationconjoint$weights <- weights
##-----------------------------------------------------
## Run Models
##-----------------------------------------------------
mod1 <- amce_additive_mod(varlist1, immigrationconjoint)
mod2 <- amce_additive_mod_cluster(varlist1, data = immigrationconjoint, clustervar = 'CaseID')
mod3 <- amce_additive_mod_cluster_weights(varlist1, data = immigrationconjoint, clustervar = 'CaseID',
                                          weightvar = 'weights')
##-----------------------------------------------------
## Run Tests
##-----------------------------------------------------
test_that("Output class is correct",{
  expect_is(mod1,"amce")
  expect_is(mod2, "amce")
  expect_is(mod3, "amce")
})

test_that("Warnings are issued",{
  # setting cluster to FALSE
  expect_warning(amce_mod(varlist1, immigrationconjoint),
                 "respondent.id is NULL - setting cluster to FALSE. Please specify a respondent.id variable if you want to estimate clustered standard errors")
})

# test_that("Interactions are calculated"){
#   
# })

#make sure no duplicates after spaces and special characters removed
varlist3 <- c(varlist1, '`Educ ation`')
tempdata <- immigrationconjoint
tempdata$`Educ ation` <- tempdata$Education

test_that("error for duplicate variables",{
  expect_error(amce_additive_mod(varlist3, tempdata),
               'Error: Variable names must be unique when whitespace and meta-characters are removed. Please rename.')
})

# add in any missing base terms for interactions
test_that("warning for missing base terms added",{
  expect_warning(amce(Chosen_Immigrant ~ Gender:Education, data = immigrationconjoint),
                 'Missing base terms for interactions added to formula')
})

## test for missing variables
test_that("error missing variables",{
  var <- 'Education'
  expect_error(amce_additive_mod(varlist1,(immigrationconjoint[, !names(immigrationconjoint) %in% var])),
               paste("Error:", var, "not in 'data'"))
})

# Make sure non-respondent varying are factors
test_that("warning change to factor", {
  temp_num <- immigrationconjoint
  temp_num$Gender <- as.numeric(temp_num$Gender)
  expect_warning(amce(Chosen_Immigrant ~ Education + Gender,
                      respondent.varying = NULL,
                      data = temp_num,
                      cluster = FALSE),
                 'Warning: Gender changed to factor')
})

## missing data
cat("\nMISSING VALUES TEST")
test_that("error missing values in data",{
  tempdf <- immigrationconjoint
  missing <- sample(x = 1:nrow(tempdf), size = .1*nrow(tempdf), replace = FALSE)
  tempdf$Education[missing] <- NA
  expect_error(amce(Chosen_Immigrant ~ Education + Gender,
                      respondent.varying = NULL,
                      data = tempdf,
                      cluster = FALSE),
                 "Error: Education has missing values in 'data'")
})

## missing respondent var in formula
cat("\nMISSING RESPONDENT VAR TEST")
test_that("error missing respondent var in formula",{
  expect_error(amce(Chosen_Immigrant ~ Education + Gender,
                    respondent.varying = "Job",
                    data = immigrationconjoint,
                    design = immigrationdesign,
                    cluster = FALSE),
               "Error: Job is specified in respondent.varying, but is not in the formula")
})



## outcome variable is numeric
cat("\nTEST NUMERIC OUTCOME")
test_that("error missing respondent var in formula",{
  tempdf <- immigrationconjoint
  tempdf$Chosen_Immigrant <- as.factor(tempdf$Chosen_Immigrant)
  expect_error(amce(Chosen_Immigrant ~ Education + Gender,
                      respondent.varying = NULL,
                      data = tempdf,
                      cluster = FALSE),
               "Error: ChosenImmigrant is not numeric or integer")
})


## user supplied baselines
cat("\nTEST BASELINE ERRORS")
test_that("error supplied baseline is not a level",{
  baselines <- list()
  baselines$Gender <- "other"
  expect_error(amce(Chosen_Immigrant ~ Gender + Education +  `Country of Origin`+ `Country of Origin` + Job,
                    data=immigrationconjoint, design=immigrationdesign, respondent.varying=NULL,
                    subset=!is.na(immigrationconjoint$ethnocentrism), respondent.id="CaseID",
                    cluster=TRUE, na.ignore=TRUE, weights=NULL, baselines = baselines),
               "Error: user supplied baseline other is not a level of Gender")
})

## sanity checks clustering
cat("\nTEST CLUSTERING")
test_that("Warning respondent id NULL setting cluster to false",{
  expect_warning(amce(Chosen_Immigrant ~ Gender + Education +  `Country of Origin`+ `Country of Origin` + Job,
                    data=immigrationconjoint, design=immigrationdesign, respondent.varying=NULL,
                    respondent.id=NULL,
                    cluster=TRUE, na.ignore=TRUE, weights=NULL),
               "respondent.id is NULL - setting cluster to FALSE. Please specify a respondent.id variable if you want to estimate clustered standard errors")
})

cat("\nTEST RESPONDENT ID")
test_that("Error respondent.id not in data",{
  expect_error(amce(Chosen_Immigrant ~ Gender + Education +  `Country of Origin`+ `Country of Origin` + Job,
                      data=immigrationconjoint, design=immigrationdesign, respondent.varying=NULL,
                      respondent.id="idvar",
                      cluster=TRUE, na.ignore=TRUE, weights=NULL),
                 "Error: respondent.id not in data")
})

## weights not in data
cat("\nTEST WEIGHTS")
test_that("Error: weights not in data",{
  expect_error(amce(Chosen_Immigrant ~ Gender + Education +  `Country of Origin` + Job,
                      data=immigrationconjoint, design=immigrationdesign, respondent.varying=NULL,
                      respondent.id=NULL,
                      cluster=FALSE, na.ignore=TRUE, weights="wts"),
                 "Error: weights not in data")
})

cat("\nTEST RESPONDENT VARYING")
test_that("Error:respondent.varying var not in the formula",{
  expect_error(amce(Chosen_Immigrant ~ Gender+  `Country of Origin` + Job,
                    data=immigrationconjoint, design=immigrationdesign, respondent.varying="Education",
                    respondent.id="CaseID",
                    cluster=TRUE, na.ignore=TRUE, weights=NULL, baselines = NULL),
               "Error: Education is specified in respondent.varying, but is not in the formula")
})

cat("\nTEST DESIGN OBJECT")
test_that("Error:not in 'design' object",{
  attribute_list <- list()
  attribute_list[["Gender"]] <- c("female","male")
  attribute_list[["Country of Origin"]] <- c("Germany","France","Mexico",
                                             "Philippines","Poland","India",
                                             "China","Sudan","Somalia","Iraq")
  temp_design <- makeDesign(type='constraints', attribute.levels=attribute_list)

  expect_error(amce(Chosen_Immigrant ~ Gender + Education + Job,
                    data=immigrationconjoint, design=temp_design, respondent.varying="Education",
                    respondent.id="CaseID",
                    cluster=TRUE, na.ignore=FALSE, weights=NULL, baselines = NULL),
               "Error: Job not in 'design' object")
})

cat("\nTEST DESIGN OBJECT IN DATA")
test_that("Error:'design' object is not in 'data'",{
  temp_conjoint <- immigrationconjoint[,-4]
  attribute_list <- list()
  attribute_list[["Gender"]] <- c("female","male")
  attribute_list[["Country of Origin"]] <- c("Germany","France","Mexico",
                                             "Philippines","Poland","India",
                                             "China","Sudan","Somalia","Iraq")
  temp_design <- makeDesign(type='constraints', attribute.levels=attribute_list)
  expect_error(amce(Chosen_Immigrant ~ Education,
                    data=temp_conjoint, design=temp_design, respondent.varying="Education",
                    respondent.id="CaseID",
                    cluster=TRUE, na.ignore=FALSE, weights=NULL, baselines = NULL),
               "Error: attribute Gender in 'design' object is not in 'data'")
})

cat("\nTEST FACTOR LEVEL OF ATTRIBUTE")
test_that("Error:factor level of attribute not in 'design' object",{
  attribute_list <- list()
  attribute_list[["Gender"]] <- c("female","male")
  attribute_list[["Country of Origin"]] <- c("Germany","France","Mexico",
                                             "Philippines","Poland","India")
  temp_design <- makeDesign(type='constraints', attribute.levels=attribute_list)

  expect_error(amce(Chosen_Immigrant ~ Gender + Education + `Country of Origin`,
                    data=immigrationconjoint, design=temp_design, respondent.varying="Education",
                    respondent.id="CaseID",
                    cluster=TRUE, na.ignore=FALSE, weights=NULL, baselines = NULL),
               "Error: factor level China of attribute CountryofOrigin not in 'design' object")
})

cat("\nTEST VALID STRING")
test_that("Error:'design' must be a valid character string ('uniform') or a conjointDesign object",{
  expect_error(amce(Chosen_Immigrant ~ Gender + Education + `Country of Origin`,
                    data=immigrationconjoint, design="temp", respondent.varying="Education",
                    respondent.id="CaseID",
                    cluster=TRUE, na.ignore=FALSE, weights=NULL, baselines = NULL),
               "Design object must be a valid character string 'uniform' or a conjointDesign object")
})

## subsetting checks
cat("\nTEST SUBSETTING")
test_that("Warning: invalid argument to 'subset' - must be a logical",{
  sub_data <- immigrationconjoint[!is.na(immigrationconjoint$ethnocentrism),]
  expect_warning(amce(Chosen_Immigrant ~ Gender + Education +  `Country of Origin` + Job,
                      data=immigrationconjoint,
                      design=immigrationdesign,
                      cluster = FALSE,
                      subset=sub_data),
               "Warning: invalid argument to 'subset' - must be a logical")
})


## subsetting checks
cat("\nTEST SUBSET LENGTH")
test_that("Warning: subset must be same length as data",{
  expect_warning(amce(Chosen_Immigrant ~ Gender + Education +  `Country of Origin` + Job,
                      data=immigrationconjoint,
                      design=immigrationdesign,
                      cluster = FALSE,
                      subset=(1 == 1)),
                 "Warning: invalid argument to 'subset' - must be the same length as the number of rows in data")
})


## read qualtrics
cat("\nTEST READ QUALTRICS")
test_that("Either responses or ranks must be non-NULL",{
  expect_error(conjoint_data <- read.qualtrics("CandidateConjointQualtrics.csv",
                                               responses=NULL,
                                               covariates=c("Q6.6"), respondentID="V1"),
                 "Either responses or ranks must be non-NULL")
})

cat("\nTEST NUMBER OF RESPONSES")
test_that("Error: Number of response columns doesn't equal number of tasks in data",{
  expect_error(conjoint_data <- read.qualtrics("CandidateConjointQualtrics.csv",
                                               responses=c("Q2.3", "Q2.7", "Q2.10"),
                                               covariates=c("Q6.6"), respondentID="V1"),
               "Error: Number of response columns doesn't equal number of tasks in data")
})

cat("\nTEST NUMBER OF COLUMNS")
test_that("Error: Number of rank columns doesn't equal number of tasks times number of profiles in data",{
  expect_error(conjoint_data <- read.qualtrics("CandidateConjointQualtrics.csv",
                                               ranks=c("Q2.3", "Q2.7", "Q2.10"),
                                               covariates=c("Q6.6"), respondentID="V1"),
               "Error: Number of rank columns doesn't equal number of tasks times number of profiles in data")
})

cat("\nTEST LEVELS OF VARIABLES")
test_that("Error: levels of variables",{
  df <- immigrationconjoint
  levels(df$Education) <- c(levels(df$Education), "college degree+")
  df$Education[df$Education == "graduate degree"] <- "college degree+"
  #table(df$Education,exclude = NULL)
  expect_error(amce(Chosen_Immigrant ~ Gender + Education + `Language Skills` +
                      `Country of Origin` + Job + `Job Experience` + `Job Plans` +
                      `Reason for Application` + `Prior Entry`, data=df,
                    cluster=TRUE, respondent.id="CaseID", design=immigrationdesign),
               "Error: levels of variable Education are not unique when whitespace and meta-characters are removed. Please rename.")
  expect_error(amce(Chosen_Immigrant ~ Gender + `Language Skills` +
                      `Country of Origin` + Job + `Job Experience` + `Job Plans` +
                      `Reason for Application` + `Prior Entry`, data=df,
                    cluster=TRUE, respondent.id="CaseID", design=immigrationdesign),
               "Error: levels of variable Education used as a dependency, are not unique when whitespace and meta-characters are removed. Please rename.")
})

'.onAttach' <- function(lib, pkg) {
  #mylib <- dirname(system.file(package = pkg))
  mylib <- lib
  title <- packageDescription(pkg, lib.loc = mylib)["Title"]
  ver <- packageDescription(pkg, lib.loc = mylib)["Version"]
  author <- packageDescription(pkg, lib.loc = mylib)["Author"]
  packageStartupMessage(pkg, ": ", title, "\nVersion: ", ver, "\nAuthors: ", author, "\n")
}

#' Estimating Causal Effects in Conjoint Experiments
#'
#' This function takes a dataset and a conjoint design and returns Average
#' Marginal Component Effects (AMCEs) and Average Component Interaction
#' Effects (ACIE) for the attributes specified in the formula. By default,
#' this function assumes uniform randomization of attribute levels and no
#' profile restrictions. If your design incorporates weighted randomization or
#' restrictions on displayable profiles, first generate a design object using
#' \code{makeDesign}. Interactions with respondent-level characteristics are
#' handled by identifying relevant variables as respondent-varying.
#'
#'
#' @param formula A \code{formula} object specifying the name of the outcome
#' variable on the left-hand side and the attributes for which effects are to
#' be estimated on the right-hand side. RHS attributes should be separated by
#' \code{+} signs. Interaction effects can be specified using standard
#' interaction syntax - joining attribute names using either \code{:} or
#' \code{*}.  However using the \code{:} syntax will produce the same results
#' as \code{*} since missing base terms are automatically added to the
#' formula. For example \code{Y ~ X1 + X2} will return AMCEs for X1 and X2.
#' \code{Y ~ X1 + X2 + X1:X2} will return AMCEs for X1 and X2 along with an
#' ACIE for X1/X2. \code{Y ~ X1*X2} and \code{Y ~ X1:X2} will produce
#' identical results to \code{Y ~ X1 + X2 + X1:X2}. Note that you can place
#' backticks around a variable name containing spaces in order to have
#' \code{\link{formula}} interpret it as a single variable name. Any
#' respondent characteristics must be designated as such in
#' \code{redpondent.varying}.
#' @param data A dataframe containing the outcome variable, attributes,
#' respondent identifiers, respondent covariate data and sampling weights from
#' a conjoint experiment.
#' @param design Either the character string \code{"uniform"} or a
#' \code{conjointDesign} object created by the \code{\link{makeDesign}}
#' function. If a \code{conjointDesign} is not passed, the function will
#' assume all attribute levels have an equal probability of being presented to
#' a respondent and that no profiles are restricted. Defaults to
#' \code{"uniform"}.
#' @param respondent.varying A vector of character strings giving the names of
#' any respondent-varying characteristics being interacted with AMCEs or ACIEs
#' in the \code{formula}.
#' @param subset A logical vector with length \code{nrow(data)} denoting which
#' rows in \code{data} should be included in estimation. This can for example
#' be used to subset the data along respondent-level covariates. Defaults to
#' \code{NULL}.
#' @param respondent.id A character string indicating the column of
#' \code{data} containing a unique identifier for each respondent. Defaults to
#' \code{NULL}.
#' @param cluster A logical indicating whether estimated standard errors
#' should be clustered on \code{respondent.id}. Defaults to \code{TRUE}.
#' @param na.ignore A logical indicating whether the function should ignore
#' missing rows in \code{data}. If \code{FALSE}, amce() will raise an error if
#' there are rows with missing values. Defaults to \code{FALSE}.
#' @param weights A character string giving the name of the column in the data
#' containing any survey weights. See documentation for \code{survey} package
#' for more information.
#' @param baselines Manually adjust the baselines of select factor variables
#' (either attributes or respondent varying) by supplying a list. Names of
#' list entries should correspond with variable names. The content of each
#' entry should be a character string giving the new baseline.
#' @return An object of class "amce" containing: \item{attributes}{A list
#' containing the names of attributes.} \item{baselines}{Baseline levels for
#' each attribute in \code{estimates}. Baselines determined using the first
#' element of \code{levels()}. If a different baseline level is desired for an
#' attribute, use the \code{relevel()} function on the variable prior to
#' calling the \code{amce()} routine or supply an alternative baseline in
#' \code{baselines} argument.} \item{continuous}{List of quantiles for any
#' non-factor variables, whether attributes or respondent varying.}
#' \item{data}{The original data.} \item{estimates}{A list containing AMCE and
#' ACIE estimates for each attribute in \code{formula}. Each element of
#' \code{estimates} corresponds to a single attribute or interaction.}
#' \item{formula}{The \code{formula} passed to the \code{amce()} routine.}
#' \item{samplesize_prof}{The number of valid profiles (rows) in the dataset}
#' \item{user.names}{A vector with the original user supplied names for any
#' attributes. These may differ from the attribute names in \code{estimates}
#' if the original names contain spaces.} \item{vcov.prof}{The modified
#' variance-covariance matrix for AMCE and ACIE estimates. Incorporates
#' cluster corrections as well as attribute dependencies. Profile varying
#' attributes only.} \item{numrespondents}{The number of respondents in the
#' dataset (if \code{respondent.id} is not \code{NULL}). }
#' \item{respondent.varying}{Names of respondent-varying variables, if any.}
#' \item{cond.formula}{The formula used for calculating estimates conditional
#' on respondent varying characteristics. Only returned when
#' respondent-varying characteristics are present.} \item{cond.estimates}{A
#' list containing AMCE and ACIE estimates conditional on the values of the
#' respondent-varying characteristics. Each element of \code{cond.estimates}
#' corresponds to a single attribute or interaction. Only returned when
#' respondent-varying characteristics are present} \item{samplesize_full}{The
#' number of valid profiles (rows) in the dataset when respondent varying
#' characteristics are included. Only returned when respondent-varying
#' characteristics are present. } \item{vcov.resp}{The modified
#' variance-covariance matrix for effect estimates conditional on
#' respondent-varying characteristics, where dependent relationships have been
#' incorporated into variances and covariances. Only returned when
#' respondent-varying characteristics are present. }
#' @seealso \code{\link{summary.amce}} for summaries and
#' \code{\link{plot.amce}} for generating a coefficient plot using
#' \code{ggplot2}.
#'
#' \code{\link{makeDesign}} to create \code{conjointDesign} objects.
#' @references Hainmueller, J., Hopkins, D., and Yamamoto T. (2014) Causal
#' Inference in Conjoint Analysis: Understanding Multi-Dimensional Choices via
#' Stated Preference Experiments. Political Analysis 22(1):1-30
#' @examples
#'
#' # Immigration Choice Conjoint Experiment Data from Hainmueller et. al. (2014).
#' data("immigrationconjoint")
#' data("immigrationdesign")
#'
#' # Run AMCE estimator using all attributes in the design
#' results <- amce(Chosen_Immigrant ~  Gender + Education + `Language Skills` +
#'                 `Country of Origin` + Job + `Job Experience` + `Job Plans` +
#'                 `Reason for Application` + `Prior Entry`, data=immigrationconjoint,
#'                 cluster=TRUE, respondent.id="CaseID", design=immigrationdesign)
#' # Print summary
#' summary(results)
#'
#' \dontrun{
#' # Run AMCE estimator using all attributes in the design with interactions
#' interaction_results <- amce(Chosen_Immigrant ~  Gender + Education + `Language Skills` +
#'                 `Country of Origin` + Job + `Job Experience` + `Job Plans` +
#'                 `Reason for Application` + `Prior Entry` + Education:`Language Skills` +
#' 		Job: `Job Experience` + `Job Plans`:`Reason for Application`,
#' 		data=immigrationconjoint, cluster=TRUE, respondent.id="CaseID",
#' 		design=immigrationdesign)
#' # Print summary
#' summary(interaction_results)
#'
#' # create weights in data
#' weights <- runif(nrow(immigrationconjoint))
#' immigrationconjoint$weights <- weights
#' # Run AMCE estimator using weights
#' results <- amce(Chosen_Immigrant ~  Gender + Education + `Language Skills` +
#'                 `Country of Origin` + Job + `Job Experience` + `Job Plans` +
#'                 `Reason for Application` + `Prior Entry`, data=immigrationconjoint,
#'                 cluster=TRUE, respondent.id="CaseID", design=immigrationdesign,
#' 		weights = "weights")
#' # Print summary
#' summary(results)
#'
#' # Include a respondent-varying interaction
#' results <- amce(Chosen_Immigrant ~ Gender + Education + Job +
#' 	   	ethnocentrism:Job + Education:Job,
#' 		data=immigrationconjoint, na.ignore = TRUE,
#' 		cluster=FALSE,design=immigrationdesign,
#' 		respondent.varying = "ethnocentrism")
#' # Print summary
#' summary(results)
#'
#' # Change the baseline for "Education"
#' baselines <- list()
#' baselines$Education <- "graduate degree"
#'
#' results <- amce(Chosen_Immigrant ~ Gender + Education + Job +
#' 		 Education:Job, data=immigrationconjoint,
#' 		 cluster=FALSE,design=immigrationdesign,
#' 		 baselines=baselines)
#' # Print summary
#' summary(results)
#' }
#'
#'




## start here

#' @examples
#' data("immigrationconjoint")
#' data("immigrationdesign")
#'
#' # Run AMCE estimator using all attributes in the design
#' results <- amce(Chosen_Immigrant ~  Gender + Education + `Language Skills` +
#'                 `Country of Origin` + Job + `Job Experience` + `Job Plans` +
#'                 `Reason for Application` + `Prior Entry`, data=immigrationconjoint,
#'                 cluster=TRUE, respondent.id="CaseID", design=immigrationdesign)
#' # Print summary
#' summary(results)
#' 
#' #'
#' \dontrun{
#' # Run AMCE estimator using all attributes in the design with interactions
#' interaction_results <- amce(Chosen_Immigrant ~  Gender + Education + `Language Skills` +
#'                 `Country of Origin` + Job + `Job Experience` + `Job Plans` +
#'                 `Reason for Application` + `Prior Entry` + Education:`Language Skills` +
#' 		Job: `Job Experience` + `Job Plans`:`Reason for Application`,
#' 		data=immigrationconjoint, cluster=TRUE, respondent.id="CaseID",
#' 		design=immigrationdesign)
#' # Print summary
#' summary(interaction_results)
#'
#' # create weights in data
#' weights <- runif(nrow(immigrationconjoint))
#' immigrationconjoint$weights <- weights
#' # Run AMCE estimator using weights
#' results <- amce(Chosen_Immigrant ~  Gender + Education + `Language Skills` +
#'                 `Country of Origin` + Job + `Job Experience` + `Job Plans` +
#'                 `Reason for Application` + `Prior Entry`, data=immigrationconjoint,
#'                 cluster=TRUE, respondent.id="CaseID", design=immigrationdesign,
#' 		weights = "weights")
#' # Print summary
#' summary(results)
#'
#' # Include a respondent-varying interaction
#' results <- amce(Chosen_Immigrant ~ Gender + Education + Job +
#' 	   	ethnocentrism:Job + Education:Job,
#' 		data=immigrationconjoint, na.ignore = TRUE,
#' 		cluster=FALSE,design=immigrationdesign,
#' 		respondent.varying = "ethnocentrism")
#' # Print summary
#' summary(results)
#'
#' # Change the baseline for "Education"
#' baselines <- list()
#' baselines$Education <- "graduate degree"
#'
#' results <- amce(Chosen_Immigrant ~ Gender + Education + Job +
#' 		 Education:Job, data=immigrationconjoint,
#' 		 cluster=FALSE,design=immigrationdesign,
#' 		 baselines=baselines)
#' # Print summary
#' summary(results)
#' }
#'
#'

\name{read.qualtrics}
\alias{read.qualtrics}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Read Data from a Conjoint Qualtrics Experiment
}
\description{
Converts the raw .CSV data file downloaded from an online conjoint experiment run using the Qualtrics survey software into a data frame usable by the \code{\link{amce}} routine. Each row of the Qualtrics .CSV file is a single survey respondent. The rows of the resulting dataframe correspond to individual profile choices. Currently, the function supports various types of outcomes in a conjoint experiment, with details explained below.
}
\usage{
read.qualtrics(filename, responses = NULL, covariates = NULL,
	respondentID = NULL, letter = "F",  new.format=FALSE, ranks = NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{filename}{
  A .CSV file containing responses exported from a Qualtrics survey experiment. The first row contains question and variable identifiers (e.g. V1, Q1). The second row contains variable names and question texts. Subsequent rows contain the answers of each respondent. This .CSV file can be exported through the "Download Data" panel in the Qualtrics web interface. Note that answers should be shown as "coded values" and not as choice text.
}
  \item{responses}{
  A character or integer vector with the identifiers of the CSV columns that contain the conjoint responses. The first element corresponds to the identifier of the first question, the second element corresponds to the second question and so on. The length of this vector must be equal to the number of conjoint tasks answered by each respondent. Identifiers are typically in the form "Q#" - e.g. \code{c("Q1", "Q3", "Q5")} would represent a three question survey where the conjoint questions are Q1, Q3 and Q5. If specified as an integer vector, the elements are the column numbers corresponding to each question response.
}
  \item{covariates}{
  A character vector denoting the column names of any other respondent-level characteristics measured in the survey that should be included in the resulting dataframe.
}
  \item{respondentID}{
  A character string denoting a column containing a unique identifier for each respondent (e.g. an IP address). This identifier will be carried over into the output. If NULL, each respondent will be given an arbitrary identifier in the output dataframe. Leave as NULL if you do not want responses to be linked back to a known respondent identifier.
}
\item{new.format}{
  Indicator for whether the .csv file is from the new Qualtrics export format with three title rows (TRUE) or from the old format (FALSE) with two title rows. Defaults to FALSE.
}
\item{ranks}{
	An integer vector with the identifiers of the CSV columns that contain the conjoint rankings or ratings. 
}
\item{letter}{
	The beginning letter used in the naming convention of levels and attributes.
}
}
\details{
This function currently only works with experiments that generate profiles using .PHP scripts created by the Conjoint Survey Design Tool. It also is only able to handle standard conjoint designs (binary outcome variable/forced choice).\cr
(\url{https://github.com/astrezhnev/conjointsdt/}). \cr

For each respondent in the .CSV file, attribute and level names are stored using the following naming convention:\cr
\bold{Level Name}: F-[task number]-[profile number]-[attribute number]\cr
\bold{Attribute Name}: F-[task number]-[attribute number]\cr
\bold{Example}: F-1-3-2 denotes the level corresponding to Task 1, Profile 3, Attribute 2\cr
F-3-3 denotes the attribute name corresponding to Task 3, Attribute 3

\bold{Special Characters}: Some special characters have been reserved for this function if (cjoint>v2.0.6) for efficienty purpose. This means some special characters cannot be used in the attribute names in your data. However, if you still want to display special characters in attribute names in your plot, you may want to use the argument \code{attribute.names} in function \code{plot.amce} to customize the display of your attribute names.

\bold{East Asian Language Support:} The read.qualtrics function relies on the read.csv function in R-core. The read.csv function only works well for some of character encoding, but not others, for East Asian languages in some Operation 
systems. In Windows, .csv files containing East Asian languages such as Chinese or Japanese should be stored in the ANSI encoding rather than UTF-8 for the read.csv function to work. Further, if you are reading the csv file in a Windows OS with a different display language as the file you are trying to read in, you will need to reconfigure R into the language of the csv file by \code{Sys.setlocale()}. 

\bold{Different types of responses}: This function supports various types of responses commonly used in conjoint analyses. Here are some illustrations on some typical types. 

1. The respondent is asked to fill in the profile she prefers the most, and her choice is restored in one response variable. In this case, set the argument responses=the response variable.

2. The respondent is asked to give each profile a rank within each task, and her ranks for each profile within each task are restored in J response variables, suppose there are J profiles within each task. In this case, set the argument ranks as a vector restoring the variables names of these responses variables, in the order of rank of profile 1 in task 1, rank of profile 2 in task 1, ... rank of profile J in task 1, rank of profile 1 in task 2, rank of profile 2 in task 2, ... rank of profile J in task 2, ..., rank of profile J in task K.

3. The respondent is asked to rate each profile within each task, and her ratings for each profile within each task are restored in J response variables, suppose there are J profiles within each task. In this case, set the argument ratings as a vector restoring the variables names of these responses variables, in the order of rating of profile 1 in task 1, rating of profile 2 in task 1, ... rating of profile J in task 1, rating of profile 1 in task 2, rating of profile 2 in task 2, ... rating of profile J in task 2, ..., rating of profile J in task K.

4. The respondent is asked to select the top L profiles she prefers within each task. L colud be smaller than the number of profiles J available within each task. Her choices are recorded with L variables for each task, indicating the first choice among J profiles in task 1, the second choice among J profiles in task 1, ..., the L-th choice among J profiles in task 1, the first choice among J profiles in task 2, the second choice among J profiles in task 2, ..., the L-th choice among J profiles in task 2, ..., the L-th choice among J profiles in task K. In this case, the read.qualtrics function should be applied in the following step:

(a) Set the argument response as a vector restoring the variables names of the first choice among J profiles in all tasks, in the order of first choice among J profiles in task 1, first choice among J profiles in task 2, ..., first choice among J profiles in task K. Save the respective data output.

(b) Set the argument response as a vector restoring the variables names of the second choice among J profiles in all tasks, in the order of second choice among J profiles in task 1, second choice among J profiles in task 2, ..., second choice among J profiles in task K. Save the respective data output.

(c) Repeat until you have completed the above steps for all top L choices. And then merge all data according to their respondentID. However, you may need to change the variable name of the responses a little bit for a successful merge.

5. The respondent is asked to select the top L profiles she prefers within each task. L colud be smaller than the number of profiles J available within each task. The questionnaire is designed in such format that the respondent fills in the ranking of top L profiles that she prefers, while leave the ranking of the other J-L profiles blank. In this case, set the argument ranks as a vector restoring the variables names of these J responses variables, in the order of rank of profile 1 in task 1, rank of profile 2 in task 1, ... rank of profile J in task 1, rank of profile 1 in task 2, rank of profile 2 in task 2, ... rank of profile J in task 2, ..., rank of profile J in task K. 

}

\value{
A dataframe in which each row corresponds to a single profile. The column "selected" denotes whether that profile was selected by the respondent. The columns "respondent" and "task" denote the respondent and task numbers to which the profile was assigned. Respondent-level covariates are appended to each row.
}
\references{
Strezhnev, A., Hainmueller, J., Hopkins, D., and Yamamoto, T. (2014) Conjoint Survey Design Tool. \url{https://github.com/astrezhnev/conjointsdt/}
}
\examples{
\dontrun{
# An example file with 5 conjoint tasks per respondent and a single covariate
# You can demonstrate this function's output using the CandidateConjointQualtrics.csv 
# demonstration file in the 'inst' subdirectory
conjoint_data <- read.qualtrics("CandidateConjointQualtrics.csv", 
responses=c("Q2.3", "Q2.7", "Q2.10", "Q2.13", "Q2.16"), 
covariates=c("Q6.6"), respondentID="V1")
}
}

\name{immigrationdesign}
\alias{immigrationdesign}
\docType{data}
\title{
Conjoint Design for the Immigration Experiment in Hainmueller et. al. (2014)
}
\description{
A "conjointDesign" object for the randomization scheme used in the immigration conjoint experiment in "Causal Inference in Conjoint Analysis." See \code{\link{immigrationconjoint}} for the accompanying dataset. 
}
\usage{data("immigrationdesign")}
\format{
A "conjointDesign" object. See \code{\link{makeDesign}} for more information about the structure.

}

\source{
Hainmueller, J., Hopkins, D., and Yamamoto T. (2014) Causal Inference in Conjoint Analysis: Understanding Multi-Dimensional Choices via Stated Preference Experiments. Political Analysis 22(1):1-30
}

\examples{
\dontrun{
# Loads the immigrationconjoint dataset and the immigrationdesign object
data("immigrationconjoint")
data("immigrationdesign")

# immigrationdesign is passed to the amce() function through the "design" argument.
results <- amce(Chosen_Immigrant ~  Gender + Education + `Language Skills` + 
                `Country of Origin` + Job + `Job Experience` + `Job Plans` + 
                `Reason for Application` + `Prior Entry`, data=immigrationconjoint, 
                cluster=TRUE, respondent.id="CaseID", design=immigrationdesign)

}
}
\keyword{datasets}

\name{plot.amce}
\alias{plot.amce}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Plot AMCE Estimates
}
\description{
\code{plot} method for "amce" objects
}
\usage{
\method{plot}{amce}(x, main = "", xlab = "Change in E[Y]", ci = 0.95,
                    colors = NULL, xlim = NULL, breaks = NULL,
                    labels = NULL, attribute.names = NULL, level.names = NULL,
                    label.baseline = TRUE, text.size = 11, text.color = "black",
                    point.size = 0.5, dodge.size = 0.9, plot.theme = NULL,
                    plot.display = "all",
					facet.names = NULL, facet.levels = NULL, 
					group.order = NULL,font.family = NULL, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x }{
  An object of class "amce", a result of a call to \code{\link{amce}}
}
  \item{main }{
  Title of the plot.
}
  \item{xlab }{
  Label of the x-axis of the plot (AMCE or ACIE). Default is "Change in E[Y]"
}
  \item{ci }{
  Levels for confidence intervals to plot around point estimates. Must be between 0 and 1. Default is .95
}
  \item{colors }{
  Vector of color names to be used for points and confidence intervals. The \code{plot} function will alternate between the colors in the vector for each attribute being plotted. If NULL, \code{plot} will use a default \code{ggplot2} color scheme.
}
  \item{xlim }{
 Numeric vector denoting the upper and lower bounds of the x-axis in the plot. If \code{NULL} the plot function will automatically set a range that includes all effect estimates.
}
  \item{breaks }{
  Numeric vector denoting where x-axis tick marks should be placed. If \code{NULL} plot will use \code{ggplot2} defaults.
}
  \item{labels }{
  Vector denoting how x-axis tick marks should be labeled. If \code{NULL} plot will use ggplot2 defaults.
}
  \item{attribute.names }{
  Character vector of attribute names to be plotted as labels. By default \code{plot.amce} will use the attribute names in the "amce" object passed to it.
}
  \item{level.names }{
  A list containing character vector elements with names in \code{attribute.names}. Each character vector in the list contains the level names to be plotted as labels beneath the corresponding attribute. By default \code{plot.amce} will use the level names in the "amce" object passed to it.
}
  \item{label.baseline }{
  If \code{TRUE}, the baseline levels for each attribute will be labeled as such. Defaults to \code{TRUE}.
}
  \item{text.size }{
  Size of text. Defaults to 11.
}
  \item{text.color }{
  Color of text in plot. Defaults to "black".
}
  \item{point.size }{
  Size of points in the plot. Defaults to 0.5.
}
  \item{dodge.size }{
  Width to dodge overlaps to the side. Defaults to 0.9.
}
  \item{plot.theme }{
  A ggplot2 'theme' object to be added to the plot. If NULL, defaults to black-and-white theme. Note that passing a theme object will override text and point color/size options. 
  }
  \item{plot.display}{
  Character string, one of "all", "unconditional", or "interaction". Option "all" will display both unconditional and interaction estimates. The "unconditional" option will display only 1 plot for unconditional estimates (both AMCE and ACIE) ignoring any facets provided to "facet.names" or respondent-varying characteristics. Option "interaction" will drop the unconditional plot and instead display only (unconditional) ACIE's or estimates conditional on respondent-varying characteristics as specified in the user-supplied option "facet.names". Defaults to "all".}
  
  \item{facet.names }{
  To facet plots (i.e., make separate plots for each value of a variable) give "facet.names" a vector of character strings containing the names of the variable(s) (either profile attribute or respondent-varying) to facet by. Unless given specific levels in "facet.levels", the plotted levels will consist of all levels of a factor variable or the quantiles of a continuous variable. Multiple facet variables cannot currently be varied at the same time within the same plot. Instead conditional effects will be calculated for one facet at a time while others are held at their first value in facet.levels, by default the bottom quantile for continuous variables and the baseline for factors.}
  
  \item{facet.levels }{
  To manually set facet levels, provide a list to "facet.levels". Names of list entries should correspond with variable names. The content of each entry should be a vector giving the desired levels, whether factors or continuous. To change the displayed names of the levels, assign names to each vector entry. 
}

  \item{group.order }{
  To manually set the order of the attributes, provide a vector to "group.order". Names of the vector entries should correspond with name of the attribute. 
}

  \item{\dots }{
  Other graphical parameters passed to \code{ggplot}.
}
  \item{font.family }{
  Will be passed to the ggplot function as the argument for font.family. If NULL, defaults will be used.
}
}

\value{
A \code{ggplot} object containing a dotplot of estimated AMCEs.
}
\references{
Hainmueller, J., Hopkins, D., and Yamamoto T. (2014) Causal Inference in Conjoint Analysis: Understanding Multi-Dimensional Choices via Stated Preference Experiments. Political Analysis 22(1):1-30
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{amce}} for the main estimation routine.
}
\examples{
\dontrun{
# Immigration Choice Conjoint Experiment Data from Hainmueller et. al. (2014).
data("immigrationconjoint")
data("immigrationdesign")

# Run AMCE estimator using all attributes in the design
results <- amce(Chosen_Immigrant ~  Gender + Education + `Language Skills` +
                `Country of Origin` + Job + `Job Experience` + `Job Plans` +
                `Reason for Application` + `Prior Entry`, data=immigrationconjoint,
                cluster=TRUE, respondent.id="CaseID", design=immigrationdesign)

# Plot results
plot(results, xlab="Change in Pr(Immigrant Preferred for Admission to U.S.)",
    ylim=c(-.3,.3), breaks=c(-.2, 0, .2), labels=c("-.2","0",".2"), text.size=13)
	
# Plot results with user-specified order of attributes
plot(results, xlab="Change in Pr(Immigrant Preferred for Admission to U.S.)",
     ylim=c(-.3,.3), breaks=c(-.2, 0, .2), labels=c("-.2","0",".2"), text.size=13,
     group.order=c("Gender","Education","Job",
                     "Language Skills","Job Experience",
                     "Job Plans","Reason for Application",
                     "Prior Entry","Country of Origin"))


# Run AMCE estimator with an interaction with a respondent-varying characteristic
interaction_results <- amce(Chosen_Immigrant ~ Gender + Education
		    + Job + ethnocentrism:Job,
		    data = immigrationconjoint,na.ignore=TRUE,
		    design = immigrationdesign, cluster = FALSE,
		    respondent.varying = "ethnocentrism")

# Plot results with additional plots for quantiles of the respondent-varying characteristic
plot(interaction_results)

# Plot results with user-specified order of attributes
plot(interaction_results, group.order=c("Gender","Education","Job"))

# Do not show output for variables that do not vary with facetted levels
plot(interaction_results,plot.display="unconditional")

# RUN AMCE estimator with an interaction between two factor variables
interaction_results <- amce(Chosen_Immigrant ~ Gender + Education + Job
		    + Education:Job, data = immigrationconjoint,
		    cluster = FALSE, design = immigrationdesign)

# Plot results with different plots for all levels of ACIE
plot(interaction_results,facet.names = "Education")


# Plot results with different plots for only two levels of one interacted variable
facet.levels1 <- list()
facet.levels1[["Education"]] <- c("college degree","graduate degree")
plot(interaction_results,facet.names = "Education",facet.levels = facet.levels1)

# Display only interaction panes
plot(interaction_results,facet.names = "Education",plot.display="interaction")

#Display only non-interaction panes
plot(interaction_results,facet.names = "Education",plot.display="unconditional")

#Change displayed attribute and level names
results <- amce(Chosen_Immigrant ~ Gender + Education + Job, data = immigrationconjoint,
		    cluster = FALSE, design = immigrationdesign)
levels.test<-list()
levels.test[["Gender"]]<-c("level1","level2")
levels.test[["Education"]]<-c("level1","b","c","d","e","f","g")
levels.test[["Job"]]<-c("a","b","c","d","e","f","g","h","i","j","k")
plot(results, level.names = levels.test, main="test", xlab="test", 
ci=0.95, breaks=c(-0.2,-0.1,0,0.1,0.2), attribute.names = c("attribute1","attribute2","attribute3"))

}
}

\name{makeDesign}
\alias{makeDesign}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Create conjoint design
}
\description{
Generates a "conjointDesign" object to be passed to \code{\link{amce}}. Average Marginal Component Effects (AMCEs) are defined relative to the distribution of potential choice profiles. When the probability of each profile being presented to a respondent is not constant (e.g. some profiles are restricted from appearing), some attributes will not be independent and simple difference-in-means estimates will not be unbiased for the AMCE. This function allows users to specify non-uniform profile assignment schemes to be used by \code{\link{amce}}.
}
\usage{
makeDesign(type="file", J=NULL, filename=NULL, attribute.levels = NULL,
            constraints=NULL, level.probs=NULL, tol=1e-14)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{type}{
  A character string - either "file", "constraints", or "array". Each option requires a different set of additional arguments. If "file", the user must specify a filename of a ".dat" file exported via the Conjoint Survey Design Tool. If "constraints," the user must provide a list of attributes and levels in "attribute.levels" along with any constraints in "constraints" and attribute randomization weights in "level.probs". If "array," the user must pass an array to J.
}
  \item{J}{
  If type="array", \code{makeDesign} requires a d-dimensional array to be passed to J where d is the number of attributes in a single profile. . Each dimension should have a number of indices corresponding to the number of levels for that attribute. Attribute and level names are taken from the \code{dimnames} of the array (level names are taken from the list elements, attribute names are taken from the names of the elements). Each cell of the array is the joint probability of assigning that particular profile (combination of attribute-levels).
}
  \item{filename}{
  If type="file", this is a character string giving the name of a design file exported from the Conjoint Survey Design Tool using the "Export design to R" option.
}
  \item{attribute.levels}{
  If type="constraints", attribute.levels is a required argument. This takes a named list with each element containing a character vector with the level names of a single attribute. The names of the elements should be the attribute names.
}
  \item{constraints}{
  If type = "constraints", this is an optional argument. A list of lists. Each element list is a "restriction" and describes a set of restricted profiles (attribute-level groupings that cannot be displayed to a respondent) with character vectors containing the attribute-levels that cannot be displayed together. If your design prevents certain profiles from being seen by respondents (for example, if the particular profile would be highly impluasible), this argument allows you to specify those restrictions. Each element of a restriction is a character vector of levels for a particular attribute. That element must have the name of one of the attributes. For each specified restriction, \code{makeDesign} will treat any profile containing any combination of the restricted levels as having zero probability of being displayed. For example, if a restriction contains two elements: A = c("1","2") and B = c("4", "5"), then any profiles containing levels 1 or 2 of attribute A and levels 4 or 5 of attribute B will restricted from the design.
}
  \item{level.probs}{
  If type = "constraints", this is an optional argument. A named list containing numeric vector elements. Each list element should have the name of one of the attributes passed in attribute.levels and the numeric vector it contains should have the names of the corresponding levels. Each element of the numeric vector specifies the marginal probability of that level appearing in a profile before any restricted profiles are eliminated. Each of the vectors should sum to 1.  If NULL, then uniform randomization is assumed (excluding constrained profiles).
}
  \item{tol}{
  A very small non-zero number used in the routine for determining which attributes are dependent (the distribution of one attribute depends on the value of another attribute). Two attributes are independent if the distribution of one attribute does not change conditional on the other - this is calculated automatically through the J matrix Do not change unless you are having issues with \code{makeDesign} incorrectly labeling independent attributes as dependent (in which case, lower the value) or incorrectly labeling dependent attributes as independent (in which case, raise the value).
}
}
\details{
See Hainmueller et. al. (2014) for details on the AMCE estimator in the presence of conditionally independent attributes. If you have no restricted profiles in your design and uniform randomization across profiles is used, then creating a design object is not necessary (the regression coefficients will give you AMCEs). However, if some profile combinations are restricted or some profiles are made to appear more often than others, it is necessary to specify them using \code{makeDesign} in order to obtain the correct AMCE estimates using \code{\link{amce}}.

You can create designs with assignment probabilities different from those of your original experiment. This allows you to estimate AMCEs for another tailored user-defined distribution of attribute-levels, such as the empirical distribution of the attributes in the population of interest. Note that the distribution of the original experiment must have a support that covers the entire support of the custom-made distribution. See Hainmueller et. al. (2014) for more details about the AMCE estimator and its relationship to the attribute-level distribution.
}
\value{
Returns an object of class "conjointDesign" containing:
\item{J }{
  A d-dimensional array where d is the number of attributes in a single profile. Each dimension has a number of indices corresponding to the number of levels for that attribute. Each entry is the probability of that profile being displayed to a survey respondent. The \code{dimnames} of the array contain the attribute and level names.
}
\item{dependence }{
  A list of character vectors denoting which attributes are marginally dependent - used in the \code{\link{amce}} estimation routine.
}
}
\references{
Strezhnev, A., Hainmueller, J., Hopkins, D., and Yamamoto, T. (2014) Conjoint Survey Design Tool. \url{https://github.com/astrezhnev/conjointsdt/}

Hainmueller, J., Hopkins, D., and Yamamoto T. (2014) Causal Inference in Conjoint Analysis: Understanding Multi-Dimensional Choices via Stated Preference Experiments. Political Analysis 22(1):1-30
}

\seealso{
\code{\link{amce}}
}
\examples{
data("immigrationconjoint")
\dontrun{
## You can load a design from a .dat file from the Conjoint SDT
immigrationdesign <- makeDesign(type="file", filename="immigrant.dat")
}

## Or you can construct the conjoint design manually in in R
attribute_list <- list()
attribute_list[["Education"]] <-c("no formal","4th grade",
                                  "8th grade","high school",
                                  "two-year college","college degree",
                                  "graduate degree")
attribute_list[["Gender"]] <- c("female","male")
attribute_list[["Country of Origin"]] <-  c("Germany","France","Mexico",
                                            "Philippines","Poland","India",
                                            "China","Sudan","Somalia","Iraq")
attribute_list[["Reason for Application"]] <- c("reunite with family",
                                                "seek better job",
                                                "escape persecution")
attribute_list[["Job"]] <- c("janitor","waiter","child care provider",
                             "gardener","financial analyst",
                             "construction worker","teacher",
                             "computer programmer","nurse",
                             "research scientist","doctor")
attribute_list[["Job Experience"]] <- c("none","1-2 years",
                                        "3-5 years","5+ years")
attribute_list[["Job Plans"]] <- c("contract with employer",
                                   "interviews with employer", "will look for work",
                                   "no plans to look for work")
attribute_list[["Prior Entry"]] <- c("never","once as tourist",
                                     "many times as tourist","six months with family",
                                     "once w/o authorization")
attribute_list[["Language Skills"]] <- c("fluent English",
                                         "broken English",
                                         "tried English but unable",
                                         "used interpreter")

# Randomization constraints in the conjoint design
constraint_list <- list()
# Constraints on Education and Job attributes
# Cannot have "doctor" with "no formal" - "high school" education
constraint_list[[1]] <- list()
constraint_list[[1]][["Education"]] <- c("no formal","4th grade",
                                          "8th grade","high school")
constraint_list[[1]][["Job"]] <- c("financial analyst",
                                  "computer programmer","research scientist",
                                  "doctor")
# Constraints on Reason for Application/Country of Origin
constraint_list[[2]] <- list()
constraint_list[[2]][["Reason for Application"]] <- c("escape persecution")
constraint_list[[2]][["Country of Origin"]] <- c("Germany","France",
                                                "Mexico","Philippines",
                                                "Poland","India")

immigrationdesign <- makeDesign(type='constraints', attribute.levels=attribute_list,
                                constraints=constraint_list)

## You can set your own user-defined distribution over the attribute-levels
marginal_weights <- list() # Uniform for all except education which is bimodal
marginal_weights[["Education"]] <- c(1/20, 1/20, 2/10, 2/10, 1/10, 2/10, 2/10)
marginal_weights[["Gender"]] <- rep(1/length(attribute_list[["Gender"]]),
                                length(attribute_list[["Gender"]]))
marginal_weights[["Country of Origin"]] <- rep(1/length(
                                      attribute_list[["Country of Origin"]]),
                                      length(attribute_list[["Country of Origin"]]))
marginal_weights[["Reason for Application"]] <- rep(1/length(
                              attribute_list[["Reason for Application"]]),
                              length(attribute_list
                              [["Reason for Application"]]))
marginal_weights[["Job"]] <- rep(1/length(attribute_list[["Job"]]),
                              length(attribute_list[["Job"]]))
marginal_weights[["Job Experience"]] <- rep(1/length(attribute_list[["Job Experience"]]),
                                      length(attribute_list[["Job Experience"]]))
marginal_weights[["Job Plans"]] <- rep(1/length(attribute_list[["Job Plans"]]),
                                      length(attribute_list[["Job Plans"]]))
marginal_weights[["Prior Entry"]] <- rep(1/length(attribute_list[["Prior Entry"]]),
                                      length(attribute_list[["Prior Entry"]]))
marginal_weights[["Language Skills"]] <- rep(1/length(attribute_list[["Language Skills"]]),
                                      length(attribute_list[["Language Skills"]]))

\dontrun{
immigrationdesign_reweight <- makeDesign(type='constraints', attribute.levels=attribute_list,
                                constraints=constraint_list, level.probs=marginal_weights)

## Note that estimated AMCEs can depend on the randomization distribution

results <- amce(Chosen_Immigrant ~  Gender + Education + Job, data=immigrationconjoint,
                cluster=TRUE, respondent.id="CaseID", design=immigrationdesign)
summary(results)

results_wt <- amce(Chosen_Immigrant ~  Gender + Education + Job, data=immigrationconjoint,
                cluster=TRUE, respondent.id="CaseID", design=immigrationdesign_reweight)
summary(results_wt)
}
}

\encoding{UTF-8}
\name{japan2014conjoint}
\alias{japan2014conjoint}
\docType{data}
\title{
Japan 2014 Conjoint Experiment Dataset from Horiuchi et. al. (2014)
}
\description{
A dataset containing the results of a conjoint survey of a representative sample of Japanese adults who were asked to choose which of two parties (with specified policy manifestos) they would support in the 2014 House of Representatives general election. Each row corresponds to a single profile presented to the respondent. See the original replication archive (Horiuchi et. al. 2014) for more detailed descriptions.
}
\usage{data("japan2014conjoint")}
\format{
  A data frame with 20,360 observations on the following 11 variables.
  \describe{
    \item{respondent}{a character vector uniquely identifying the respondent}
    \item{respondentIndex}{a numeric vector uniquely indexing the respondent}
    \item{task}{a numeric vector indexing the task presented to the respondent}
    \item{profile}{a numeric vector indexing the profile presented to the respondent}
    \item{selected}{a numeric vector indicating whether the profile was selected}
    \item{Consumption tax}{a factor indicating the profile position on `Consumption Tax`}
    \item{Consumption tax.rowpos}{a numeric vector indicating which row the attribute `Consumption Tax` appeared in the given profile}
    \item{Employment}{a factor indicating the profile position on `Employment`}
    \item{Employment.rowpos}{a numeric vector indicating which row the attribute `Employment` appeared in the given profile}
    \item{Monetary and fiscal policy}{a factor indicating the profile position on `Monetary and Fiscal Policy`}
    \item{Monetary and fiscal policy.rowpos}{a numeric vector indicating which row the attribute `Monetary and Fiscal Policy` appeared in the given profile}
    \item{Economic growth strategy}{a factor indicating the profile position on `Economic Growth Strategy`}
    \item{Economic growth strategy.rowpos}{a factor indicating which row the attribute `Economic Growth Strategy` appeared in the given profile}
    \item{Nuclear power}{a factor indicating the profile position on `Nuclear Power`}
    \item{Nuclear power.rowpos}{a factor indicating which row the attribute `Nuclear Power` appeared in the given profile}
    \item{TPP}{a factor indicating the profile position on `Trans-Pacific Partnership (TPP)`}
    \item{TPP.rowpos}{a factor indicating which row the attribute `Trans-Pacific Partnership (TPP)` appeared in the given profile}
    \item{Collective self-defense}{a factor indicating which row the attribute `Collective Self-Defense` appeared in the given profile}
    \item{Collective self-defense.rowpos}{a factor indicating which row the attribute `Collective Self-Defense` appeared in the given profile}
    \item{Constitutional revision}{a factor indicating the profile position on `Constitutional Revision`}
    \item{Constitutional revision.rowpos}{a factor indicating which row the attribute `Constitutional Revision` appeared in the given profile}
    \item{National assembly seat reduction}{a factor indicating the profile position on `National Assembly Seat Reduction`}
    \item{National assembly seat reduction.rowpos}{a factor indicating which row the attribute `National Assembly Seat Reduction` appeared in the given profile}
    \item{wgt}{post-stratification weights to map the survey sample to the census population}
  }
}
\source{
Horiuchi, Yusaku; Smith, Daniel M.; Yamamoto, Teppei, 2017, "Replication Data for: Measuring Voters' Multidimensional Policy Preferences with Conjoint Analysis: Application to Japan's 2014 Election", https://doi.org/10.7910/DVN/KUMMUJ, Harvard Dataverse, V1
}

\section{Note}{
The origional survey was conducted in Japanese. To comply with CRAN's policy, we translate the variable names into English.
}

\examples{
data("japan2014conjoint")

# Run AMCE estimator using all attributes and uniform design
results <- amce(selected ~ `Consumption tax` + `Employment` + `Monetary and fiscal policy` + 
                  `Economic growth strategy` + `Nuclear power` + `TPP` + 
                  `Collective self-defense` + `Constitutional revision` + 
                  `National assembly seat reduction`,  data=japan2014conjoint, cluster=TRUE, 
                  respondent.id="respondentIndex", weights="wgt", design="uniform")
}
\keyword{datasets}

\name{summary.amce}
\alias{summary.amce}
\alias{print.summary.amce}

\title{
Summarizing AMCE estimates
}
\description{
\code{summary} method for class "amce"
}
\usage{
\method{summary}{amce}(object, covariate.values=NULL, ...)

\method{print}{summary.amce}(x, digits=5, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{object}{
  An object of class "amce", a result of a call to \code{\link{amce}}.
  }
  \item{covariate.values}{
   An optional list containing a vector at which conditional effects will be calculated in the case of AMCE and ACIE's conditional on respondent-varying characteristics. The class of the values in the vector must match the class of the respondent-varying characteristic in question. If the "amce" object contains respondent varying characteristics, when set to NULL (default) interaction effects will be reported at quantiles in the case of a continuous variable and levels in the case of a factor. Names of list entries must correspond to variable names. If there are multiple respondent-varying characteristics then while each is varied in turn, all others will be held at first value of their entry in \code{covariate.values}. This is the bottom quantile in the case of a continuous variable and the baseline in the case of a factor variable. 
  }
  \item{x}{
  An object of class "summary.amce", a result of a call to \code{summary.amce}.
  }
  \item{digits}{
  The number of significant digits to use when printing.
  }
  \item{\dots}{
  Further arguments from other methods.
}
}
\value{
The function \code{summary.amce} computes and returns formatted data frames of effect estimates returned by \code{\link{amce}}
\item{amce }{A dataframe containing AMCE estimates and standard errors. Each row corresponds to a single attribute-level effect. }
\item{baselines_amce }{Baseline levels for each attribute relative to which the AMCEs are calculated. }
\item{acie }{A dataframe containing ACIE estimates and standard errors, if any. Each row corresponds to a single attribute-level effect. }
\item{baselines_acie }{Baseline levels for each attribute relative to which the ACIEs are calculated. }
\item{baselines_amce_resp }{Baseline levels for conditional AMCE estimates, if any, relative to which interactions are calculated. }
\item{baselines_acie_resp }{Baseline levels for conditional ACIE estimates, if any, relative to which interactions are calculated. }
\item{samplesize_estimates }{The number of valid profiles (rows) in the dataset when only effects of profile varying attributes are calculated. }
\item{samplesize_resp }{The number of valid profiles (rows) in the dataset when respondent-varying characteristics are incorporated. }
\item{numrespondents }{The number of respondents in the dataset (if a \code{respondent.id} argument was passed to \code{amce})}
\item{table_values_amce}{A dataframe giving the names of additional tables of AMCE estimates conditional on respondent-varying characteristics. A separate table is produced for each level of each respondent varying characteristic. }
\item{table_values_acie}{A dataframe giving the names of tables of ACIE estimates conditional on respondent-varying characteristics. A separate table is produced for each level of each respondent varying characteristic. }
}

\seealso{
The estimation function \code{\link{amce}}.
}

\examples{

\dontrun{

#Results with respondent-varying characteristics
results <-amce(Chosen_Immigrant ~ Gender + Education + Education:ethnocentrism +
	`Country of Origin`+ `Country of Origin`:ethnocentrism + Job +
	Job:ethnocentrism + `Job Experience` + `Job Experience`:ethnocentrism,
	data=immigrationconjoint, design=immigrationdesign,cluster=FALSE,
	respondent.varying="ethnocentrism", na.ignore=TRUE)

#Calculate conditional estimates at user-supplied levels
int.vals<-list()
int.vals[["ethnocentrism"]]<-c(60,77,88,99,45)
summary(results, covariate.values = int.vals)

}
}

\name{view}
\alias{view}
\alias{view.amce}

\title{
Graphical interfaces for conjoint analysis
}
\description{
Launches a graphical interface (in R Shiny) for a given conjoint analysis operation. This allows the user to interactively perform a conjoint analysis task without having to specify any code in R. Graphical interface is launched locally in user's default web browser. It is recommended that this feature be used for exploratory analysis only.
}
\examples{
\dontrun{
# Launch graphical interface to interactively run AMCE
view(amce) 
}
}

\name{amce}
\alias{amce}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Estimating Causal Effects in Conjoint Experiments
}
\description{
This function takes a dataset and a conjoint design and returns Average Marginal Component Effects (AMCEs) and Average Component Interaction Effects (ACIE) for the attributes specified in the formula. By default, this function assumes uniform randomization of attribute levels and no profile restrictions. If your design incorporates weighted randomization or restrictions on displayable profiles, first generate a design object using \code{makeDesign}. Interactions with respondent-level characteristics are handled by identifying relevant variables as respondent-varying.}
\usage{
amce(formula, data, design = "uniform",
	      respondent.varying = NULL, subset = NULL,
	      respondent.id = NULL, cluster = TRUE, na.ignore=FALSE,
	      weights = NULL, baselines = NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{formula}{
  A \code{formula} object specifying the name of the outcome variable on the left-hand side and the attributes for which effects are to be estimated on the right-hand side. RHS attributes should be separated by \code{+} signs. Interaction effects can be specified using standard interaction syntax - joining attribute names using either \code{:} or \code{*}.  However using the \code{:} syntax will produce the same results as \code{*} since missing base terms are automatically added to the formula. For example \code{Y ~ X1 + X2} will return AMCEs for X1 and X2. \code{Y ~ X1 + X2 + X1:X2} will return AMCEs for X1 and X2 along with an ACIE for X1/X2. \code{Y ~ X1*X2} and \code{Y ~ X1:X2} will produce identical results to \code{Y ~ X1 + X2 + X1:X2}. Note that you can place backticks around a variable name containing spaces in order to have \code{\link{formula}} interpret it as a single variable name. Any respondent characteristics must be designated as such in \code{redpondent.varying}. 
}
  \item{data}{
  A dataframe containing the outcome variable, attributes, respondent identifiers, respondent covariate data and sampling weights from a conjoint experiment.
}
  \item{design}{
  Either the character string \code{"uniform"} or a \code{conjointDesign} object created by the \code{\link{makeDesign}} function. If a \code{conjointDesign} is not passed, the function will assume all attribute levels have an equal probability of being presented to a respondent and that no profiles are restricted. Defaults to \code{"uniform"}.
}
\item{respondent.varying}{
A vector of character strings giving the names of any respondent-varying characteristics being interacted with AMCEs or ACIEs in the \code{formula}.
}
  \item{subset}{
  A logical vector with length \code{nrow(data)} denoting which rows in \code{data} should be included in estimation. This can for example be used to subset the data along respondent-level covariates. Defaults to \code{NULL}.
}
  \item{respondent.id}{
  A character string indicating the column of \code{data} containing a unique identifier for each respondent. Defaults to \code{NULL}.
}
  \item{cluster}{
  A logical indicating whether estimated standard errors should be clustered on \code{respondent.id}. Defaults to \code{TRUE}.
}
  \item{na.ignore}{
  A logical indicating whether the function should ignore missing rows in \code{data}. If \code{FALSE}, amce() will raise an error if there are rows with missing values. Defaults to \code{FALSE}.
  }
\item{weights}{
 A character string giving the name of the column in the data containing any survey weights. See documentation for \code{survey} package for more information.
}
\item{baselines}{
Manually adjust the baselines of select factor variables (either attributes or respondent varying) by supplying a list. Names of list entries should correspond with variable names. The content of each entry should be a character string giving the new baseline.
}
}

\value{
An object of class "amce" containing:
\item{attributes}{A list containing the names of attributes.}
\item{baselines}{Baseline levels for each attribute in \code{estimates}. Baselines determined using the first element of \code{levels()}. If a different baseline level is desired for an attribute, use the \code{relevel()} function on the variable prior to calling the \code{amce()} routine or supply an alternative baseline in \code{baselines} argument.}
\item{continuous}{List of quantiles for any non-factor variables, whether attributes or respondent varying.}
\item{data}{The original data.}
\item{estimates}{A list containing AMCE and ACIE estimates for each attribute in \code{formula}. Each element of \code{estimates} corresponds to a single attribute or interaction.}
\item{formula}{The \code{formula} passed to the \code{amce()} routine.}
\item{samplesize_prof}{The number of valid profiles (rows) in the dataset}
\item{user.names}{A vector with the original user supplied names for any attributes. These may differ from the attribute names in \code{estimates} if the original names contain spaces.}
\item{vcov.prof}{The modified variance-covariance matrix for AMCE and ACIE estimates. Incorporates cluster corrections as well as attribute dependencies. Profile varying attributes only.}
\item{numrespondents}{The number of respondents in the dataset (if \code{respondent.id} is not \code{NULL}). }
\item{respondent.varying}{Names of respondent-varying variables, if any.}
\item{cond.formula}{The formula used for calculating estimates conditional on respondent varying characteristics. Only returned when respondent-varying characteristics are present.}
\item{cond.estimates}{A list containing estimated effects of respondent-varying characteristics conditional on attribute values. Each element of \code{cond.estimates} corresponds to a single attribute or interaction. Only returned when respondent-varying characteristics are present. To obtain AMCE and ACIE estimates conditional on the values of the respondent-varying characteristics see \code{\link{summary.amce}}}
\item{samplesize_full}{The number of valid profiles (rows) in the dataset when respondent varying characteristics are included. Only returned when respondent-varying characteristics are present. }
\item{vcov.resp}{The modified variance-covariance matrix for effect estimates conditional on respondent-varying characteristics, where dependent relationships have been incorporated into variances and covariances. Only returned when respondent-varying characteristics are present. }
}
\references{
Hainmueller, J., Hopkins, D., and Yamamoto T. (2014) Causal Inference in Conjoint Analysis: Understanding Multi-Dimensional Choices via Stated Preference Experiments. Political Analysis 22(1):1-30
}
\seealso{
\code{\link{summary.amce}} for summaries and \code{\link{plot.amce}} for generating a coefficient plot using \code{ggplot2}.

\code{\link{makeDesign}} to create \code{conjointDesign} objects.
}
\examples{
\dontrun{
# Immigration Choice Conjoint Experiment Data from Hainmueller et. al. (2014).
data("immigrationconjoint")
data("immigrationdesign")

# Run AMCE estimator using all attributes in the design
results <- amce(Chosen_Immigrant ~  Gender + Education + `Language Skills` +
                `Country of Origin` + Job + `Job Experience` + `Job Plans` +
                `Reason for Application` + `Prior Entry`, data=immigrationconjoint,
                cluster=TRUE, respondent.id="CaseID", design=immigrationdesign)
# Print summary
summary(results)


# Run AMCE estimator using all attributes in the design with interactions
interaction_results <- amce(Chosen_Immigrant ~  Gender + Education + `Language Skills` +
                `Country of Origin` + Job + `Job Experience` + `Job Plans` +
                `Reason for Application` + `Prior Entry` + Education:`Language Skills` +
		Job: `Job Experience` + `Job Plans`:`Reason for Application`,
		data=immigrationconjoint, cluster=TRUE, respondent.id="CaseID",
		design=immigrationdesign)
# Print summary
summary(interaction_results)

# create weights in data
weights <- runif(nrow(immigrationconjoint))
immigrationconjoint$weights <- weights
# Run AMCE estimator using weights
results <- amce(Chosen_Immigrant ~  Gender + Education + `Language Skills` +
                `Country of Origin` + Job + `Job Experience` + `Job Plans` +
                `Reason for Application` + `Prior Entry`, data=immigrationconjoint,
                cluster=TRUE, respondent.id="CaseID", design=immigrationdesign,
		weights = "weights")
# Print summary
summary(results)

# Include a respondent-varying interaction
results <- amce(Chosen_Immigrant ~ Gender + Education + Job +
	   	ethnocentrism:Job + Education:Job,
		data=immigrationconjoint, na.ignore = TRUE,
		cluster=FALSE,design=immigrationdesign,
		respondent.varying = "ethnocentrism")
# Print summary
summary(results)

# Change the baseline for "Education"
baselines <- list()
baselines$Education <- "graduate degree"

results <- amce(Chosen_Immigrant ~ Gender + Education + Job +
		 Education:Job, data=immigrationconjoint,
		 cluster=FALSE,design=immigrationdesign,
		 baselines=baselines)
# Print summary
summary(results)
}

}

\name{cjoint-package}
\alias{cjoint-package}
\alias{cjoint}
\docType{package}
\title{
cjoint: A Package for Estimating Average Marginal Component-specific Effects for Conjoint Survey Experiments
}
\description{
This package allows researchers to estimate the causal effects of attributes in conjoint survey experiments. It implements the Average Marginal Component-specific Effects (AMCE) estimator presented in Hainmueller, J., Hopkins, D., and Yamamoto T. (2014) Causal Inference in Conjoint Analysis: Understanding Multi-Dimensional Choices via Stated Preference Experiments. Political Analysis 22(1):1-30
}
\details{
\tabular{ll}{
Package: \tab cjoint\cr
Type: \tab Package\cr
Version: \tab 2.1.2\cr
Date: \tab 2025-03-20\cr
License: \tab GPL (>= 2)\cr
}

}
\author{
Authors: Soubhik Barari, Elissa Berwick, Jens Hainmueller, Daniel Hopkins, Sean Liu, Anton Strezhnev, Teppei Yamamoto

Maintainer: Anton Strezhnev <astrezhnev@uchicago.edu>

}
\references{
Hainmueller, J., Hopkins, D., and Yamamoto T. (2014) Causal Inference in Conjoint Analysis: Understanding Multi-Dimensional Choices via Stated Preference Experiments. Political Analysis 22(1):1-30
}

\keyword{ package }

\name{immigrationconjoint}
\alias{immigrationconjoint}
\docType{data}
\title{
Immigration Conjoint Experiment Dataset from Hainmueller et. al. (2014)
}
\description{
A dataset containing the results of a conjoint survey of a representative sample of American adults who were asked to choose which hypothetical immigrants they think should be admitted into the United States. Each row corresponds to a single profile presented to the respondent.
}
\usage{data("immigrationconjoint")}
\format{
  A data frame with 13,960 observations on the following 16 variables.
  \describe{
    \item{\code{CaseID}}{a numeric vector indicating the respondent to which the particular profile corresponds}
    \item{\code{contest_no}}{a numeric vector indicating the number of the task to which the profile corresponds}
    \item{\code{Education}}{a factor with levels \code{no formal}, \code{4th grade}, \code{8th grade}, \code{high school}, \code{two-year college}, \code{college degree}, \code{graduate degree}}
    \item{\code{Gender}}{a factor with levels \code{female}, \code{male}}
    \item{\samp{Country of Origin}}{a factor with levels \code{India}, \code{Germany}, \code{France}, \code{Mexico}, \code{Philippines}, \code{Poland}, \code{China}, \code{Sudan}, \code{Somalia}, \code{Iraq}}
    \item{\samp{Reason for Application}}{a factor with levels \code{reunite with family}, \code{seek better job}, \code{escape persecution}}
    \item{\code{Job}}{a factor with levels \code{janitor}, \code{waiter}, \code{child care provider}, \code{gardener}, \code{financial analyst}, \code{construction worker}, \code{teacher}, \code{computer programmer}, \code{nurse}, \code{research scientist}, \code{doctor}}
    \item{\samp{Job Experience}}{a factor with levels \code{none}, \code{1-2 years}, \code{3-5 years}, \code{5+ years}}
    \item{\samp{Job Plans}}{a factor with levels \code{will look for work}, \code{contract with employer}, \code{interviews with employer}, \code{no plans to look for work}}
    \item{\samp{Prior Entry}}{a factor with levels \code{never}, \code{once as tourist}, \code{many times as tourist}, \code{six months with family}, \code{once w/o authorization}}
    \item{\samp{Language Skills}}{a factor with levels \code{fluent English}, \code{broken English}, \code{tried English but unable}, \code{used interpreter}}
    \item{\code{Chosen_Immigrant}}{a numeric vector denoting whether the immigrant profile was selected}
    \item{\code{ethnocentrism}}{a numeric vector}
    \item{\code{profile}}{a numeric vector giving the profile number}
    \item{\code{LangPos}}{a numeric vector}
    \item{\code{PriorPos}}{a numeric vector}
  }
}

\source{
Hainmueller, J., Hopkins, D., and Yamamoto T. (2014) Causal Inference in Conjoint Analysis: Understanding Multi-Dimensional Choices via Stated Preference Experiments. Political Analysis 22(1):1-30
}

\examples{
\dontrun{
data("immigrationconjoint")
data("immigrationdesign")

# Run AMCE estimator using all attributes in the design
results <- amce(Chosen_Immigrant ~  Gender + Education + `Language Skills` + 
                `Country of Origin` + Job + `Job Experience` + `Job Plans` + 
                `Reason for Application` + `Prior Entry`, data=immigrationconjoint, 
                cluster=TRUE, respondent.id="CaseID", design=immigrationdesign)
}
}
\keyword{datasets}
